@@ -13,11 +13,11 @@ from ocgis import env
 from ocgis.base import get_variables, get_dimension_names, AbstractOcgisObject
 from ocgis.constants import TagName, DimensionMapKey, HeaderName
 from ocgis.exc import SampleSizeNotImplemented, DefinitionValidationError, UnitsValidationError
-from ocgis.util.conformer import conform_array_by_dimension_names
+from ocgis.util.broadcaster import broadcast_array_by_dimension_names
 from ocgis.util.helpers import get_default_or_apply, get_iter
 from ocgis.util.logging_ocgis import ocgis_lh
 from ocgis.util.units import get_are_units_equal_by_string_or_cfunits
-from ocgis.variable.base import Variable, VariableCollection
+from ocgis.variable.base import Variable, VariableCollection, get_default_fill_value_from_dtype
 
 # Standard dimension order for data arrays.
 STANDARD_DIMENSIONS = (DimensionMapKey.REALIZATION, DimensionMapKey.TIME, DimensionMapKey.LEVEL,
@@ -211,6 +211,11 @@ class AbstractFunction(AbstractOcgisObject):
         else:
             fill_value = self.fill_value
 
+        # Provide a default fill value for file only operations. This will guarantee variable value arrays are
+        # initialized with a default fill value.
+        if file_only and fill_value is None:
+            fill_value = get_default_fill_value_from_dtype(dtype)
+
         attrs = OrderedDict()
         attrs['standard_name'] = self.standard_name
         attrs['long_name'] = self.long_name
@@ -541,12 +546,12 @@ class AbstractFunction(AbstractOcgisObject):
 
             # Swap axes for the calculation values, the fill array for the calculation result, and (potentially) the
             # sample size.
-            carr, carr_fill = [conform_array_by_dimension_names(t, src_names_extra_removed, STANDARD_DIMENSIONS)
+            carr, carr_fill = [broadcast_array_by_dimension_names(t, src_names_extra_removed, STANDARD_DIMENSIONS)
                                for t in [extras_removed, extras_removed_fill]]
             if calc_sample_size:
-                carr_fill_sample_size = conform_array_by_dimension_names(extras_removed_fill_sample_size,
-                                                                         src_names_extra_removed,
-                                                                         STANDARD_DIMENSIONS)
+                carr_fill_sample_size = broadcast_array_by_dimension_names(extras_removed_fill_sample_size,
+                                                                           src_names_extra_removed,
+                                                                           STANDARD_DIMENSIONS)
 
             if not calc_sample_size:
                 yld = (carr, carr_fill)
@@ -583,72 +588,6 @@ class AbstractFieldFunction(AbstractFunction):
         return self.calculate(**self.parms)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class AbstractUnivariateFunction(AbstractFunction):
-    """
-    Base class for functions accepting a single univariate input.
-    """
-
-    # Some calculations use a temporal group dimension but do not want any temporal aggregation to occur after the
-    # calculation.
-    should_temporally_aggregate = True
-    #: Optional sequence of acceptable string units definitions for input variables. If this is set to ``None``, no unit
-    #: validation will occur.
-    required_units = None
-
-    def __init__(self, *args, **kwargs):
-        super(AbstractUnivariateFunction, self).__init__(*args, **kwargs)
-
-        if self.calc_sample_size and self.tgd is None:
-            msg = 'Sample sizes not relevant for scalar transforms with no temporal grouping. Setting to False.'
-            ocgis_lh(msg=msg, logger='calc.base', level=logging.WARN)
-            self.calc_sample_size = False
-
-    def validate_units(self, variable):
-        if self.required_units is not None:
-            matches = [get_are_units_equal_by_string_or_cfunits(variable.units, target, try_cfunits=env.USE_CFUNITS) \
-                       for target in self.required_units]
-            if not any(matches):
-                raise UnitsValidationError(variable, self.required_units, self.key)
-
-    def _execute_(self):
-        for variable, calculation_name in self.iter_calculation_targets():
-            crosswalk = self._get_dimension_crosswalk_(variable)
-
-            fill_dimensions = variable.dimensions
-            # Some calculations introduce a temporal group dimension during initialization and perform the temporal
-            # aggregation implicit to the function.
-            if self.tgd is not None and not self.should_temporally_aggregate:
-                time_dimension_name = self.field.time.dimensions[0].name
-                dimension_name = get_dimension_names(fill_dimensions)
-                time_index = dimension_name.index(time_dimension_name)
-                fill_dimensions = list(variable.dimensions)
-                fill_dimensions[time_index] = self.tgd.dimensions[0]
-
-            fill = self.get_fill_variable(variable, calculation_name, fill_dimensions, self.file_only)
-
-            if not self.file_only:
-                arr = self.get_variable_value(variable)
-                arr_fill = self.get_variable_value(fill)
-
-                for yld in self._iter_conformed_arrays_(crosswalk, variable.shape, arr, arr_fill, None):
-                    carr, carr_fill = yld
-                    res_calculation = self.calculate(carr, **self.parms)
-                    carr_fill.data[:] = res_calculation.data
-                    carr_fill.mask[:] = res_calculation.mask
-
-                # Setting the values ensures the mask is updated on the output variables.
-                fill.set_value(arr_fill)
-
-            if self.tgd is not None and self.should_temporally_aggregate:
-                fill = self._get_temporal_agg_fill_(fill, calculation_name, self.file_only, f=self.aggregate_temporal,
-                                                    parms={})
-            else:
-                fill = {'fill': fill}
-
-            self._add_to_collection_(fill)
-
-
 @six.add_metaclass(abc.ABCMeta)
 class AbstractParameterizedFunction(AbstractFunction):
     """
@@ -734,6 +673,72 @@ class AbstractParameterizedFunction(AbstractFunction):
         return ret
 
 
+@six.add_metaclass(abc.ABCMeta)
+class AbstractUnivariateFunction(AbstractFunction):
+    """
+    Base class for functions accepting a single univariate input.
+    """
+
+    # Some calculations use a temporal group dimension but do not want any temporal aggregation to occur after the
+    # calculation.
+    should_temporally_aggregate = True
+    #: Optional sequence of acceptable string units definitions for input variables. If this is set to ``None``, no unit
+    #: validation will occur.
+    required_units = None
+
+    def __init__(self, *args, **kwargs):
+        super(AbstractUnivariateFunction, self).__init__(*args, **kwargs)
+
+        if self.calc_sample_size and self.tgd is None:
+            msg = 'Sample sizes not relevant for scalar transforms with no temporal grouping. Setting to False.'
+            ocgis_lh(msg=msg, logger='calc.base', level=logging.WARN)
+            self.calc_sample_size = False
+
+    def validate_units(self, variable):
+        if self.required_units is not None:
+            matches = [get_are_units_equal_by_string_or_cfunits(variable.units, target, try_cfunits=env.USE_CFUNITS) \
+                       for target in self.required_units]
+            if not any(matches):
+                raise UnitsValidationError(variable, self.required_units, self.key)
+
+    def _execute_(self):
+        for variable, calculation_name in self.iter_calculation_targets():
+            crosswalk = self._get_dimension_crosswalk_(variable)
+
+            fill_dimensions = variable.dimensions
+            # Some calculations introduce a temporal group dimension during initialization and perform the temporal
+            # aggregation implicit to the function.
+            if self.tgd is not None and not self.should_temporally_aggregate:
+                time_dimension_name = self.field.time.dimensions[0].name
+                dimension_name = get_dimension_names(fill_dimensions)
+                time_index = dimension_name.index(time_dimension_name)
+                fill_dimensions = list(variable.dimensions)
+                fill_dimensions[time_index] = self.tgd.dimensions[0]
+
+            fill = self.get_fill_variable(variable, calculation_name, fill_dimensions, self.file_only)
+
+            if not self.file_only:
+                arr = self.get_variable_value(variable)
+                arr_fill = self.get_variable_value(fill)
+
+                for yld in self._iter_conformed_arrays_(crosswalk, variable.shape, arr, arr_fill, None):
+                    carr, carr_fill = yld
+                    res_calculation = self.calculate(carr, **self.parms)
+                    carr_fill.data[:] = res_calculation.data
+                    carr_fill.mask[:] = res_calculation.mask
+
+                # Setting the values ensures the mask is updated on the output variables.
+                fill.set_value(arr_fill)
+
+            if self.tgd is not None and self.should_temporally_aggregate:
+                fill = self._get_temporal_agg_fill_(fill, calculation_name, self.file_only, f=self.aggregate_temporal,
+                                                    parms={})
+            else:
+                fill = {'fill': fill}
+
+            self._add_to_collection_(fill)
+
+
 @six.add_metaclass(abc.ABCMeta)
 class AbstractUnivariateSetFunction(AbstractUnivariateFunction):
     """
@@ -766,7 +771,9 @@ class AbstractUnivariateSetFunction(AbstractUnivariateFunction):
 @six.add_metaclass(abc.ABCMeta)
 class AbstractMultivariateFunction(AbstractFunction):
     """
-    Base class for functions operating on multivariate inputs.
+    Base class for functions operating on multivariate inputs. Multivariate functions also double as set functions
+    (i.e. they can temporally group). This can be turned off at the class level by setting ``time_aggregation_external``
+    to ``False``.
     """
     # : Optional dictionary mapping unit definitions for required variables.
     #: For example: required_units = {'tas':'fahrenheit','rhs':'percent'}
