@@ -1,24 +1,26 @@
 import abc
 import json
+from abc import ABCMeta
 from contextlib import contextmanager
 from copy import deepcopy
 from warnings import warn
 
 import six
 
-from ocgis import constants
+from ocgis import constants, GridUnstruct
 from ocgis import vm
 from ocgis.base import AbstractOcgisObject, raise_if_empty
 from ocgis.base import get_variable_names
 from ocgis.collection.field import Field
-from ocgis.constants import MPIWriteMode, TagName, KeywordArgument
+from ocgis.constants import MPIWriteMode, TagName, KeywordArgument, OcgisConvention, VariableName
 from ocgis.driver.dimension_map import DimensionMap
-from ocgis.exc import DefinitionValidationError, NoDataVariablesFound, DimensionMapError
+from ocgis.exc import DefinitionValidationError, NoDataVariablesFound, DimensionMapError, VariableMissingMetadataError, \
+    GridDeficientError
 from ocgis.util.helpers import get_group
 from ocgis.util.logging_ocgis import ocgis_lh
-from ocgis.variable.base import SourcedVariable, VariableCollection
+from ocgis.variable.base import SourcedVariable
 from ocgis.variable.dimension import Dimension
-from ocgis.vmachine.mpi import find_dimension_in_sequence, OcgDist
+from ocgis.vmachine.mpi import OcgDist
 
 
 # TODO: Make the driver accept no arguments at initialization. The request dataset should be passed around as a parameter.
@@ -32,9 +34,12 @@ class AbstractDriver(AbstractOcgisObject):
     :type rd: :class:`~ocgis.RequestDataset`
     """
 
+    common_extension = None  # The standard file extension commonly associated with the canonical file format.
+    default_axes_positions = (0, 1)  # Standard axes index for Y and X respectively.
     _default_crs = None
+    _esmf_fileformat = None  # The associated ESMF file type. This may be None.
+    _esmf_grid_class = constants.ESMFGridClass.GRID  # The ESMF grid class type.
     _priority = False
-    common_extension = None
 
     def __init__(self, rd):
         self.rd = rd
@@ -48,33 +53,33 @@ class AbstractDriver(AbstractOcgisObject):
     def __str__(self):
         return '"{0}"'.format(self.key)
 
-    # tdk: remove me
     @property
     def crs(self):
+        """Blocks setting the CRS attribute on a driver."""
+
         raise NotImplementedError
 
+    @property
+    def data_model(self):
+        """
+        Return the data model definition for the driver if it has one. NetCDF has NETCDF3_CLASSIC, etc.
+
+        :rtype: str | ``None``
+        """
+        return None
+
     @property
     def dimension_map_raw(self):
         if self._dimension_map_raw is None:
-            self._dimension_map_raw = get_dimension_map_raw(self, self.metadata_raw)
+            self._dimension_map_raw = create_dimension_map_raw(self, self.metadata_raw)
         return self._dimension_map_raw
 
     @property
     def dist(self):
         if self._dist is None:
-            self._dist = self.get_dist()
+            self._dist = self.create_dist()
         return self._dist
 
-    @property
-    def metadata_raw(self):
-        if self._metadata_raw is None:
-            self._metadata_raw = self.get_metadata()
-        return self._metadata_raw
-
-    @property
-    def metadata_source(self):
-        return self.rd.metadata
-
     @abc.abstractproperty
     def extensions(self):
         """
@@ -88,6 +93,21 @@ class AbstractDriver(AbstractOcgisObject):
     def key(self):
         """:rtype: str"""
 
+    @property
+    def metadata_raw(self):
+        if self._metadata_raw is None:
+            # If there is no URI, we must be working with metadata passed to the request dataset.
+            if self.rd._uri is None and self.rd.opened is None:
+                res = self.rd.metadata
+            else:
+                res = self.get_metadata()
+            self._metadata_raw = res
+        return self._metadata_raw
+
+    @property
+    def metadata_source(self):
+        return self.rd.metadata
+
     @abc.abstractproperty
     def output_formats(self):
         """
@@ -96,6 +116,19 @@ class AbstractDriver(AbstractOcgisObject):
         :rtype: list[str, ...]
         """
 
+    @staticmethod
+    def array_resolution(value, axis):
+        """
+        Optionally overloaded by subclasses to calculate the "resolution" of an array. This makes the most sense in the
+        context of coordinate variables. This method should return a float resolution.
+
+        :param value: The value array target.
+        :type value: :class:`numpy.ndarray`
+        :param int axis: The target axis for the resolution calculation.
+        :rtype: float
+        """
+        raise NotImplementedError
+
     @classmethod
     def close(cls, obj, rd=None):
         # If the request dataset has an opened file object, do not close the file as we expect the client to handle
@@ -115,102 +148,112 @@ class AbstractDriver(AbstractOcgisObject):
             ret = crs
         return ret
 
-    def get_dimension_map(self, group_metadata):
-        """:rtype: :class:`ocgis.DimensionMap`"""
-        return DimensionMap()
+    def create_dimension_map(self, group_metadata, **kwargs):
+        """
+        Create a dimension map for a group from its metadata.
+
+        :param dict group_metadata: Group metadata to use when creating the dimension map.
+        :rtype: :class:`~ocgis.DimensionMap`
+        """
+        ret = DimensionMap()
+        ret.set_driver(self.key)
+        return ret
 
     @staticmethod
-    def get_data_variable_names(group_metadata, group_dimension_map):
+    def create_dimensions(group_metadata):
         """
-        Return a tuple of data variable names using the metadata and dimension map.
-        """
-        return tuple(group_metadata['variables'].keys())
+        Create dimension objects. The key may differ from the dimension name. In which case, we can assume the dimension
+        is being renamed.
 
-    def get_dist(self):
-        """
-        :rtype: :class:`ocgis.OcgVM`
+        :param dict group_metadata: Metadata dictionary for the target group.
+        :rtype: list
         """
+        gmd = group_metadata.get('dimensions', {})
+        dims = {}
+        for k, v in gmd.items():
+            size = v['size']
+            if v.get('isunlimited', False):
+                size_current = size
+                size = None
+            else:
+                size_current = None
+            dims[k] = Dimension(v.get('name', k), size=size, size_current=size_current, src_idx='auto', source_name=k)
+        return dims
 
+    def create_dist(self, metadata=None):
+        """
+        Create a distribution from global metadata. In general, this should not be overloaded by subclasses.
+
+        :param dict metadata: Global metadata to use for creating a distribution.
+
+        :rtype: :class:`ocgis.OcgDist`
+        """
         ompi = OcgDist(size=vm.size, ranks=vm.ranks)
-        # ompi = OcgDist()
 
         # Convert metadata into a grouping consistent with the MPI dimensions.
-        metadata = {None: self.metadata_source}
+        if metadata is None:
+            metadata = self.metadata_source
+        metadata = {None: metadata}
         for group_index in iter_all_group_keys(metadata):
             group_meta = get_group(metadata, group_index)
 
             # Add the dimensions to the distribution object.
-            dimensions = self._get_dimensions_main_(group_meta)
-            for dimension_name, dimension_meta in list(group_meta['dimensions'].items()):
-                target_dimension = find_dimension_in_sequence(dimension_name, dimensions)
-                target_dimension.dist = group_meta['dimensions'][dimension_name].get('dist', False)
-                ompi.add_dimension(target_dimension, group=group_index)
+            dimensions = self.create_dimensions(group_meta)
 
-            try:
-                dimension_map = self.rd.dimension_map.get_group(group_index)
-            except DimensionMapError:
-                # Likely a user-provided dimension map.
-                continue
+            # Only build a distribution if the group has more than one dimension.
+            if len(dimensions) == 0:
+                _ = ompi.get_group(group=group_index)
+            else:
+                for dimension_name, dimension_meta in list(group_meta['dimensions'].items()):
+                    target_dimension = dimensions[dimension_name]
+                    target_dimension.dist = group_meta['dimensions'][dimension_name].get('dist', False)
+                    ompi.add_dimension(target_dimension, group=group_index)
+                try:
+                    dimension_map = self.rd.dimension_map.get_group(group_index)
+                except DimensionMapError:
+                    # Likely a user-provided dimension map.
+                    continue
 
-            # dimension_map = get_group(self.rd.dimension_map, group_index, has_root=False)
-            distributed_dimension_name = self.get_distributed_dimension_name(dimension_map,
-                                                                             group_meta['dimensions'])
-            # Allow no distributed dimensions to be returned.
-            if distributed_dimension_name is not None:
-                for target_rank in range(ompi.size):
-                    distributed_dimension = ompi.get_dimension(distributed_dimension_name, group=group_index,
-                                                               rank=target_rank)
-                    distributed_dimension.dist = True
+                # dimension_map = get_group(self.rd.dimension_map, group_index, has_root=False)
+                distributed_dimension_name = self.get_distributed_dimension_name(dimension_map,
+                                                                                 group_meta['dimensions'])
+                # Allow no distributed dimensions to be returned.
+                if distributed_dimension_name is not None:
+                    for target_rank in range(ompi.size):
+                        distributed_dimension = ompi.get_dimension(distributed_dimension_name, group=group_index,
+                                                                   rank=target_rank)
+                        distributed_dimension.dist = True
 
-            # Add the variables to the distribution object.
-            for variable_name, variable_meta in list(group_meta['variables'].items()):
-                ompi.add_variable(variable_name, dimensions=variable_meta['dimensions'], group=group_index)
-
-        # tdk: this will have to be moved to account for slicing
         ompi.update_dimension_bounds()
         return ompi
 
-    def get_distributed_dimension_name(self, dimension_map, dimensions_metadata):
-        """Return the preferred distributed dimension name."""
-        return None
+    def create_field(self, *args, **kwargs):
+        """
+        Create a field object. In general, this should not be overloaded by subclasses.
 
-    def get_dump_report(self, indent=0, group_metadata=None, first=True, global_attributes_name='global'):
-        lines = []
-        if first:
-            lines.append('OCGIS Driver Key: ' + self.key + ' {')
-            group_metadata = group_metadata or self.metadata_source
-        else:
-            indent += 2
-        lines += get_dump_report_for_group(group_metadata, global_attributes_name=global_attributes_name, indent=indent)
-        for group_name, group_metadata in list(group_metadata.get('groups', {}).items()):
-            lines.append('')
-            lines.append(' ' * indent + 'group: ' + group_name + ' {')
-            dump_lines = self.get_dump_report(group_metadata=group_metadata, first=False, indent=indent,
-                                              global_attributes_name=group_name)
-            lines += dump_lines
-            lines.append(' ' * indent + '  }' + ' // group: {}'.format(group_name))
-        if first:
-            lines.append('}')
-        return lines
-
-    def get_field(self, *args, **kwargs):
-        vc = kwargs.pop('vc', None)
+        :keyword bool format_time: ``(=True)`` If ``False``, do not convert numeric times to Python date objects.
+        :keyword str grid_abstraction: ``(='auto')`` If provided, use this grid abstraction.
+        :keyword raw_field: ``(=None)`` If provided, modify this field instead.
+        :type raw_field: None | :class:`~ocgis.Field`
+        :param kwargs: Additional keyword arguments to :meth:`~ocgis.driver.base.AbstractDriver.create_raw_field`.
+        :return: :class:`ocgis.Field`
+        """
+        kwargs = kwargs.copy()
+        raw_field = kwargs.pop('raw_field', None)
         format_time = kwargs.pop(KeywordArgument.FORMAT_TIME, True)
-        if KeywordArgument.GRID_ABSTRACTION in kwargs:
-            grid_abstraction = kwargs.pop(KeywordArgument.GRID_ABSTRACTION)
-        else:
-            grid_abstraction = constants.UNINITIALIZED
+        grid_abstraction = kwargs.pop(KeywordArgument.GRID_ABSTRACTION, self.rd.grid_abstraction)
+        grid_is_isomorphic = kwargs.pop('grid_is_isomorphic', self.rd.grid_is_isomorphic)
 
-        if vc is None:
+        if raw_field is None:
             # Get the raw variable collection from source.
             new_kwargs = kwargs.copy()
             new_kwargs['source_name'] = None
-            vc = self.get_variable_collection(*args, **new_kwargs)
+            raw_field = self.create_raw_field(*args, **new_kwargs)
 
         # Get the appropriate metadata for the collection.
-        group_metadata = self.get_group_metadata(vc.group, self.metadata_source)
+        group_metadata = self.get_group_metadata(raw_field.group, self.metadata_source)
         # Always pull the dimension map from the request dataset. This allows it to be overloaded.
-        dimension_map = self.get_group_metadata(vc.group, self.rd.dimension_map)
+        dimension_map = self.get_group_metadata(raw_field.group, self.rd.dimension_map)
 
         # Modify the coordinate system variable. If it is overloaded on the request dataset, then the variable
         # collection needs to be updated to hold the variable and any alternative coordinate systems needs to be
@@ -230,9 +273,9 @@ class AbstractDriver(AbstractOcgisObject):
             elif crs is not None:
                 to_add = crs
         if to_remove is not None:
-            vc.pop(to_remove, None)
+            raw_field.pop(to_remove, None)
         if to_add is not None:
-            vc.add_variable(to_add, force=True)
+            raw_field.add_variable(to_add, force=True)
         # Overload the dimension map with the CRS.
         if to_add is not None:
             # dimension_map[DimensionMapKey.CRS][DimensionMapKey.VARIABLE] = to_add.name
@@ -240,14 +283,17 @@ class AbstractDriver(AbstractOcgisObject):
 
         # Remove the mask variable if present in the raw dimension map and the source dimension map is set to None.
         if self.rd.dimension_map.get_spatial_mask() is None and self.dimension_map_raw.get_spatial_mask() is not None:
-            vc.pop(self.dimension_map_raw.get_spatial_mask())
+            raw_field.pop(self.dimension_map_raw.get_spatial_mask())
 
         # Convert the raw variable collection to a field.
-        kwargs['dimension_map'] = dimension_map
+        # TODO: Identify a way to remove this code block; field should be appropriately initialized; format_time and grid_abstraction are part of a dimension map.
+        kwargs[KeywordArgument.DIMENSION_MAP] = dimension_map
         kwargs[KeywordArgument.FORMAT_TIME] = format_time
-        if grid_abstraction != constants.UNINITIALIZED:
+        if grid_abstraction != 'auto':
             kwargs[KeywordArgument.GRID_ABSTRACTION] = grid_abstraction
-        field = Field.from_variable_collection(vc, *args, **kwargs)
+        if grid_is_isomorphic != 'auto':
+            kwargs['grid_is_isomorphic'] = grid_is_isomorphic
+        field = Field.from_variable_collection(raw_field, *args, **kwargs)
 
         # If this is a source grid for regridding, ensure the flag is updated.
         field.regrid_source = self.rd.regrid_source
@@ -277,11 +323,115 @@ class AbstractDriver(AbstractOcgisObject):
 
         # Load child fields.
         for child in list(field.children.values()):
-            kwargs['vc'] = child
-            field.children[child.name] = self.get_field(*args, **kwargs)
+            kwargs['raw_field'] = child
+            field.children[child.name] = self.create_field(*args, **kwargs)
 
         return field
 
+    def create_raw_field(self, group_metadata=None, group_name=None, name=None, source_name=constants.UNINITIALIZED,
+                         parent=None, uid=None):
+        """
+        Create a raw field object. This field object should interpret metadata explicitly (i.e. no dimension map). In
+        general this method should not be overloaded by subclasses.
+
+        :param dict group_metadata: Metadata dictionary for the current group.
+        :param str group_name: The name of the current group being processed.
+        :param str name: See :class:`~ocgis.base.AbstractNamedObject`
+        :param str source_name: See :class:`~ocgis.base.AbstractNamedObject`
+        :param parent: :class:`~ocgis.variable.base.AbstractContainer`
+        :param int uid: See :class:`~ocgis.variable.base.AbstractContainer`
+        :return: :class:`~ocgis.Field`
+        """
+        if group_metadata is None:
+            group_metadata = self.rd.metadata
+
+        field = Field(name=name, source_name=source_name, uid=uid, attrs=group_metadata.get('global_attributes'))
+        for var in self.create_variables(group_metadata, parent=field).values():
+            field.add_variable(var, force=True)
+        if parent is not None:
+            parent.add_child(field)
+        for k, v in group_metadata.get('groups', {}).items():
+            _ = self.create_raw_field(v, name=k, parent=field, group_name=k)
+        return field
+
+    def create_variables(self, group_metadata, parent=None):
+        """
+        Create a dictionary of variable objects. The keys are the variable names.
+
+        :param dict group_metadata: Metadata for the group to create variables from.
+        :param parent: See :class:`~ocgis.variable.base.AbstractContainer`
+        :return: dict
+        """
+        vmeta = group_metadata['variables']
+        vars = {}
+        for k, v in vmeta.items():
+            # Dimensions are set in this sourced variable initialization call. This is to allow the group hierarchy to
+            # be determined by the parents' relationships.
+            nvar = SourcedVariable(name=v.get('name', k), dtype=v.get('dtype'), request_dataset=self.rd, source_name=k,
+                                   parent=parent)
+            vars[k] = nvar
+        return vars
+
+    @staticmethod
+    def get_data_variable_names(group_metadata, group_dimension_map):
+        """
+        Return a tuple of data variable names using the metadata and dimension map.
+        """
+        return tuple(group_metadata['variables'].keys())
+
+    def get_distributed_dimension_name(self, dimension_map, dimensions_metadata):
+        """Return the preferred distributed dimension name."""
+        return None
+
+    def get_dump_report(self, indent=0, group_metadata=None, first=True, global_attributes_name='global'):
+        lines = []
+        if first:
+            lines.append('OCGIS Driver Key: ' + self.key + ' {')
+            group_metadata = group_metadata or self.metadata_source
+        else:
+            indent += 2
+        lines += get_dump_report_for_group(group_metadata, global_attributes_name=global_attributes_name, indent=indent)
+        for group_name, group_metadata in list(group_metadata.get('groups', {}).items()):
+            lines.append('')
+            lines.append(' ' * indent + 'group: ' + group_name + ' {')
+            dump_lines = self.get_dump_report(group_metadata=group_metadata, first=False, indent=indent,
+                                              global_attributes_name=group_name)
+            lines += dump_lines
+            lines.append(' ' * indent + '  }' + ' // group: {}'.format(group_name))
+        if first:
+            lines.append('}')
+        return lines
+
+    @classmethod
+    def get_esmf_fileformat(cls):
+        """
+        Get the ESMF file format associated with the driver. The string should be an accessible attribute on :attr:`ESMF.constants.FileFormat`.
+
+        :rtype: str
+        """
+        import ESMF
+        return getattr(ESMF.constants.FileFormat, cls._esmf_fileformat)
+
+    @classmethod
+    def get_esmf_grid_class(cls):
+        """
+        Get the ESMF grid class.
+
+        :rtype: :class:`ESMF.Grid` | :class:`ESMF.Mesh`
+        """
+        return constants.ESMFGridClass.get_esmf_class(cls._esmf_grid_class)
+
+    @staticmethod
+    def get_grid(field):
+        """
+        Construct a grid object and return it. If a grid object may not be constructed, return ``None``.
+
+        :param field: The field object to use for constructing the grid.
+        :type field: :class:`~ocgis.Field`
+        :rtype: :class:`ocgis.spatial.grid.AbstractGrid` | ``None``
+        """
+        raise NotImplementedError
+
     @staticmethod
     def get_group_metadata(group_index, metadata, has_root=False):
         return get_group(metadata, group_index, has_root=has_root)
@@ -291,10 +441,22 @@ class AbstractDriver(AbstractOcgisObject):
         :rtype: dict
         """
 
-        return self._get_metadata_main_()
+        metadata_subclass = self._get_metadata_main_()
+
+        # Use the predicate (filter) if present on the request dataset.
+        # TODO: Should handle groups?
+        pred = self.rd.predicate
+        if pred is not None:
+            to_pop = []
+            for var_name in metadata_subclass['variables'].keys():
+                if not pred(var_name):
+                    to_pop.append(var_name)
+            for var_name in to_pop:
+                metadata_subclass['variables'].pop(var_name)
+
+        return metadata_subclass
 
     def get_source_metadata_as_json(self):
-        # tdk: test
 
         def _jsonformat_(d):
             for k, v in d.items():
@@ -312,20 +474,45 @@ class AbstractDriver(AbstractOcgisObject):
         _jsonformat_(meta)
         return json.dumps(meta)
 
-    def get_variable_collection(self, **kwargs):
-        """
-        :rtype: :class:`ocgis.new_interface.variable.VariableCollection`
+    @staticmethod
+    def get_or_create_spatial_mask(*args, **kwargs):
         """
+        Get or create the spatial mask variable and return its mask value as a boolean array.
 
-        if KeywordArgument.DRIVER not in kwargs:
-            kwargs[KeywordArgument.DRIVER] = self
-        dimension = list(self.dist.get_group(rank=vm.rank)['dimensions'].values())[0]
-        ret = VariableCollection(**kwargs)
-        for v in list(self.metadata_source['variables'].values()):
-            nvar = SourcedVariable(name=v['name'], dimensions=dimension, dtype=v['dtype'], request_dataset=self.rd)
-            ret.add_variable(nvar)
+        :param tuple args: See table
+
+        ===== ======================================================= ===================================
+        Index Type                                                    Description
+        ===== ======================================================= ===================================
+        0     :class:`ocgis.spatial.base.AbstractXYZSpatialContainer` Target XYZ spatial container
+        1:    <varying>                                               See :meth:`ocgis.Variable.get_mask`
+        ===== ======================================================= ===================================
+
+        :param kwargs: See keyword arguments to :meth:`ocgis.Variable.get_mask`
+        :rtype: :class:`numpy.ndarray`
+        """
+        from ocgis.spatial.base import create_spatial_mask_variable
+
+        args = list(args)
+        sobj = args[0]
+        args = args[1:]
+
+        create = kwargs.get(KeywordArgument.CREATE, False)
+        mask_variable = sobj.mask_variable
+        ret = None
+        if mask_variable is None:
+            if create:
+                mask_variable = create_spatial_mask_variable(VariableName.SPATIAL_MASK, None, sobj.dimensions)
+                sobj.set_mask(mask_variable)
+        if mask_variable is not None:
+            sobj.driver.validate_spatial_mask(mask_variable)
+            ret = mask_variable.get_mask(*args, **kwargs)
         return ret
 
+    def get_variable_collection(self, **kwargs):
+        """Here for backwards compatibility."""
+        return self.create_raw_field(**kwargs)
+
     def get_variable_metadata(self, variable_object):
         variable_metadata = get_variable_metadata_from_request_dataset(self, variable_object)
         return variable_metadata
@@ -363,7 +550,8 @@ class AbstractDriver(AbstractOcgisObject):
 
     @classmethod
     def get_variable_write_fill_value(cls, variable):
-        return cls.get_variable_for_writing(variable).fill_value
+        ret = cls.get_variable_for_writing(variable).fill_value
+        return ret
 
     @classmethod
     def get_variable_write_value(cls, variable):
@@ -386,21 +574,23 @@ class AbstractDriver(AbstractOcgisObject):
         # Create the dimensions if they are not present.
         if variable._dimensions is None:
             dist = self.dist
-            desired_dimensions = variable_metadata['dimensions']
-            new_dimensions = []
-            for d in desired_dimensions:
-                try:
-                    to_append = dist.get_dimension(d, group=variable.group, rank=vm.rank)
-                except KeyError:
-                    # Rank may not be mapped due to scoped VM.
-                    if vm.is_live:
-                        raise
+            desired_dimensions = variable_metadata.get('dimensions')
+            # Variable may not have any associated dimensions (attribute-only variable).
+            if desired_dimensions is not None:
+                new_dimensions = []
+                for d in desired_dimensions:
+                    try:
+                        to_append = dist.get_dimension(d, group=variable.group, rank=vm.rank)
+                    except KeyError:
+                        # Rank may not be mapped due to scoped VM.
+                        if vm.is_live:
+                            raise
+                        else:
+                            variable.convert_to_empty()
+                            break
                     else:
-                        variable.convert_to_empty()
-                        break
-                else:
-                    new_dimensions.append(to_append)
-            super(SourcedVariable, variable).set_dimensions(new_dimensions)
+                        new_dimensions.append(to_append)
+                super(SourcedVariable, variable).set_dimensions(new_dimensions)
 
         # Call the subclass variable initialization routine.
         self._init_variable_from_source_main_(variable, variable_metadata)
@@ -475,6 +665,38 @@ class AbstractDriver(AbstractOcgisObject):
             ret = cls._open_(uri, mode=mode, **kwargs)
         return ret
 
+    @staticmethod
+    def set_spatial_mask(sobj, value, cascade=False):
+        """
+        Set the spatial mask on an XYZ spatial container.
+
+        :param sobj: Target XYZ spatial container
+        :type sobj: :class:`ocgis.spatial.base.AbstractXYZSpatialContainer`
+        :param value: The spatial mask value. This may be a variable or a boolean array. If it is a boolean array, a
+         spatial mask variable will be created and this array set as its mask.
+        :type value: :class:`ocgis.Variable` | :class:`numpy.ndarray`
+        :param bool cascade: If ``True``, cascade the mask across shared dimensions on the grid.
+        """
+        from ocgis.spatial.grid import grid_set_mask_cascade
+        from ocgis.spatial.base import create_spatial_mask_variable
+        from ocgis.variable.base import Variable
+
+        if isinstance(value, Variable):
+            sobj.parent.add_variable(value, force=True)
+            mask_variable = value
+        else:
+            mask_variable = sobj.mask_variable
+            if mask_variable is None:
+                dimensions = sobj.dimensions
+                mask_variable = create_spatial_mask_variable(VariableName.SPATIAL_MASK, value, dimensions)
+                sobj.parent.add_variable(mask_variable)
+            else:
+                mask_variable.set_mask(value)
+        sobj.dimension_map.set_spatial_mask(mask_variable)
+
+        if cascade:
+            grid_set_mask_cascade(sobj)
+
     def validate_field(self, field):
         pass
 
@@ -492,6 +714,20 @@ class AbstractDriver(AbstractOcgisObject):
                                                                                                                cls.output_formats)
                 ocgis_lh(logger='driver', exc=DefinitionValidationError('output_format', msg))
 
+    @staticmethod
+    def validate_spatial_mask(mask_variable):
+        """
+        Validate the spatial mask variable.
+
+        :param mask_variable: Target spatial mask variable
+        :type mask_variable: :class:`ocgis.Variable`
+        :raises: ValueError
+        """
+        if mask_variable.attrs.get('ocgis_role') != 'spatial_mask':
+            msg = 'Mask variable "{}" must have an "ocgis_role" attribute with a value of "spatial_mask".'.format(
+                mask_variable.name)
+            raise ValueError(msg)
+
     def write_variable(self, *args, **kwargs):
         """Write a variable. Not applicable for tabular formats."""
         raise NotImplementedError
@@ -499,7 +735,6 @@ class AbstractDriver(AbstractOcgisObject):
     @classmethod
     def write_field(cls, field, opened_or_path, **kwargs):
         raise_if_empty(field)
-
         vc_to_write = cls._get_field_write_target_(field)
         cls.write_variable_collection(vc_to_write, opened_or_path, **kwargs)
 
@@ -517,31 +752,54 @@ class AbstractDriver(AbstractOcgisObject):
                 raise ValueError('Only paths allowed for parallel writes.')
 
         if write_mode is None:
-            if vm.size > 1:
-                write_modes = [MPIWriteMode.TEMPLATE, MPIWriteMode.FILL]
-            else:
-                write_modes = [MPIWriteMode.NORMAL]
+            write_modes = cls._get_write_modes_(vm, **kwargs)
         else:
             write_modes = [write_mode]
 
+        # vm.rank_print('tkd: write_modes', write_modes)
+
+        # Global string lengths are needed by the write. Set those while we still have global access.
+        for var in vc.values():
+            if var._string_max_length_global is None:
+                var.set_string_max_length_global()
+
         for write_mode in write_modes:
             cls._write_variable_collection_main_(vc, opened_or_path, write_mode, **kwargs)
 
+    @staticmethod
+    def _close_(obj):
+        """
+        Close and finalize the open file object.
+        """
+        obj.close()
+
+    @staticmethod
+    def _gc_nchunks_dst_(grid_chunker):
+        """
+        Calculate the default chunking decomposition for a destination grid in the grid chunker. The decomposition should
+        be a tuple of integers.
+
+        >>> (10, 5)
+        >>> (12,)
+
+        :param grid_chunker: The grid chunker object.
+        :type grid_chunker: :class:`~ocgis.spatial.grid_chunker.GridChunker`
+        :rtype: tuple
+        """
+        raise NotImplementedError
+
+    @classmethod
+    def _get_write_modes_(cls, the_vm, **kwargs):
+        if the_vm.size > 1:
+            write_modes = [MPIWriteMode.TEMPLATE, MPIWriteMode.FILL]
+        else:
+            write_modes = [MPIWriteMode.NORMAL]
+        return write_modes
+
     def _get_crs_main_(self, group_metadata):
         """Return the coordinate system variable or None if not found."""
         return None
 
-    def _get_dimensions_main_(self, group_metadata):
-        """
-        :param dict group_metadata: Metadata dictionary for the target group.
-        :return: A sequence of dimension objects.
-        :rtype: sequence
-        """
-
-        gmd = group_metadata['dimensions']
-        dims = [Dimension(dref['name'], size=dref['size'], src_idx='auto') for dref in list(gmd.values())]
-        return tuple(dims)
-
     @abc.abstractmethod
     def _get_metadata_main_(self):
         """
@@ -555,14 +813,6 @@ class AbstractDriver(AbstractOcgisObject):
     def _get_field_write_target_(cls, field):
         return field
 
-    @staticmethod
-    def _close_(obj):
-        """
-        Close and finalize the open file object.
-        """
-
-        obj.close()
-
     @abc.abstractmethod
     def _init_variable_from_source_main_(self, variable_object, variable_metadata):
         """Initialize everything but dimensions on the target variable."""
@@ -625,8 +875,40 @@ class AbstractTabularDriver(AbstractDriver):
         return ret
 
 
+@six.add_metaclass(ABCMeta)
+class AbstractUnstructuredDriver(AbstractOcgisObject):
+    default_axes_positions = (0, 0)  # Standard axes index for Y and X respectively.
+    _esmf_grid_class = constants.ESMFGridClass.MESH
+
+    @staticmethod
+    def get_element_dimension(gc):
+        """See :meth:`ocgis.spatial.geomc.AbstractGeometryCoordinates.element_dim`"""
+        cindex = gc.cindex
+        if cindex is None:
+            ret = gc.archetype.dimensions[0]
+        else:
+            ret = cindex.dimensions[0]
+        return ret
+
+    @staticmethod
+    def get_grid(field):
+        try:
+            ret = GridUnstruct(parent=field)
+        except GridDeficientError:
+            ret = None
+        return ret
+
+    @staticmethod
+    def get_multi_break_value(cindex):
+        """See :meth:`ocgis.spatial.geomc.AbstractGeometryCoordinates.multi_break_value`"""
+        mbv_name = OcgisConvention.Name.MULTI_BREAK_VALUE
+        return cindex.attrs.get(mbv_name)
+
+
 @contextmanager
 def driver_scope(ocgis_driver, opened_or_path=None, mode='r', **kwargs):
+    kwargs = kwargs.copy()
+
     if opened_or_path is None:
         try:
             # Attempt to get the request dataset from the driver. If not there, assume we are working with the driver
@@ -648,6 +930,8 @@ def driver_scope(ocgis_driver, opened_or_path=None, mode='r', **kwargs):
         should_close = False
     else:
         should_close = True
+        if rd is not None and rd.driver_kwargs is not None:
+            kwargs.update(rd.driver_kwargs)
         opened_or_path = ocgis_driver.open(uri=opened_or_path, mode=mode, rd=rd, **kwargs)
 
     try:
@@ -674,7 +958,7 @@ def format_attribute_for_dump_report(attr_value):
     return ret
 
 
-def get_dimension_map_raw(driver, group_metadata):
+def create_dimension_map_raw(driver, group_metadata):
     ret = DimensionMap.from_metadata(driver, group_metadata)
     return ret
 
@@ -729,7 +1013,12 @@ def get_dump_report_for_group(group, global_attributes_name='global', indent=0):
 
 
 def get_variable_metadata_from_request_dataset(driver, variable):
-    return get_group(driver.metadata_source, variable.group, has_root=False)['variables'][variable._source_name]
+    variables_metadata = get_group(driver.metadata_source, variable.group, has_root=False)['variables']
+    try:
+        ret = variables_metadata[variable._source_name]
+    except KeyError:
+        raise VariableMissingMetadataError(variable._source_name)
+    return ret
 
 
 def iter_all_group_keys(ddict, entry=None, has_root=True):
