@@ -1,7 +1,8 @@
+import itertools
 import logging
+from abc import ABCMeta
 from collections import OrderedDict
 from copy import deepcopy
-from warnings import warn
 
 import netCDF4 as nc
 import numpy as np
@@ -11,28 +12,38 @@ from netCDF4._netCDF4 import VLType, MFDataset, MFTime
 from ocgis import constants, vm
 from ocgis import env
 from ocgis.base import orphaned, raise_if_empty
-from ocgis.constants import MPIWriteMode, DimensionMapKey, KeywordArgument, DriverKey, CFName
+from ocgis.collection.field import Field
+from ocgis.constants import MPIWriteMode, DimensionMapKey, KeywordArgument, DriverKey, CFName, SourceIndexType
 from ocgis.driver.base import AbstractDriver, driver_scope
-from ocgis.driver.dimension_map import DimensionMap
-from ocgis.exc import ProjectionDoesNotMatch, PayloadProtectedError, OcgWarning, NoDataVariablesFound
+from ocgis.exc import ProjectionDoesNotMatch, PayloadProtectedError, NoDataVariablesFound, \
+    GridDeficientError
 from ocgis.util.helpers import itersubclasses, get_iter, get_formatted_slice, get_by_key_list, is_auto_dtype, get_group
 from ocgis.util.logging_ocgis import ocgis_lh
-from ocgis.variable.base import SourcedVariable, ObjectType, VariableCollection, \
-    get_slice_sequence_using_local_bounds
-from ocgis.variable.crs import CFCoordinateReferenceSystem, CoordinateReferenceSystem
+from ocgis.variable.base import SourcedVariable, ObjectType, get_slice_sequence_using_local_bounds
+from ocgis.variable.crs import CFCoordinateReferenceSystem, CoordinateReferenceSystem, CFRotatedPole, CFSpherical, \
+    AbstractProj4CRS
 from ocgis.variable.dimension import Dimension
 from ocgis.variable.temporal import TemporalVariable
+from ocgis.vmachine.mpi import OcgDist
 
 
 class DriverNetcdf(AbstractDriver):
     """
     Driver for netCDF files that avoids any hint of metadata.
+
+    Driver keyword arguments (``driver_kwargs``) to the request dataset:
+
+    * Any keyword arguments to dataset or multi-file dataset creation.
     """
     extensions = ('.*\.nc', 'http.*')
     key = DriverKey.NETCDF
     output_formats = 'all'
     common_extension = 'nc'
 
+    @property
+    def data_model(self):
+        return self.metadata_source['file_format']
+
     @staticmethod
     def get_data_variable_names(group_metadata, group_dimension_map):
         return tuple()
@@ -41,11 +52,6 @@ class DriverNetcdf(AbstractDriver):
     def get_variable_for_writing_temporal(cls, temporal_variable):
         return temporal_variable.value_numtime
 
-    def get_variable_collection(self, **kwargs):
-        with driver_scope(self) as ds:
-            ret = read_from_collection(ds, self.rd, parent=None, uid=kwargs.pop('uid', None))
-        return ret
-
     def get_variable_value(self, variable):
         return get_value_from_request_dataset(variable)
 
@@ -92,15 +98,15 @@ class DriverNetcdf(AbstractDriver):
             if isinstance(dtype, ObjectType):
                 dtype = dtype.create_vltype(dataset, dimensions[0].name + '_VLType')
             # Assume we are writing string data if the data type is object.
-            elif dtype == str:
+            elif dtype == str or var.is_string_object:
                 dtype = 'S1'
 
             if len(dimensions) > 0:
                 # Special handling for string variables.
                 if dtype == 'S1':
-                    max_length = max([len(e) for e in var.get_value()])
-                    dimensions = [var.dimensions[0],
-                                  Dimension('{}_ocgis_slen'.format(var.name), max_length)]
+                    max_length = var.string_max_length_global
+                    assert max_length is not None
+                    dimensions = [var.dimensions[0], Dimension('{}_ocgis_slen'.format(var.name), max_length)]
                 dimensions = list(dimensions)
                 # Convert the unlimited dimension to fixed size if requested.
                 for idx, d in enumerate(dimensions):
@@ -113,7 +119,9 @@ class DriverNetcdf(AbstractDriver):
                 dimensions = [d.name for d in dimensions]
 
             # Only use the fill value if something is masked.
-            if len(dimensions) > 0 and not file_only and var.has_masked_values:
+            is_nc3 = dataset.data_model.startswith('NETCDF3')
+            if ((len(dimensions) > 0 and var.has_masked_values) and not file_only) or (
+                    is_nc3 and not var.has_allocated_value and len(dimensions) > 0):
                 fill_value = cls.get_variable_write_fill_value(var)
             else:
                 # Copy from original attributes.
@@ -138,15 +146,23 @@ class DriverNetcdf(AbstractDriver):
                 data_value = cls.get_variable_write_value(var)
                 # Only write allocated values.
                 if data_value is not None:
-                    if var.dtype == str:
+                    if var.dtype == str or var.is_string_object:
                         for idx in range(fill_slice[0].start, fill_slice[0].stop):
-                            curr_value = data_value[idx]
+                            try:
+                                curr_value = data_value[idx - fill_slice[0].start]
+                            except Exception as e:
+                                msg = "Variable name is '{}'. Original message: ".format(var.name) + e.message
+                                raise e.__class__(msg)
                             for sidx, sval in enumerate(curr_value):
                                 ncvar[idx, sidx] = sval
                     elif var.ndim == 0:
                         ncvar[:] = data_value
                     else:
-                        ncvar.__setitem__(fill_slice, data_value)
+                        try:
+                            ncvar.__setitem__(fill_slice, data_value)
+                        except Exception as e:
+                            msg = "Variable name is '{}'. Original message: ".format(var.name) + e.message
+                            raise e.__class__(msg)
 
         # Only set variable attributes if this is not a fill operation.
         if write_mode != MPIWriteMode.FILL:
@@ -158,17 +174,6 @@ class DriverNetcdf(AbstractDriver):
 
     @classmethod
     def _write_variable_collection_main_(cls, vc, opened_or_path, write_mode, **kwargs):
-        """
-        Write the variable collection to an open netCDF dataset or file path.
-
-        :param dataset: The open dataset object or path for the write.
-        :type dataset: :class:`netCDF4.Dataset` or str
-        :param bool file_only: If ``True``, we are not filling the value variables. Only the file schema and dimension
-         values will be written.
-        :param bool unlimited_to_fixedsize: If ``True``, convert the unlimited dimension to fixed size.
-        :param kwargs: Extra keyword arguments in addition to ``dimensions`` and ``fill_value`` to pass to
-         ``createVariable``. See http://unidata.github.io/netcdf4-python/netCDF4.Dataset-class.html#createVariable
-        """
         assert write_mode is not None
 
         dataset_kwargs = kwargs.get('dataset_kwargs', {})
@@ -180,13 +185,19 @@ class DriverNetcdf(AbstractDriver):
         else:
             mode = 'w'
 
+        # For an asynchronous write, treat everything like a single rank.
+        if write_mode == MPIWriteMode.ASYNCHRONOUS:
+            possible_ranks = [0]
+        else:
+            possible_ranks = vm.ranks
+
         # Write the data on each rank.
-        for idx, rank_to_write in enumerate(vm.ranks):
+        for idx, rank_to_write in enumerate(possible_ranks):
             # The template write only occurs on the first rank.
             if write_mode == MPIWriteMode.TEMPLATE and rank_to_write != 0:
                 pass
             # If this is not a template write, fill the data.
-            elif vm.rank == rank_to_write:
+            elif write_mode == MPIWriteMode.ASYNCHRONOUS or vm.rank == rank_to_write:
                 with driver_scope(cls, opened_or_path=opened_or_path, mode=mode, **dataset_kwargs) as dataset:
                     # Write global attributes if we are not filling data.
                     if write_mode != MPIWriteMode.FILL:
@@ -211,14 +222,21 @@ class DriverNetcdf(AbstractDriver):
                     dataset.sync()
             vm.barrier()
 
-    def _get_dimensions_main_(self, group_metadata):
-        return tuple(get_dimensions_from_netcdf_metadata(group_metadata, list(group_metadata['dimensions'].keys())))
-
     def _get_metadata_main_(self):
         with driver_scope(self) as ds:
             ret = parse_metadata(ds)
         return ret
 
+    @classmethod
+    def _get_write_modes_(cls, the_vm, **kwargs):
+        dataset_kwargs = kwargs.get(KeywordArgument.DATASET_KWARGS, {})
+
+        ret = AbstractDriver._get_write_modes_(the_vm, **kwargs)
+        if env.USE_NETCDF4_MPI and the_vm.size > 1:
+            if dataset_kwargs.get('format', 'NETCDF4') == 'NETCDF4' and dataset_kwargs.get('parallel', True):
+                ret = [MPIWriteMode.ASYNCHRONOUS]
+        return ret
+
     def _init_variable_from_source_main_(self, variable, variable_object):
         init_variable_using_metadata_for_netcdf(variable, self.rd.metadata)
 
@@ -227,9 +245,19 @@ class DriverNetcdf(AbstractDriver):
         """
         :rtype: object
         """
+        kwargs = kwargs.copy()
         group_indexing = kwargs.pop('group_indexing', None)
+        lvm = kwargs.pop('vm', vm)
 
         if isinstance(uri, six.string_types):
+            # Open the dataset in parallel if we want to use the netCDF MPI capability. It may not be available even in
+            # parallel.
+            if mode == 'w' and lvm.size > 1:
+                if kwargs.get('format', 'NETCDF4') == 'NETCDF4':
+                    if kwargs.get('parallel') is None and env.USE_NETCDF4_MPI:
+                        kwargs['parallel'] = True
+                    if kwargs.get('parallel') and kwargs.get('comm') is None:
+                        kwargs['comm'] = lvm.comm
             ret = nc.Dataset(uri, mode=mode, **kwargs)
         else:
             ret = nc.MFDataset(uri, **kwargs)
@@ -241,25 +269,42 @@ class DriverNetcdf(AbstractDriver):
         return ret
 
 
-class DriverNetcdfCF(DriverNetcdf):
-    """
-    Metadata-aware netCDF driver.
-    """
-    key = DriverKey.NETCDF_CF
-    _default_crs = env.DEFAULT_COORDSYS
-    _priority = True
+@six.add_metaclass(ABCMeta)
+class AbstractDriverNetcdfCF(DriverNetcdf):
+    def create_dimension_map(self, group_metadata, strict=False):
+        dmap = super(AbstractDriverNetcdfCF, self).create_dimension_map(group_metadata, strict=strict)
 
-    def get_dimension_map(self, group_metadata, strict=False):
-        # Get dimension variable metadata. This involves checking for the presence of any bounds variables.
+        # Check for time, level, and realization variables. This involves checking for the presence of any bounds
+        # variables.
+        axes = {'realization': 'R', 'time': 'T', 'level': 'Z'}
+        entries = self._create_dimension_map_entries_dict_(axes, group_metadata, strict)
+        self._update_dimension_map_with_entries_(entries, dmap)
+
+        # Check for coordinate system variables. This will check every variable.
+        crs_name = get_coordinate_system_variable_name(self, group_metadata)
+        if crs_name is not None:
+            dmap.set_crs(crs_name)
+
+        # Check for a spatial mask.
+        variables = group_metadata['variables']
+        for varname, var in variables.items():
+            if 'ocgis_role' in var.get('attrs', {}):
+                if var['attrs']['ocgis_role'] == 'spatial_mask':
+                    dmap.set_spatial_mask(varname, attrs=var['attrs'])
+                    break
+
+        return dmap
+
+    @staticmethod
+    def _create_dimension_map_entries_dict_(axes, group_metadata, strict, attr_name='axis'):
         variables = group_metadata['variables']
-        dimensions = group_metadata['dimensions']
-        axes = {'realization': 'R', 'time': 'T', 'level': 'Z', 'x': 'X', 'y': 'Y'}
         check_bounds = list(axes.keys())
-        check_bounds.pop(check_bounds.index('realization'))
+        if 'realization' in check_bounds:
+            check_bounds.pop(check_bounds.index('realization'))
 
         # Get the main entry for each axis.
         for k, v in list(axes.items()):
-            axes[k] = get_dimension_map_entry(v, variables, dimensions, strict=strict)
+            axes[k] = create_dimension_map_entry(v, variables, strict=strict, attr_name=attr_name)
 
         # Attempt to find bounds for each entry (ignoring realizations).
         for k in check_bounds:
@@ -275,24 +320,75 @@ class DriverNetcdfCF(DriverNetcdf):
                         ocgis_lh(msg, logger='nc.driver', level=logging.WARNING)
                         bounds_var = None
                 axes[k]['bounds'] = bounds_var
+        entries = {k: v for k, v in list(axes.items()) if v is not None}
+        return entries
 
-        # Create the template dimension map dictionary.
-        ret = {k: v for k, v in list(axes.items()) if v is not None}
+    def _get_crs_main_(self, group_metadata):
+        return get_crs_variable(group_metadata, self.rd.rotated_pole_priority)
 
-        # Check for coordinate system variables. This will check every variable.
-        crs_name = get_coordinate_system_variable_name(self, group_metadata)
-        if crs_name is not None:
-            ret[DimensionMapKey.CRS] = {DimensionMapKey.VARIABLE: crs_name}
+    @staticmethod
+    def _update_dimension_map_with_entries_(entries, dimension_map):
+        for k, v in entries.items():
+            variable_name = v.pop('variable')
+            dimension_map.set_variable(k, variable_name, **v)
 
-        ret = DimensionMap.from_dict(ret)
 
-        # Check for a spatial mask.
-        for varname, var in group_metadata['variables'].items():
-            if 'ocgis_role' in var.get('attrs', {}):
-                if var['attrs']['ocgis_role'] == 'spatial_mask':
-                    ret.set_spatial_mask(varname, attrs=var['attrs'])
+class DriverNetcdfCF(AbstractDriverNetcdfCF):
+    """
+    Metadata-aware netCDF driver interpreting CF-Grid by default.
+    """
+    key = DriverKey.NETCDF_CF
+    _esmf_fileformat = 'GRIDSPEC'
+    _default_crs = env.DEFAULT_COORDSYS
+    _priority = True
 
-        return ret
+    @staticmethod
+    def array_resolution(value, axis):
+        """See :meth:`ocgis.driver.base.AbstractDriver.array_resolution`"""
+        if value.size == 1:
+            return 0.0
+        else:
+            resolution_limit = constants.RESOLUTION_LIMIT
+            is_vectorized = value.ndim == 1
+            if is_vectorized:
+                target = np.abs(np.diff(np.abs(value[0:resolution_limit])))
+            else:
+                if axis == 0:
+                    target = np.abs(np.diff(np.abs(value[:, 0:resolution_limit]), axis=axis))
+                elif axis == 1:
+                    target = np.abs(np.diff(np.abs(value[0:resolution_limit, :]), axis=axis))
+                else:
+                    raise NotImplementedError(axis)
+            ret = np.mean(target)
+            return ret
+
+    def create_dimension_map(self, group_metadata, strict=False):
+        dmap = super(DriverNetcdfCF, self).create_dimension_map(group_metadata, strict=strict)
+
+        # Get dimension variable metadata. This involves checking for the presence of any bounds variables.
+        axes = {'x': 'X', 'y': 'Y'}
+        entries = self._create_dimension_map_entries_dict_(axes, group_metadata, strict, attr_name='axis')
+        self._update_dimension_map_with_entries_(entries, dmap)
+
+        # Hack to get the original coordinate system. Tell the driver to get rotated pole if it can from the metadata.
+        original_rpp = self.rd.rotated_pole_priority
+        try:
+            self.rd.rotated_pole_priority = True
+            raw_crs = self.get_crs(group_metadata)
+        finally:
+            self.rd.rotated_pole_priority = original_rpp
+        # Swap out correct axis variables for spherical as opposed to rotated latitude longitude
+        if not self.rd.rotated_pole_priority and isinstance(raw_crs, CFRotatedPole):
+            crs = self.get_crs(group_metadata)
+            if isinstance(crs, CFSpherical):
+                axes = {DimensionMapKey.X: {'value': CFName.StandardName.LONGITUDE, 'axis': 'X'},
+                        DimensionMapKey.Y: {'value': CFName.StandardName.LATITUDE, 'axis': 'Y'}}
+                crs_entries = self._create_dimension_map_entries_dict_(axes, group_metadata, strict,
+                                                                       attr_name=CFName.STANDARD_NAME)
+                self._update_dimension_map_with_entries_(crs_entries, dmap)
+                env.SET_GRID_AXIS_ATTRS = False
+                env.COORDSYS_ACTUAL = raw_crs
+        return dmap
 
     @staticmethod
     def get_data_variable_names(group_metadata, group_dimension_map):
@@ -335,18 +431,37 @@ class DriverNetcdfCF(DriverNetcdf):
             ret = None
         return ret
 
-    def _get_crs_main_(self, group_metadata):
-        return get_crs_variable(group_metadata)
+    @staticmethod
+    def get_grid(field):
+        from ocgis import Grid
+        try:
+            ret = Grid(parent=field)
+        except GridDeficientError:
+            ret = None
+        return ret
+
+    @staticmethod
+    def _gc_iter_dst_grid_slices_(grid_chunker):
+        slice_store = []
+        ydim_name = grid_chunker.dst_grid.dimensions[0].name
+        xdim_name = grid_chunker.dst_grid.dimensions[1].name
+        dst_grid_shape_global = grid_chunker.dst_grid.shape_global
+        for idx in range(grid_chunker.dst_grid.ndim):
+            splits = grid_chunker.nchunks_dst[idx]
+            size = dst_grid_shape_global[idx]
+            slices = create_slices_for_dimension(size, splits)
+            slice_store.append(slices)
+        for slice_y, slice_x in itertools.product(*slice_store):
+            yield {ydim_name: create_slice_from_tuple(slice_y),
+                   xdim_name: create_slice_from_tuple(slice_x)}
 
     @classmethod
     def _get_field_write_target_(cls, field):
         """Collective!"""
 
-        # These changes to the field can be maintained following a write.
         if field.crs is not None:
-            field.crs.format_field(field)
+            field.crs.format_spatial_object(field)
 
-        # Putting units on bounds for netCDF-CF can confuse some parsers.
         grid = field.grid
         if grid is not None:
             # If any grid pieces are masked, ensure the mask is created across all grids.
@@ -362,14 +477,36 @@ class DriverNetcdfCF(DriverNetcdf):
             if create_mask and not grid.has_mask:
                 grid.get_mask(create=True)
 
+            # Putting units on bounds for netCDF-CF can confuse some parsers.
             if grid.has_bounds:
                 field = field.copy()
                 field.x.bounds.attrs.pop('units', None)
                 field.y.bounds.attrs.pop('units', None)
 
+        # Remove the current coordinate system if this is a dummy coordinate system.
+        if env.COORDSYS_ACTUAL is not None:
+            field = field.copy()
+            field.set_crs(env.COORDSYS_ACTUAL, should_add=True)
+
         return field
 
 
+def create_slice_from_tuple(tup):
+    return slice(tup[0], tup[1])
+
+
+def create_slices_for_dimension(size, splits):
+    ompi = OcgDist(size=splits)
+    dimname = 'foo'
+    ompi.create_dimension(dimname, size, dist=True)
+    ompi.update_dimension_bounds()
+    slices = []
+    for rank in range(splits):
+        dimension = ompi.get_dimension(dimname, rank=rank)
+        slices.append(dimension.bounds_local)
+    return slices
+
+
 def parse_metadata(rootgrp, fill=None):
     if fill is None:
         fill = OrderedDict()
@@ -391,12 +528,16 @@ def read_from_collection(target, request_dataset, parent=None, name=None, source
     except NoDataVariablesFound:
         rename_variable_map = {}
 
-    ret = VariableCollection(attrs=get_netcdf_attributes(target), parent=parent, name=name, source_name=source_name,
-                             uid=uid)
+    ret = Field(attrs=get_netcdf_attributes(target), parent=parent, name=name, source_name=source_name, uid=uid)
+    pred = request_dataset.predicate
     for varname, ncvar in target.variables.items():
+        if pred is not None and not pred(varname):
+            continue
         source_name = varname
         name = rename_variable_map.get(varname, varname)
-        ret[name] = SourcedVariable(name=name, request_dataset=request_dataset, parent=ret, source_name=source_name)
+        sv = SourcedVariable(name=name, request_dataset=request_dataset, parent=ret, source_name=source_name)
+        ret[name] = sv
+
     for group_name, ncgroup in list(target.groups.items()):
         child = read_from_collection(ncgroup, request_dataset, parent=ret, name=group_name, uid=uid)
         ret.add_child(child)
@@ -422,17 +563,17 @@ def init_variable_using_metadata_for_netcdf(variable, metadata):
             variable._dtype = desired_dtype
 
         if variable._fill_value == 'auto':
-            if var['fill_value_packed'] is not None:
+            if var.get('fill_value_packed') is not None:
                 desired_fill_value = var['fill_value_packed']
             else:
-                desired_fill_value = var['fill_value']
+                desired_fill_value = var.get('fill_value')
             variable._fill_value = deepcopy(desired_fill_value)
 
         variable_attrs = variable._attrs
         # Offset and scale factors are not supported by OCGIS. The data is unpacked when written to a new output file.
-        # tdk: consider supporting offset and scale factors
+        # TODO: Consider supporting offset and scale factors for write operations.
         exclude = ['add_offset', 'scale_factor']
-        for k, v in list(var['attrs'].items()):
+        for k, v in list(var.get('attrs', {}).items()):
             if k in exclude:
                 continue
             if k not in variable_attrs:
@@ -456,23 +597,6 @@ def get_coordinate_system_variable_name(driver_object, group_metadata):
     return crs_name
 
 
-def get_dimensions_from_netcdf_metadata(metadata, desired_dimensions):
-    new_dimensions = []
-    for dim_name in desired_dimensions:
-        dim = metadata['dimensions'][dim_name]
-        dim_length = dim['size']
-        if dim['isunlimited']:
-            length = None
-            length_current = dim_length
-        else:
-            length = dim_length
-            length_current = None
-        # tdk: identify best method to remove the need to set 'auto' when creating a source index
-        new_dim = Dimension(dim_name, size=length, size_current=length_current, src_idx='auto')
-        new_dimensions.append(new_dim)
-    return new_dimensions
-
-
 def get_value_from_request_dataset(variable):
     if variable.protected:
         raise PayloadProtectedError(variable.name)
@@ -491,9 +615,10 @@ def get_value_from_request_dataset(variable):
         ncvar = source.variables[desired_name]
 
         # Allow multi-unit time values for temporal variables.
-        if isinstance(variable, TemporalVariable) and isinstance(source, MFDataset):
+        if isinstance(variable, TemporalVariable) and isinstance(source, MFDataset) and rd.format_time:
+            # MFTime may fail if time_bnds do not have a calendar attribute.
+            # Use rd.dimension_map.set_bounds('time', None) to disable indexing on time_bnds.
             ncvar = MFTime(ncvar)
-
         ret = get_variable_value(ncvar, variable.dimensions)
     return ret
 
@@ -514,18 +639,27 @@ def get_variable_value(variable, dimensions):
         to_format = [None] * len(dimensions)
         for idx in range(len(dimensions)):
             current_dimension = dimensions[idx]
-            if current_dimension._src_idx is None:
+            si_type = current_dimension._src_idx_type
+            if si_type is None:
                 if current_dimension.bounds_local is None:
                     to_insert = slice(0, len(current_dimension))
                 else:
                     to_insert = slice(*current_dimension.bounds_local)
-            else:
+            elif si_type == SourceIndexType.FANCY:
                 to_insert = current_dimension._src_idx
+            elif si_type == SourceIndexType.BOUNDS:
+                to_insert = slice(*current_dimension._src_idx)
+            else:
+                raise NotImplementedError(si_type)
             to_format[idx] = to_insert
         slc = get_formatted_slice(to_format, len(dimensions))
     else:
         slc = slice(None)
-    ret = variable.__getitem__(slc)
+    try:
+        ret = variable.__getitem__(slc)
+    except IndexError:
+        # TODO: Hack! Slicing the top-level MFTime variable does not work with multifiles.
+        ret = super(MFTime, variable).__getitem__(slc)
     return ret
 
 
@@ -533,7 +667,7 @@ def create_dimension_or_pass(dim, dataset, write_mode=MPIWriteMode.NORMAL):
     if dim.name not in dataset.dimensions:
         if dim.is_unlimited:
             size = None
-        elif write_mode == MPIWriteMode.TEMPLATE:
+        elif write_mode in (MPIWriteMode.TEMPLATE, MPIWriteMode.ASYNCHRONOUS):
             lower, upper = dim.bounds_global
             size = upper - lower
         else:
@@ -541,7 +675,7 @@ def create_dimension_or_pass(dim, dataset, write_mode=MPIWriteMode.NORMAL):
         dataset.createDimension(dim.name, size)
 
 
-def get_crs_variable(metadata, to_search=None):
+def get_crs_variable(metadata, rotated_pole_priority, to_search=None):
     found = []
     variables = metadata['variables']
 
@@ -551,7 +685,7 @@ def get_crs_variable(metadata, to_search=None):
                 continue
         for potential in itersubclasses(CFCoordinateReferenceSystem):
             try:
-                crs = potential.load_from_metadata(vname, metadata)
+                crs = potential.load_from_metadata(vname, metadata, strict=False)
                 found.append(crs)
                 break
             except ProjectionDoesNotMatch:
@@ -566,14 +700,49 @@ def get_crs_variable(metadata, to_search=None):
     else:
         crs = found[0]
 
+    # Allow spherical coordinate systems to overload rotated pole if they are present in the metadata.
+    if not rotated_pole_priority and isinstance(crs, CFRotatedPole):
+        try:
+            crs = CFSpherical.load_from_metadata(None, metadata, depth=2)
+        except ProjectionDoesNotMatch:
+            # No spherical coordinate system data available. Fall back on previous crs.
+            pass
+
+    # If there is no CRS, check for OCGIS coordinate systems.
+    if crs is None:
+        try:
+            crs = AbstractProj4CRS.load_from_metadata(metadata)
+        except ProjectionDoesNotMatch:
+            pass
+
     return crs
 
 
-def get_dimension_map_entry(axis, variables, dimensions, strict=False):
+def create_dimension_map_entry(src, variables, strict=False, attr_name='axis'):
+    """
+    Create a dimension map entry dictionary by searching variable metadata using attribute constraints.
+
+    :param src: The source information to use for constructing the entry. If ``src`` is a dictionary, it must have two
+     entries. The key ``'value'`` corresponds to the string attribute value. The key ``'axis'`` is the representative
+     axis to assign the source value (for example ``'X'`` or ``'Y'``).
+    :type src: str | dict
+    :param dict variables: The metadata entries for the group's variables.
+    :param bool strict: If ``False``, do not use a strict interpretation of metadata. Allow some standard approaches for
+     handling metadata exceptions.
+    :param str attr_name: Name of the attribute to use for checking the attribute values form ``src``.
+    :return: dict
+    """
+    if isinstance(src, dict):
+        axis = src['axis']
+        attr_value = src['value']
+    else:
+        axis = src
+        attr_value = src
+
     axis_vars = []
     for variable in list(variables.values()):
-        vattrs = variable['attrs']
-        if vattrs.get('axis') == axis:
+        vattrs = variable.get('attrs', {})
+        if vattrs.get(attr_name) == attr_value:
             if len(variable['dimensions']) == 0:
                 pass
             else:
@@ -581,7 +750,7 @@ def get_dimension_map_entry(axis, variables, dimensions, strict=False):
 
     # Try to find by default names.
     if not strict and len(axis_vars) == 0:
-        possible_names = CFName.get_name_mapping().get(axis, [])
+        possible_names = CFName.get_axis_mapping().get(axis, [])
         for pn in possible_names:
             if pn in list(variables.keys()):
                 axis_vars.append(variables[pn]['name'])
@@ -602,8 +771,7 @@ def get_dimension_map_entry(axis, variables, dimensions, strict=False):
     elif len(axis_vars) > 1:
         msg = 'Multiple axis (axis="{}") possibilities found using variable(s) "{}". Use a dimension map to specify ' \
               'the appropriate coordinate dimensions.'
-        w = OcgWarning(msg.format(axis, axis_vars))
-        warn(w)
+        ocgis_lh(msg.format(axis, axis_vars), level=logging.WARN, logger='ocgis.driver.nc', force=True)
         ret = None
     else:
         ret = None
@@ -622,6 +790,12 @@ def get_netcdf_attributes(target):
     return attributes
 
 
+def remove_netcdf_attribute(filename, variable_name, attr_name):
+    with driver_scope(DriverNetcdf, opened_or_path=filename, mode='a') as ds:
+        var = ds[variable_name]
+        var.delncattr(attr_name)
+
+
 def update_group_metadata(rootgrp, fill):
     global_attributes = get_netcdf_attributes(rootgrp)
     fill.update({'global_attributes': global_attributes})
