@@ -7,7 +7,7 @@ import six
 
 from ocgis import constants
 from ocgis import env
-from ocgis.constants import DMK
+from ocgis.constants import DMK, DriverKey
 from ocgis.driver.dimension_map import DimensionMap
 from ocgis.driver.registry import get_driver_class, driver_registry
 from ocgis.driver.request.base import AbstractRequestObject
@@ -19,8 +19,7 @@ from ocgis.util.units import get_units_object, get_are_units_equivalent
 
 class RequestDataset(AbstractRequestObject):
     """
-    A :class:`~ocgis.RequestDataset` contains all the information necessary to find and subset a variable (by time
-    and/or level) contained in a local or OpenDAP-hosted CF dataset.
+    Contains all the information necessary to create an OCGIS field via an OCGIS driver.
 
     >>> from ocgis import RequestDataset
     >>> uri = 'http://some.opendap.dataset'
@@ -29,8 +28,9 @@ class RequestDataset(AbstractRequestObject):
     >>> variable = 'tasmax'
     >>> rd = RequestDataset(uri, variable)
 
-    :param uri: The absolute path (URLs included) to the data's location.
-    :type uri: :class:`str` | `sequence` of :class:`str`
+    :param uri: The absolute path (URLs included) to the data's location. If ``None``, either ``opened`` or ``metadata``
+     must be provided.
+    :type uri: :class:`str` | `sequence` of :class:`str` | ``None``
 
     >>> uri = 'http://some.opendap.dataset'
     >>> uri = '/path/to/local/file.nc'
@@ -73,9 +73,9 @@ class RequestDataset(AbstractRequestObject):
 
     >>> t_conform_units_to = 'days since 1949-1-1'
 
-    :param str grid_abstraction: Abstract the geometry data to either ``'point'`` or ``'polygon'``. If ``'polygon'`` is 
-     not possible due to missing bounds, ``'point'`` will be used instead. If ``'auto'`` (the default), identify the 
-     grid abstraction automatically.
+    :param str grid_abstraction: Abstract the geometry data to either ``'point'`` or ``'polygon'``. If ``'polygon'`` is
+     not possible due to missing bounds, ``'point'`` will be used instead. If ``'auto'`` (the default), identify the
+     grid abstraction automatically. Unstructured data formats also allow for ``'line'``.
 
     .. note:: The ``abstraction`` argument in :class:`~ocgis.OcgOperations` will overload this.
 
@@ -90,19 +90,23 @@ class RequestDataset(AbstractRequestObject):
     :param conform_units_to: Destination units for conversion. If this parameter is set, then the :mod:`cf_units` module
      must be installed.
     :type conform_units_to: :class:`str` | :class:`cfunits.Units` | `sequence` of possible types
-    :param str driver: If ``None``, autodiscover the appropriate driver. Acceptable values are listed below.
+    :param driver: If ``None``, autodiscover the appropriate driver. Acceptable values are listed below. Class objects
+     for the associated driver key are also accepted.
+    :type driver: str | :class:`~ocgis.driver.base.AbstractDriver`
 
-    ============= ================= =============================================
-    Value         File Extension(s) Description
-    ============= ================= =============================================
-    ``'netcdf-cf' ``'nc'``          A netCDF file using a CF metadata convention.
-    ``'netcdf' `` ``'nc'``          A netCDF file with no metadata convention.
-    ``'vector'``  ``'shp'``         An ESRI Shapefile or other vector source.
-    ``'csv'``     ``'csv'``         A CSV file.
-    ============= ================= =============================================
+    ================== ================= ======================================================================
+    Value              File Extension(s) Description
+    ================== ================= ======================================================================
+    ``'netcdf-cf'``    ``'nc'``          A netCDF file using a CF-Grid metadata convention.
+    ``'netcdf-ugrid'`` ``'nc'``          A netCDF file using the UGRID (Unstructured Grid) metadata convention.
+    ``'netcdf-scrip'`` ``'nc'``          A netCDF file using the SCRIP metadata convention.
+    ``'netcdf'``       ``'nc'``          A netCDF file with no metadata convention.
+    ``'vector'``       ``'shp'``         An ESRI Shapefile or other vector source.
+    ``'csv'``          ``'csv'``         A CSV file.
+    ================== ================= ======================================================================
 
     :param str field_name: Name of the requested field in the output collection. If ``None``, defaults to the variable
-     name or names joined by ``_``.
+     defaults to the data variable name. If there are multiple data variables, the default name is ``'ocgis_field'``.
     :param bool regrid_source: If ``False``, do not regrid this dataset. This is relevant only if a
      ``regrid_destination`` dataset is present. Please see :ref:`esmpy-regridding` for an overview.
     :param bool regrid_destination: If ``True``, use this dataset as the destination grid for a regridding operation.
@@ -110,22 +114,35 @@ class RequestDataset(AbstractRequestObject):
      an overview.
     :param rename_variable: A sequence with the same length as ``variable``. Provides new names for the variables.
     :type rename_variable: `sequence` of :class:`str`
-    :param dict metadata: Overload the metadata that would normally be loaded by the driver.
+    :param dict metadata: Overload the metadata that would normally be loaded by the driver. If metadata is provided and
+     ``uri`` is ``None``, a field will be created by interpreting the provided metadata.
     :param opened: An open file used as a write target for the driver.
     :type opened: varies by ``driver`` class
     :param int uid: A unique identifier for the request dataset.
+    :param predicate: A filter function returning ``True`` if a variable should be included in the output field. The
+      function should take a single argument which is a sequence of string variable names. This function is applied
+      directly to the metadata before other functions (i.e. identifying data variables).
+    :type predicate: `function`
+    :param bool rotated_pole_priority: If ``False``, attempt to use representative spherical coordinates if available in
+     a dataset having a rotated pole coordinate system. If ``True``, use the rotated coordinate even if representative
+     coordinates are available.
+    
+    >>> predicate = lambda x: x.startswith('w')
 
     .. _time units: http://netcdf4-python.googlecode.com/svn/trunk/docs/netCDF4-module.html#num2date
     .. _time calendar: http://netcdf4-python.googlecode.com/svn/trunk/docs/netCDF4-module.html#num2date
+
+    :param dict driver_kwargs: Any keyword arguments to driver creation. See the driver documentation for a description
+     of accepted parameters. These are often format-specific and not easily generalized.
+    :param grid_is_isomorphic: See documentation for :class:`ocgis.Field`
     """
 
-    # TODO: driver-specific option for netcdf: grid_abstraction - perhaps driver_options?
     def __init__(self, uri=None, variable=None, units=None, time_range=None, time_region=None,
                  time_subset_func=None, level_range=None, conform_units_to=None, crs='auto', t_units=None,
-                 t_calendar=None, t_conform_units_to=None, grid_abstraction='auto', dimension_map=None,
-                 field_name=None, driver=None, regrid_source=True, regrid_destination=False, metadata=None,
-                 format_time=True, opened=None, uid=None, rename_variable=None):
-
+                 t_calendar=None, t_conform_units_to=None, grid_abstraction='auto', grid_is_isomorphic='auto',
+                 dimension_map=None, field_name=None, driver=None, regrid_source=True, regrid_destination=False,
+                 metadata=None, format_time=True, opened=None, uid=None, rename_variable=None, predicate=None,
+                 rotated_pole_priority=False, driver_kwargs=None):
         self._is_init = True
 
         self._field_name = field_name
@@ -133,30 +150,42 @@ class RequestDataset(AbstractRequestObject):
         self._time_range = None
         self._time_region = None
         self._time_subset_func = None
+        self._driver_kwargs = driver_kwargs
 
+        if rename_variable is not None:
+            rename_variable = get_tuple(rename_variable)
+        self._rename_variable = rename_variable
+
+        self.rotated_pole_priority = rotated_pole_priority
+        self.predicate = predicate
         if dimension_map is not None and isinstance(dimension_map, dict):
             dimension_map = DimensionMap.from_dict(dimension_map)
         self._dimension_map = dimension_map
 
         self._metadata = deepcopy(metadata)
         self._uri = None
-        self._rename_variable = rename_variable
         self.uid = uid
 
         # This is an "open" file-like object that may be passed in-place of file location parameters.
-        self.opened = opened
-        if self.opened is not None and driver is None:
+        self._opened = opened
+        if opened is not None and driver is None:
             msg = 'If "opened" is not None, then a "driver" must be provided.'
             ocgis_lh(logger='request', exc=RequestValidationError('driver', msg))
 
         # Field creation options.
         self.format_time = format_time
         self.grid_abstraction = grid_abstraction
+        self.grid_is_isomorphic = grid_is_isomorphic
         # Flag used for regridding to determine if the coordinate system was assigned during initialization.
         self._has_assigned_coordinate_system = False if crs == 'auto' else True
 
         if uri is None:
-            if opened is None:
+            # Fields may be created from pure metadata.
+            if metadata is not None:
+                # The default OCGIS driver is NetCDF.
+                if driver is None:
+                    driver = DriverKey.NETCDF_CF
+            elif opened is None:
                 ocgis_lh(logger='request', exc=RequestValidationError('uri', 'Cannot be None'))
         else:
             self._uri = get_uri(uri)
@@ -165,7 +194,7 @@ class RequestDataset(AbstractRequestObject):
             klass = get_autodiscovered_driver(uri)
         else:
             klass = get_driver(driver)
-        self.driver = klass(self)
+        self._driver = klass(self)
 
         if variable is not None:
             variable = get_tuple(variable)
@@ -208,11 +237,6 @@ class RequestDataset(AbstractRequestObject):
         else:
             return False
 
-    # tdk: remove
-    @property
-    def _name(self):
-        raise NotImplementedError
-
     def __iter__(self):
         attrs = ['variable', 'units', 'conform_units_to']
         for ii in range(len(self)):
@@ -262,6 +286,15 @@ class RequestDataset(AbstractRequestObject):
             ret = self._crs
         return ret
 
+    @property
+    def driver(self):
+        """
+        Get the driver to use for field creation.
+
+        :return: :class:`~ocgis.Field`
+        """
+        return self._driver
+
     @property
     def level_range(self):
         return self._level_range.value
@@ -280,6 +313,10 @@ class RequestDataset(AbstractRequestObject):
         self._dimension_map.update_dimensions_from_metadata(self.metadata)
         return self._dimension_map
 
+    @property
+    def driver_kwargs(self):
+        return self._driver_kwargs
+
     @property
     def field_name(self):
         if self._field_name is None:
@@ -316,8 +353,15 @@ class RequestDataset(AbstractRequestObject):
         if self._metadata is None:
             metadata = self.driver.metadata_raw
             self._metadata = deepcopy(metadata)
+            if self._rename_variable is not None:
+                for k, v in self.rename_variable_map.items():
+                    self._metadata['variables'][k]['name'] = v
         return self._metadata
 
+    @property
+    def opened(self):
+        return self._opened
+
     @property
     def time_range(self):
         return self._time_range.value
@@ -330,11 +374,6 @@ class RequestDataset(AbstractRequestObject):
             ret = get_first_or_tuple(list(get_iter(self._rename_variable)))
         return ret
 
-    @rename_variable.setter
-    def rename_variable(self, value):
-        value = get_tuple(value)
-        self._rename_variable = value
-
     @property
     def rename_variable_map(self):
         ret = {}
@@ -366,7 +405,6 @@ class RequestDataset(AbstractRequestObject):
 
     @property
     def time_subset_func(self):
-        # tdk: implement
         return self._time_subset_func.value
 
     @time_subset_func.setter
@@ -397,14 +435,13 @@ class RequestDataset(AbstractRequestObject):
             for v, u in zip(get_tuple(self.variable), value):
                 m[v]['attrs']['units'] = u
 
-    # tdk: remove
-    @property
-    def _units(self):
-        raise NotImplementedError
-
     @property
     def uri(self):
-        return get_first_or_tuple(self._uri)
+        if self._uri is None:
+            ret = self._uri
+        else:
+            ret = get_first_or_tuple(self._uri)
+        return ret
 
     @property
     def variable(self):
@@ -423,10 +460,7 @@ class RequestDataset(AbstractRequestObject):
 
         return ret
 
-    def get(self, *args, **kwargs):
-        return self.get_field(*args, **kwargs)
-
-    def get_field(self, *args, **kwargs):
+    def create_field(self, *args, **kwargs):
         """
         :rtype: :class:`~ocgis.interface.base.Field`
         """
@@ -441,13 +475,22 @@ class RequestDataset(AbstractRequestObject):
             except NoDataVariablesFound:
                 name = constants.MiscName.DEFAULT_FIELD_NAME
             kwargs['name'] = name
-        return self.driver.get_field(*args, **kwargs)
+        return self.driver.create_field(*args, **kwargs)
 
-    def get_variable_collection(self, **kwargs):
+    def create_raw_field(self, **kwargs):
         """
-        :rtype: `VariableCollection`
+        Return a raw field with no metadata interpretation.
+
+        :rtype: :class:`~ocgis.Field`
         """
-        return self.driver.get_variable_collection(**kwargs)
+        return self.driver.create_raw_field(**kwargs)
+
+    def get(self, *args, **kwargs):
+        return self.get_field(*args, **kwargs)
+
+    def get_field(self, *args, **kwargs):
+        """Here for backwards compatibility."""
+        return self.create_field(*args, **kwargs)
 
     def inspect(self):
         """
@@ -466,8 +509,12 @@ class RequestDataset(AbstractRequestObject):
         else:
             lr = '{0} to {1} (inclusive)'.format(self.level_range[0], self.level_range[1])
 
+        try:
+            data_variable = self.variable
+        except NoDataVariablesFound:
+            data_variable = None
         rows = ['    URI: {0}'.format(self.uri),
-                '    Variable: {0}'.format(self.variable),
+                '    Data Variable(s): {0}'.format(data_variable),
                 '    Time Range: {0}'.format(tr),
                 '    Time Region/Selection: {0}'.format(self.time_region),
                 '    Level Range: {0}'.format(lr)]
