@@ -144,12 +144,13 @@ class OcgOperations(AbstractOcgisObject):
         self.prefix = Prefix(prefix or env.PREFIX)
         self.output_format = OutputFormat(output_format)
         self.output_format_options = OutputFormatOptions(output_format_options)
-        self.agg_selection = AggregateSelection(agg_selection, output_format=self.output_format)
+        self.agg_selection = agg_selection  # AggregateSelection(agg_selection, output_format=self.output_format)
         self.geom_select_sql_where = GeomSelectSqlWhere(geom_select_sql_where)
         self.geom_select_uid = GeomSelectUid(geom_select_uid or select_ugid)
         self.geom_uid = GeomUid(geom_uid)
         self.geom = Geom(geom, select_ugid=self.geom_select_uid, geom_uid=self.geom_uid,
-                         geom_select_sql_where=self.geom_select_sql_where, union=self.agg_selection)
+                         geom_select_sql_where=self.geom_select_sql_where, union=self.agg_selection,
+                         output_format_options=self.output_format_options)
         self.vector_wrap = VectorWrap(vector_wrap)
         self.allow_empty = AllowEmpty(allow_empty)
         self.dir_output = DirOutput(dir_output or env.DIR_OUTPUT)
@@ -372,7 +373,7 @@ class OcgOperations(AbstractOcgisObject):
                 ocgis_lh(level=logging.WARN, msg=msg, logger='operations')
                 self._get_object_('output_crs')._value = Spherical()
 
-        # Only WGS84 coordinate system may be written to GeoJSON.
+        # Only Spherical coordinate systems may be written to GeoJSON.
         if self.output_format == constants.OutputFormatName.GEOJSON:
             msg = 'Only data with a WGS84 or Spherical projection may be written to GeoJSON.'
             if self.output_crs is not None:
