@@ -1,17 +1,19 @@
 import logging
 from copy import deepcopy
 
-from ocgis import Variable, vm
 from ocgis import env, constants
+from ocgis import vm
 from ocgis.base import raise_if_empty, AbstractOcgisObject
 from ocgis.calc.engine import CalculationEngine
 from ocgis.collection.field import Field
 from ocgis.collection.spatial import SpatialCollection
-from ocgis.constants import WrappedState, HeaderName, WrapAction, SubcommName
-from ocgis.exc import ExtentError, EmptySubsetError, BoundsAlreadyAvailableError, SubcommNotFoundError
+from ocgis.constants import WrappedState, HeaderName, WrapAction, SubcommName, KeywordArgument
+from ocgis.exc import ExtentError, EmptySubsetError, BoundsAlreadyAvailableError, SubcommNotFoundError, \
+    NoDataVariablesFound, WrappedStateEvalTargetMissing
 from ocgis.spatial.spatial_subset import SpatialSubsetOperation
 from ocgis.util.helpers import get_default_or_apply
 from ocgis.util.logging_ocgis import ocgis_lh, ProgressOcgOperations
+from ocgis.variable.base import create_typed_variable_from_data_model
 from ocgis.variable.crs import CFRotatedPole, Spherical, WGS84
 
 
@@ -172,6 +174,9 @@ class OperationsEngine(AbstractOcgisObject):
             except TypeError:
                 # The alias is used for logging, etc. If it cannot be constructed easily, leave it as None.
                 alias = None
+        except NoDataVariablesFound:
+            # If an alias is not provided and there are no data variables, set to None as this is used only for logging.
+            alias = None
 
         ocgis_lh('processing...', self._subset_log, alias=alias, level=logging.DEBUG)
         # Create the field object. Field objects may be passed directly to operations.
@@ -588,7 +593,7 @@ def _update_aggregation_wrapping_crs_(obj, alias, sfield, subset_sdim, subset_ug
         ocgis_lh('after sfield.geom.get_unioned in _update_aggregation_wrapping_crs_', obj._subset_log, alias=alias,
                  ugid=subset_ugid, level=logging.DEBUG)
 
-        # None is returned for the non-root process. Check we are in parallel and create and empty field.
+        # None is returned for the non-root process. Check we are in parallel and create an empty field.
         if sfield is None:
             if vm.size == 1:
                 raise ValueError('None should not be returned from get_unioned if running on a single processor.')
@@ -601,8 +606,10 @@ def _update_aggregation_wrapping_crs_(obj, alias, sfield, subset_sdim, subset_ug
 
         if not vm.is_null and subset_sdim is not None and subset_sdim.geom is not None:
             # Add the unique geometry identifier variable. This should match the selection geometry's identifier.
-            new_gid_variable = Variable(name=HeaderName.ID_GEOMETRY, value=subset_sdim.geom.ugid.get_value(),
-                                        dimensions=sfield.geom.dimensions)
+            new_gid_variable_kwargs = dict(name=HeaderName.ID_GEOMETRY, value=subset_sdim.geom.ugid.get_value(),
+                                           dimensions=sfield.geom.dimensions)
+            dm = get_data_model(obj.ops)
+            new_gid_variable = create_typed_variable_from_data_model('int', data_model=dm, **new_gid_variable_kwargs)
             sfield.geom.set_ugid(new_gid_variable)
 
     if vm.is_null:
@@ -613,7 +620,11 @@ def _update_aggregation_wrapping_crs_(obj, alias, sfield, subset_sdim, subset_ug
     raise_if_empty(sfield)
     ocgis_lh(msg='before wrapped_state in _update_aggregation_wrapping_crs_', logger=obj._subset_log,
              level=logging.DEBUG)
-    wrapped_state = sfield.wrapped_state
+    try:
+        wrapped_state = sfield.wrapped_state
+    except WrappedStateEvalTargetMissing:
+        # If there is no target for wrapping evaluation, then consider this unknown.
+        wrapped_state = WrappedState.UNKNOWN
     ocgis_lh(msg='after wrapped_state in _update_aggregation_wrapping_crs_', logger=obj._subset_log,
              level=logging.DEBUG)
 
@@ -635,7 +646,10 @@ def _update_aggregation_wrapping_crs_(obj, alias, sfield, subset_sdim, subset_ug
                 geom = sfield.geom
                 copied_geom = geom.get_value().copy()
                 geom.set_value(copied_geom)
-                geom.wrap()
+                # Some grids do not play nicely with wrapping. Bounds may be less than zero for an unwrapped grid.
+                # Force wrapping if it is requested. Normally, when force is false there is a pass-through that will
+                # leave the data untouched.
+                geom.wrap(force=True)
                 ocgis_lh('finished wrapping output geometries', obj._subset_log, alias=alias, ugid=subset_ugid,
                          level=logging.DEBUG)
 
@@ -679,3 +693,18 @@ def _update_wrapping_(obj, field_object):
                     field_object.wrap()
                 else:
                     field_object.unwrap()
+
+
+def get_data_model(ops):
+    if ops.output_format_options is None:
+        ret = None
+    else:
+        ret = ops.output_format_options.get(KeywordArgument.DATA_MODEL)
+
+    if ret is None:
+        ret = ops._get_object_('dataset')
+        if ret.data_model is not None:
+            ret = ret.data_model[0]
+        else:
+            ret = None
+    return ret
