@@ -19,12 +19,14 @@ import numpy as np
 import six
 from shapely import wkt
 from shapely.geometry import Point
+from shapely.geometry.base import BaseMultipartGeometry
 
 from ocgis import RequestDataset
 from ocgis import SourcedVariable
 from ocgis import Variable, Dimension
 from ocgis import env
 from ocgis.collection.field import Field
+from ocgis.constants import GridAbstraction, KeywordArgument
 from ocgis.spatial.geom_cabinet import GeomCabinet
 from ocgis.spatial.grid import Grid, get_geometry_variable
 from ocgis.util.helpers import get_iter, pprint_dict, get_bounds_from_1d, get_date_list, create_exact_field_value
@@ -45,8 +47,9 @@ Definitions for various "attrs":
  * data: test requires a data file
  * icclim: test requires ICCLIM
  * benchmark: test used for benchmarking/performance
+ * cli: test related to the command line interface. requires click as a dependency.
 
-nosetests -vs --with-id -a '!slow,!remote' ocgisf
+nosetests -vs --with-id -a '!slow,!remote' ocgis
 """
 
 
@@ -93,7 +96,7 @@ class TestBase(unittest.TestCase):
         self.assertSetEqual(set(sequence1), set(sequence2), msg=msg)
 
     def assertDescriptivesAlmostEqual(self, desired_descriptives, actual_arr):
-        actual_descriptives = self.get_descriptive_statistics(actual_arr)
+        actual_descriptives = self.create_array_descriptive_statistics(actual_arr)
         for k, v in list(desired_descriptives.items()):
             if k == 'shape':
                 self.assertEqual(v, actual_descriptives['shape'])
@@ -181,7 +184,8 @@ class TestBase(unittest.TestCase):
                 data_to_check = (arr1.data, arr2.data)
             else:
                 data_to_check = (arr1, arr2)
-            self.assertTrue(np.all(arr1.mask == arr2.mask))
+            if check_arr_type:
+                self.assertTrue(np.all(arr1.mask == arr2.mask))
             if check_fill_value:
                 self.assertEqual(arr1.fill_value, arr2.fill_value)
         else:
@@ -231,7 +235,6 @@ class TestBase(unittest.TestCase):
             self.assertEqual(set(src.dimensions.keys()), set(dest.dimensions.keys()))
 
             for varname, var in src.variables.items():
-
                 if varname in ignore_variables:
                     continue
 
@@ -253,7 +256,7 @@ class TestBase(unittest.TestCase):
                                 self.assertNumpyAllClose(var_value, dvar_value)
                             else:
                                 self.assertNumpyAll(var_value, dvar_value, check_arr_dtype=check_types,
-                                                    check_fill_value=check_fill_value)
+                                                    check_fill_value=check_fill_value, check_arr_type=check_types)
                 except (AssertionError, AttributeError):
                     # Some zero-length netCDF variables should not be tested for value equality. Values are meaningless
                     # and only the attributes should be tested for equality.
@@ -387,13 +390,72 @@ class TestBase(unittest.TestCase):
         else:
             raise AssertionError('Arrays are equivalent within precision.')
 
+    def assertPolygonSimilar(self, lhs, rhs, places=6, check_type=True):
+        """
+        Assert two polygons are similar using a variety of methods. The two polygons could still differ. This avoids
+        checking exact coordinates and ordering.
+
+        :param lhs: The actual polygon.
+        :type lhs: :class:`~shapely.geometry.polygon.Polygon`
+        :param lhs: The desired polygon.
+        :type lhs: :class:`~shapely.geometry.polygon.Polygon`
+        :param int places: Numpy of decimal places to use for the fuzzy comparison.
+        :param bool check_type: If ``True``, compare the types of the inputs.
+        """
+
+        self.assertAlmostEqual(lhs.area, rhs.area, places=places)
+        self.assertAlmostEqual(lhs.intersection(rhs).area, lhs.area, places=places)
+        if check_type:
+            self.assertEqual(type(lhs), type(rhs))
+        if check_type and (isinstance(lhs, BaseMultipartGeometry) or isinstance(rhs, BaseMultipartGeometry)):
+            self.assertEqual(len(lhs), len(rhs))
+            for l, r in zip(lhs, rhs):
+                self.assertAlmostEqual(l.area, r.area, places=places)
+
     def assertWarns(self, warning, meth):
         with warnings.catch_warnings(record=True) as warning_list:
             warnings.simplefilter('always')
             meth()
             self.assertTrue(any(item.category == warning for item in warning_list))
 
-    def get_descriptive_statistics(self, arr):
+    def assertWeightFilesEquivalent(self, global_weights_filename, merged_weights_filename):
+        """Assert weight files are equivalent."""
+
+        nwf = RequestDataset(merged_weights_filename).get()
+        gwf = RequestDataset(global_weights_filename).get()
+        nwf_row = nwf['row'].get_value()
+        gwf_row = gwf['row'].get_value()
+        self.assertAsSetEqual(nwf_row, gwf_row)
+        nwf_col = nwf['col'].get_value()
+        gwf_col = gwf['col'].get_value()
+        self.assertAsSetEqual(nwf_col, gwf_col)
+        nwf_S = nwf['S'].get_value()
+        gwf_S = gwf['S'].get_value()
+        self.assertEqual(nwf_S.sum(), gwf_S.sum())
+        unique_src = np.unique(nwf_row)
+        diffs = []
+        for us in unique_src.flat:
+            nwf_S_idx = np.where(nwf_row == us)[0]
+            nwf_col_sub = nwf_col[nwf_S_idx]
+            nwf_S_sub = nwf_S[nwf_S_idx].sum()
+
+            gwf_S_idx = np.where(gwf_row == us)[0]
+            gwf_col_sub = gwf_col[gwf_S_idx]
+            gwf_S_sub = gwf_S[gwf_S_idx].sum()
+
+            self.assertAsSetEqual(nwf_col_sub, gwf_col_sub)
+
+            diffs.append(nwf_S_sub - gwf_S_sub)
+        diffs = np.abs(diffs)
+        self.assertLess(diffs.max(), 1e-14)
+
+    @staticmethod
+    def barrier_print(*args, **kwargs):
+        from ocgis.vmachine.mpi import barrier_print
+        barrier_print(*args, **kwargs)
+
+    @staticmethod
+    def create_array_descriptive_statistics(arr):
         ret = {'mean': arr.mean(),
                'min': arr.min(),
                'max': arr.max(),
@@ -470,7 +532,8 @@ class TestBase(unittest.TestCase):
                 value_temporal.append(start)
                 start += delta
                 ctr += 1
-        temporal = TemporalVariable(value=value_temporal, dimensions='time', name='time')
+        tdim = Dimension('time')
+        temporal = TemporalVariable(value=value_temporal, dimensions=tdim, name='time')
         variable_shape.append(temporal.shape[0])
         variable_dimensions.append('time')
 
@@ -689,9 +752,12 @@ class TestBase(unittest.TestCase):
             path = os.path.split(path)[0]
         subprocess.call(['nautilus', path])
 
-    def ncdump(self, path, header_only=True):
+    def ncdump(self, path, header_only=True, variable=None):
         cmd = ['ncdump']
-        if header_only:
+        if variable is not None:
+            cmd.append('-v')
+            cmd.append(variable)
+        if header_only and variable is None:
             cmd.append('-h')
         cmd.append(path)
         subprocess.check_call(cmd)
@@ -719,6 +785,10 @@ class TestBase(unittest.TestCase):
     def print_scope(self):
         return print_scope()
 
+    def rank_print(self, *args, **kwargs):
+        from ocgis.vmachine.mpi import rank_print
+        rank_print(*args, **kwargs)
+
     def setUp(self):
         from ocgis import vm
         vm.__init__()
@@ -967,7 +1037,7 @@ class AbstractTestInterface(TestBase):
         return var
 
     def get_gridxy(self, with_2d_variables=False, crs=None, with_xy_bounds=False, with_value_mask=False,
-                   with_parent=False):
+                   with_parent=False, abstraction=GridAbstraction.AUTO):
 
         dest_mpi = OcgDist()
         dest_mpi.create_dimension('xdim', 3)
@@ -1002,6 +1072,7 @@ class AbstractTestInterface(TestBase):
             if with_xy_bounds:
                 vx.set_extrapolated_bounds('xbounds', 'bounds')
                 vy.set_extrapolated_bounds('ybounds', 'bounds')
+                dest_mpi.add_dimension(vx.bounds.dimensions[-1])
 
             if with_parent:
                 np.random.seed(1)
@@ -1024,6 +1095,7 @@ class AbstractTestInterface(TestBase):
         if with_parent:
             parent = variable_collection_scatter(parent, dest_mpi)
             kwds['parent'] = parent
+        kwds[KeywordArgument.ABSTRACTION] = abstraction
 
         grid = Grid(svx, svy, **kwds)
 
@@ -1152,7 +1224,8 @@ class AbstractTestField(TestBase):
             elif month_count == 2:
                 temporal_stop = dt(2000, 2, 29, 12)
             else:
-                raise NotImplementedError
+                temporal_stop = dt(2000 + month_count // 12,
+                                   1 + month_count % 12, 1, 12)
             temporal_value = get_date_list(temporal_start, temporal_stop, 1)
             delta_bounds = datetime.timedelta(hours=12)
             if with_bounds:
@@ -1267,3 +1340,15 @@ def create_gridxy_global(resolution=1.0, with_bounds=True, wrapped=True, crs=Non
         grid.set_extrapolated_bounds('xbounds', 'ybounds', 'bounds')
 
     return grid
+
+
+def get_geometry_dictionaries(uid='UGID'):
+    coordinates = [('France', [2.8, 47.16]),
+                   ('Germany', [10.5, 51.29]),
+                   ('Italy', [12.2, 43.4])]
+    geom = []
+    for ugid, coordinate in enumerate(coordinates, start=1):
+        point = Point(coordinate[1][0], coordinate[1][1])
+        geom.append({'geom': point,
+                     'properties': {uid: ugid, 'COUNTRY': coordinate[0]}})
+    return geom
