@@ -10,14 +10,19 @@ from shapely.geometry import Point
 from shapely.geometry import box
 from shapely.geometry.base import BaseGeometry
 
-from ocgis import RequestDataset, vm
+from ocgis import RequestDataset, vm, DimensionMap
 from ocgis import constants
-from ocgis.collection.field import Field
-from ocgis.constants import HeaderName, KeywordArgument, DriverKey, DimensionMapKey
+from ocgis.base import get_variable_names
+from ocgis.collection.field import Field, get_name_mapping
+from ocgis.collection.spatial import SpatialCollection
+from ocgis.constants import HeaderName, KeywordArgument, DriverKey, DimensionMapKey, DMK
+from ocgis.conv.nc import NcConverter
 from ocgis.driver.csv_ import DriverCSV
 from ocgis.driver.nc import DriverNetcdf
 from ocgis.driver.vector import DriverVector
-from ocgis.spatial.grid import Grid, create_grid_mask_variable
+from ocgis.spatial.base import create_spatial_mask_variable
+from ocgis.spatial.geom_cabinet import GeomCabinetIterator
+from ocgis.spatial.grid import Grid
 from ocgis.test.base import attr, AbstractTestInterface
 from ocgis.util.helpers import reduce_multiply
 from ocgis.variable.base import Variable
@@ -69,6 +74,11 @@ class TestField(AbstractTestInterface):
         actual = f.dimension_map.get_dimension(DimensionMapKey.GEOM)
         self.assertEqual(actual, ['the_geom_dim'])
 
+        # Test dimension map does not have any entries at initialization.
+        actual = Field()
+        desired = actual.dimension_map.as_dict()
+        self.assertEqual(len(desired), 0)
+
     def test_system_crs_and_grid_abstraction(self):
         f = Field(grid_abstraction='point')
         grid = self.get_gridxy(with_xy_bounds=True)
@@ -95,6 +105,49 @@ class TestField(AbstractTestInterface):
         actual = field.dimension_map.get_dimension('time')
         self.assertEqual(actual, ['the_time'])
 
+    def test_system_add_variable(self):
+        """Test adding variables from spatial collections."""
+
+        # Create a few separate fields.
+        variable_names = tuple(['a', 'b', 'c'])
+        fields = [self.get_field(variable_name=v) for v in variable_names]
+        # Create spatial collections containing those fields.
+        scs = []
+        for field in fields:
+            sc = SpatialCollection()
+            sc.add_field(field, None)
+            scs.append(sc)
+
+        # Destination spatial collection to add variables to from source spatial collections.
+        grow = scs[0]
+        # Loop over source fields.
+        for idx in range(1, len(scs)):
+            # Loop over child fields and spatial containers in the current source spatial collection.
+            for field, container in scs[idx].iter_fields(yield_container=True):
+                # TODO: This should be adjusted to allow easier selection with empty fields.
+                try:
+                    # Case when we have spatial containers.
+                    grow_field = grow.get_element(field_name=field.name, container_ugid=container)
+                except KeyError:
+                    # Case without spatial containers.
+                    grow_field = grow.get_element(field.name)
+                # Add data variables to the grow field.
+                for dv in field.data_variables:
+                    grow_field.add_variable(dv.extract(), is_data=True)
+
+        # Assert all variables are present on the grow field.
+        actual = grow.get_element()
+        self.assertEqual(get_variable_names(actual.data_variables), variable_names)
+
+        # Write the spatial collection using a converter.
+        conv = NcConverter([grow], outdir=self.current_dir_output, prefix='out.nc')
+        conv.write()
+
+        # Assert all variables are present.
+        rd = RequestDataset(conv.path)
+        actual = rd.get()
+        self.assertEqual(get_variable_names(actual.data_variables), variable_names)
+
     def test_system_properties(self):
         """Test field properties."""
 
@@ -203,6 +256,15 @@ class TestField(AbstractTestInterface):
         field = Field(variables=[crs])
         self.assertEqual(len(field.dimensions), 0)
 
+    @attr('data')
+    def test_from_records(self):
+        gci = GeomCabinetIterator(path=self.path_state_boundaries)
+        actual = Field.from_records(gci, data_model='NETCDF3_CLASSIC')
+        desired = {'UGID': np.int32,
+                   'ID': np.int32}
+        for v in desired.keys():
+            self.assertEqual(actual[v].get_value().dtype, desired[v])
+
     def test_get_by_tag(self):
         v1 = Variable(name='tas')
         v2 = Variable(name='tasmax')
@@ -212,19 +274,47 @@ class TestField(AbstractTestInterface):
         t = field.get_by_tag('other')
         self.assertAsSetEqual([ii.name for ii in t], tags['other'])
 
+    def test_get_name_mapping(self):
+        dimension_map = {'crs': {'variable': 'latitude_longitude'}, 'level': {'variable': None, 'dimension': []},
+                         'time': {'variable': u'time', 'attrs': {'axis': 'T'}, 'bounds': u'time_bnds',
+                                  'dimension': [u'time']}, 'driver': 'netcdf-cf', 'spatial_mask': {'variable': None},
+                         'groups': {}, 'realization': {'variable': None, 'dimension': []},
+                         'y': {'variable': u'lat', 'attrs': {}, 'bounds': u'lat_vertices', 'dimension': [u'rlat']},
+                         'x': {'variable': u'lon', 'attrs': {}, 'bounds': u'lon_vertices', 'dimension': [u'rlon']}}
+        dimension_map = DimensionMap.from_dict(dimension_map)
+
+        actual = get_name_mapping(dimension_map)
+        desired = {'y': [u'rlat'], 'x': [u'rlon'], 'time': [u'time']}
+        self.assertEqual(actual, desired)
+
     def test_grid(self):
         # Test mask variable information is propagated through property.
-        grid = self.get_gridxy()
+        grid = self.get_gridxy(with_xy_bounds=True)
+        self.assertTrue(grid.is_vectorized)
+        self.assertTrue(grid.has_bounds)
         np.random.seed(1)
         value = np.random.rand(*grid.shape)
         select = value > 0.4
-        mask_var = create_grid_mask_variable('nonstandard', select, grid.dimensions)
+        mask_var = create_spatial_mask_variable('nonstandard', select, grid.dimensions)
         grid.set_mask(mask_var)
         field = Field(grid=grid)
+        self.assertTrue(field.grid.has_bounds)
         self.assertEqual(field.dimension_map.get_spatial_mask(), mask_var.name)
-        # field.dimension_map.pprint()
         self.assertNumpyAll(field.grid.get_mask(), mask_var.get_mask())
 
+        # Test dimension map bounds are updated appropriately.
+        dim = Dimension('count', 2)
+        x = Variable(name='x', value=[1., 2.], dimensions=dim)
+        y = Variable(name='y', value=[1., 2.], dimensions=dim)
+        xb = Variable(name='xb', value=[[0., 1.5], [1.5, 2.5]], dimensions=[dim, 'bounds'])
+        yb = Variable(name='yb', value=[[0., 1.5], [1.5, 2.5]], dimensions=[dim, 'bounds'])
+        variables = [x, y, xb, yb]
+        dmap = DimensionMap()
+        dmap.set_variable(DMK.X, x, bounds=xb)
+        dmap.set_variable(DMK.Y, y, bounds=yb)
+        f = Field(dimension_map=dmap, variables=variables)
+        self.assertTrue(f.grid.has_bounds)
+
     def test_iter(self):
         field = self.get_ocgfield_example()
         field.set_geom(field.grid.get_abstraction_geometry())
@@ -232,7 +322,7 @@ class TestField(AbstractTestInterface):
 
         geom2 = field.geom.deepcopy()
         geom2.set_name('geom2')
-        geom2.extract()
+        geom2 = geom2.extract()
         field.add_variable(geom2)
         self.assertIn(geom2.name, field)
 
@@ -294,11 +384,33 @@ class TestField(AbstractTestInterface):
 
         tas2 = field['tas'].deepcopy()
         tas2.set_name('tas2')
-        tas2.extract()
+        tas2 = tas2.extract()
 
         field.add_variable(tas2, is_data=True)
         self.assertIsNotNone(list(field.iter(melted=True)))
 
+    def test_set_geom(self):
+        f = Field()
+        self.assertIsNone(f.crs)
+
+        g = GeometryVariable(value=[Point(1, 2)], dimensions='geom', crs=Spherical())
+
+        f.set_geom(g)
+
+    def test_set_x(self):
+        f = Field()
+        var = Variable('x', value=[1, 2], dimensions='xdim')
+        f.set_x(var, 'xdim')
+
+        var2 = Variable('x2', value=[3, 4], dimensions='xdim2')
+        f.set_x(var2, 'xdim2')
+
+        self.assertNotIn(var.name, f)
+
+        f.set_x(None, None)
+        self.assertEqual(len(f), 0)
+        self.assertIsNone(f.x)
+
     def test_time(self):
         units = [None, 'days since 2012-1-1']
         calendar = [None, '365_day']
@@ -316,7 +428,7 @@ class TestField(AbstractTestInterface):
             var = k.variable_type(name='time', value=value, attrs=attrs, dimensions=['one'])
             bounds_var = k.bounds_variable_type(name='time_bnds', value=bounds, dimensions=['one', 'two'])
             f = Field(variables=[var, bounds_var], dimension_map=dimension_map)
-            self.assertTrue(len(f.dimension_map._storage) > 1)
+            self.assertTrue(len(f.dimension_map._storage) == 1)
             self.assertTrue(f.time.has_bounds)
             self.assertIsInstance(f.time, TemporalVariable)
             self.assertIsInstance(f.time.bounds, TemporalVariable)
