@@ -1,40 +1,113 @@
 import pickle
+from collections import OrderedDict
 from copy import deepcopy
+from unittest import SkipTest
 
 import fiona
 import numpy as np
-from shapely.geometry.geo import shape
+from mock import mock
+from shapely.geometry.geo import shape, box
 
 from ocgis import GeomCabinet, vm
 from ocgis import RequestDataset
 from ocgis import env
 from ocgis.base import get_variable_names
 from ocgis.collection.field import Field
-from ocgis.constants import DimensionMapKey
-from ocgis.driver.base import iter_all_group_keys
+from ocgis.constants import DimensionMapKey, DMK, KeywordArgument, MPIWriteMode
+from ocgis.driver.base import iter_all_group_keys, driver_scope
 from ocgis.driver.dimension_map import DimensionMap
-from ocgis.driver.nc import DriverNetcdf, DriverNetcdfCF
+from ocgis.driver.nc import DriverNetcdf, DriverNetcdfCF, remove_netcdf_attribute, get_crs_variable
 from ocgis.exc import OcgWarning, CannotFormatTimeError, \
     NoDataVariablesFound
+from ocgis.ops.core import OcgOperations
 from ocgis.spatial.grid import Grid
 from ocgis.test.base import TestBase, attr, create_gridxy_global
+from ocgis.util.addict import Dict
 from ocgis.util.helpers import get_group
 from ocgis.variable.base import Variable, ObjectType, VariableCollection, SourcedVariable
-from ocgis.variable.crs import WGS84, CoordinateReferenceSystem, CFSpherical
+from ocgis.variable.crs import WGS84, CoordinateReferenceSystem, CFSpherical, CFRotatedPole
 from ocgis.variable.dimension import Dimension
 from ocgis.variable.geom import GeometryVariable
 from ocgis.variable.temporal import TemporalVariable
 from ocgis.vmachine.mpi import MPI_RANK, MPI_COMM, OcgDist, variable_scatter
 
 
+class Test(TestBase):
+    def test_get_crs_variable(self):
+        # Test with no linking to data variables.
+        metadata = {'variables': {'rotated_pole': {'dimensions': (),
+                                                   'dtype': np.dtype('S1'),
+                                                   'fill_value': 'auto',
+                                                   'name': u'rotated_pole',
+                                                   'dtype_packed': None,
+                                                   'attrs': OrderedDict([(
+                                                       u'grid_mapping_name',
+                                                       u'rotated_latitude_longitude'),
+                                                       (
+                                                           u'grid_north_pole_latitude',
+                                                           90.0),
+                                                       (
+                                                           u'grid_north_pole_longitude',
+                                                           360.0)]),
+                                                   'fill_value_packed': None}}}
+        var = get_crs_variable(metadata, False)
+        self.assertIsInstance(var, CFRotatedPole)
+
+    def test_get_variable_value(self):
+        path1 = self.get_temporary_file_path('f1.nc')
+        path2 = self.get_temporary_file_path('f2.nc')
+
+        tdim = Dimension('time')
+        t1 = TemporalVariable(value=[1, 2, 3, 4, 5], dimensions=tdim, dtype=np.int32)
+        k = {KeywordArgument.DATASET_KWARGS: {KeywordArgument.FORMAT: 'NETCDF4_CLASSIC'}}
+        t1.parent.write(path1, **k)
+        t2 = TemporalVariable(value=[6, 7, 8, 9, 10], dimensions=tdim, dtype=np.int32)
+        t2.parent.write(path2, **k)
+
+        rd = RequestDataset([path1, path2])
+        tin = rd.get().time
+        sub = tin[2:8]
+        self.assertEqual(sub.get_value().tolist(), list(range(3, 9)))
+
+    def test_remove_netcdf_attribute(self):
+        path = self.get_temporary_file_path('foo.nc')
+        var = Variable(name='test', attrs={'remove_me': 10})
+        var.write(path)
+
+        remove_netcdf_attribute(path, var.name, 'remove_me')
+
+        with self.nc_scope(path) as ds:
+            actual = ds.variables[var.name]
+            self.assertFalse(hasattr(actual, 'remove_me'))
+
+
 class TestDriverNetcdf(TestBase):
+    def create_rank_valued_netcdf(self):
+        rank_size = 10
+        size_global = vm.size_global
+        with vm.scoped('write rank netcdf', [0]):
+            if not vm.is_null:
+                path = self.get_temporary_file_path('dist_desired.nc')
+                dim = Dimension('dist_dim', rank_size * size_global)
+                var = Variable(name='data', dimensions=dim, attrs={'hi': 5})
+                for rank in range(size_global):
+                    value = np.ones(rank_size) + (10 * (rank + 1))
+                    bounds = (rank_size * rank, rank_size * rank + rank_size)
+                    var.get_value()[bounds[0]: bounds[1]] = value
+                var.parent.attrs = {'hi_dataset_level': 'whee'}
+                var.write(path)
+            else:
+                path = None
+        path = vm.bcast(path)
+        return path
+
     def test_init(self):
         path = self.get_temporary_file_path('foo.nc')
         with self.nc_scope(path, 'w') as ds:
             ds.createDimension('a', 2)
         rd = RequestDataset(uri=path, driver='netcdf')
         self.assertIsInstance(rd.driver, DriverNetcdf)
-        vc = rd.get_variable_collection()
+        vc = rd.create_raw_field()
         self.assertEqual(len(vc), 0)
 
     def test_system_changing_field_name(self):
@@ -52,7 +125,7 @@ class TestDriverNetcdf(TestBase):
 
         rd = RequestDataset(path1)
         # rd.inspect()
-        nvc = rd.get_variable_collection()
+        nvc = rd.create_raw_field()
         nvc2 = nvc.children['vc2']
         self.assertIsNone(nvc2['var2']._value)
         self.assertEqual(nvc2.name, 'vc2')
@@ -63,10 +136,73 @@ class TestDriverNetcdf(TestBase):
         nvc.write(path2)
         rd2 = RequestDataset(path2)
         # rd2.inspect()
-        n2vc = rd2.get_variable_collection()
+        n2vc = rd2.create_raw_field()
         self.assertEqual(n2vc.children[nvc2.name].name, nvc2.name)
 
-    def test_get_dist(self):
+    def test_system_concatenating_files(self):
+        field = self.get_field(ntime=5, nrlz=0, nlevel=0)
+        paths = []
+        for tidx in range(field.time.shape[0]):
+            sub = field.get_field_slice({'time': tidx})
+            path = self.get_temporary_file_path('time_subset_{}.nc'.format(tidx))
+            paths.append(path)
+            sub.write(path, dataset_kwargs={'format': 'NETCDF4_CLASSIC'})
+
+        rd = RequestDataset(paths)
+        ops = OcgOperations(dataset=rd, output_format='nc')
+        ret = ops.execute()
+        actual_field = RequestDataset(ret).get()
+        actual = actual_field.data_variables[0].get_value()
+        self.assertNumpyAll(actual, field.data_variables[0].get_value())
+        self.assertNumpyAll(actual_field.time.value_numtime, field.time.value_numtime)
+
+    @mock.patch('netCDF4.Dataset')
+    @mock.patch('netCDF4.MFDataset')
+    @mock.patch('ocgis.RequestDataset', spec_set=True)
+    def test_system_driver_kwargs(self, m_RequestDataset, m_MFDataset, m_Dataset):
+        m_RequestDataset.driver_kwargs = {'clobber': False}
+        m_RequestDataset.opened = None
+        uri = 'a/path/foo.nc'
+        m_RequestDataset.uri = uri
+
+        driver = DriverNetcdf(m_RequestDataset)
+        driver.inquire_opened_state = mock.Mock(return_value=False)
+
+        with driver_scope(driver) as _:
+            m_Dataset.assert_called_once_with(uri, mode='r', clobber=False)
+
+        m_RequestDataset.driver_kwargs = {'aggdim': 'not_time'}
+        m_RequestDataset.opened = None
+        uri = ['a/path/foo1.nc', 'a/path/foo2.nc']
+        m_RequestDataset.uri = uri
+        with driver_scope(driver) as _:
+            m_MFDataset.assert_called_once_with(uri, aggdim='not_time')
+
+    def test_system_renaming_dimensions_on_variables(self):
+        var1 = Variable(name='var1', value=[1, 2], dimensions='dim')
+        var2 = Variable(name='var2', value=[3, 4], dimensions='dim')
+        vc = VariableCollection(variables=[var1, var2])
+        path = self.get_temporary_file_path('out.nc')
+        vc.write(path)
+
+        rd = RequestDataset(path)
+        meta = rd.metadata
+        meta['dimensions']['new_dim'] = {'size': 2}
+        meta['variables']['var2']['dimensions'] = ('new_dim',)
+
+        field = rd.get()
+        self.assertEqual(field['var2'].dimensions[0].name, 'new_dim')
+        self.assertEqual(field['var2'].get_value().tolist(), [3, 4])
+
+        path2 = self.get_temporary_file_path('out2.nc')
+        field.write(path2)
+
+        rd2 = RequestDataset(path2)
+        field2 = rd2.get()
+        self.assertEqual(field2['var2'].dimensions[0].name, 'new_dim')
+        self.assertEqual(field2['var2'].get_value().tolist(), [3, 4])
+
+    def test_create_dist(self):
 
         def _create_dimensions_(ds, k):
             if k.dim_count > 0:
@@ -92,7 +228,7 @@ class TestDriverNetcdf(TestBase):
             rd = RequestDataset(uri=path)
             driver = DriverNetcdf(rd)
 
-            actual = driver.get_dist().mapping
+            actual = driver.create_dist().mapping
 
             # All dimensions are not distributed.
             for keyseq in iter_all_group_keys(actual[MPI_RANK]):
@@ -102,16 +238,13 @@ class TestDriverNetcdf(TestBase):
 
             if k.dim_count == 0 and k.nested:
                 desired = {None: {'variables': {}, 'dimensions': {}, 'groups': {
-                    'nest1': {'variables': {}, 'dimensions': {}, 'groups': {
-                        'nest2': {'variables': {}, 'dimensions': {}, 'groups': {'nest1': {'variables': {},
-                                                                                          'dimensions': {
-                                                                                              'outlier': Dimension(
-                                                                                                  name='outlier',
-                                                                                                  size=4,
-                                                                                                  size_current=4,
-                                                                                                  dist=False,
-                                                                                                  src_idx='auto')},
-                                                                                          'groups': {}}}}}}}}}
+                    u'nest1': {'variables': {}, 'dimensions': {}, 'groups': {
+                        u'nest2': {'variables': {}, 'dimensions': {},
+                                   'groups': {u'nest3': {'variables': {}, 'dimensions': {}, 'groups': {}},
+                                              u'nest1': {'variables': {}, 'dimensions': {
+                                                  u'outlier': Dimension(name='outlier', size=4, size_current=4,
+                                                                        dist=False, is_empty=False, src_idx='auto')},
+                                                         'groups': {}}}}}}}}}
                 self.assertEqual(actual[MPI_RANK], desired)
 
             if k.dim_count == 2 and k.nested:
@@ -182,6 +315,40 @@ class TestDriverNetcdf(TestBase):
                    '    } // group: group1_group1', '  } // group: group1', '}']
         self.assertEqual(lines, desired)
 
+    @attr('data')
+    def test_create_field(self):
+        # test updating of regrid source flag
+        rd = self.test_data.get_rd('cancm4_tas')
+        driver = DriverNetcdf(rd)
+        field = driver.create_field()
+        self.assertTrue(field.regrid_source)
+        rd.regrid_source = False
+        driver = DriverNetcdf(rd)
+        field = driver.create_field()
+        self.assertFalse(field.regrid_source)
+
+        # test flag with an assigned coordinate system
+        rd = self.test_data.get_rd('cancm4_tas')
+        driver = DriverNetcdf(rd)
+        field = driver.create_field()
+        self.assertFalse(field._has_assigned_coordinate_system)
+
+        rd = self.test_data.get_rd('cancm4_tas', kwds={'crs': WGS84()})
+        self.assertTrue(rd._has_assigned_coordinate_system)
+        driver = DriverNetcdf(rd)
+        field = driver.create_field()
+        self.assertTrue(field._has_assigned_coordinate_system)
+
+    @mock.patch('ocgis.OcgVM')
+    def test_get_write_modes(self, m_vm):
+        # Test asynchronous write mode is chosen.
+        m_vm.size = 2
+        env.USE_NETCDF4_MPI = True
+        kwds = {KeywordArgument.DATASET_KWARGS: {'parallel': True}}
+        actual = DriverNetcdf._get_write_modes_(m_vm, **kwds)
+        desired = [MPIWriteMode.ASYNCHRONOUS]
+        self.assertEqual(actual, desired)
+
     def test_open(self):
         # Test with a multi-file dataset.
         path1 = self.get_temporary_file_path('foo1.nc')
@@ -193,9 +360,17 @@ class TestDriverNetcdf(TestBase):
                 b[:] = idx
         uri = [path1, path2]
         rd = RequestDataset(uri=uri, driver=DriverNetcdf)
-        field = rd.get_variable_collection()
+        field = rd.create_raw_field()
         self.assertEqual(field['b'].get_value().tolist(), [0, 1])
 
+    @mock.patch('netCDF4.Dataset')
+    @mock.patch('ocgis.OcgVM')
+    def test_open_parallel_netcdf4_python(self, m_vm, m_Dataset):
+        m_vm.size = 2
+        env.USE_NETCDF4_MPI = True
+        _ = DriverNetcdf._open_('a/path', mode='w', vm=m_vm)
+        m_Dataset.assert_called_with('a/path', mode='w', parallel=True, comm=m_vm.comm)
+
     def test_write_variable(self):
         path = self.get_temporary_file_path('foo.nc')
         var = Variable(name='height', value=10.0, dimensions=[])
@@ -227,13 +402,13 @@ class TestDriverNetcdf(TestBase):
                 var.foo = 'bar'
         else:
             path_in, path_out = [None] * 2
-        path_in = MPI_COMM.bcast(path_in)
-        path_out = MPI_COMM.bcast(path_out)
+        path_in = vm.bcast(path_in)
+        path_out = vm.bcast(path_out)
 
         rd = RequestDataset(path_in)
         rd.metadata['dimensions']['seven']['dist'] = True
         driver = DriverNetcdf(rd)
-        vc = driver.get_variable_collection()
+        vc = driver.create_raw_field()
         with vm.scoped_by_emptyable('write', vc):
             if not vm.is_null:
                 vc.write(path_out)
@@ -254,10 +429,43 @@ class TestDriverNetcdf(TestBase):
 
         rd = RequestDataset(path_in)
         driver = DriverNetcdf(rd)
-        vc = driver.get_variable_collection()
+        vc = driver.create_raw_field()
         vc.write(path_out, dataset_kwargs={'format': 'NETCDF3_CLASSIC'}, variable_kwargs={'zlib': True})
 
-        self.assertNcEqual(path_in, path_out)
+        self.assertNcEqual(path_in, path_out, ignore_attributes={'var_seven': ['_FillValue']})
+
+    @attr('mpi')
+    def test_write_variable_collection_netcdf4_mpi(self):
+        # TODO: TEST: Test writing a grouped netCDF file in parallel.
+
+        self.add_barrier = False
+        if not env.USE_NETCDF4_MPI:
+            raise SkipTest('not env.USE_NETCDF4_MPI')
+        path = self.create_rank_valued_netcdf()
+
+        # if vm.rank == 0:
+        #     self.ncdump(path, header_only=False)
+
+        rd = RequestDataset(path, driver='netcdf')
+        rd.metadata['dimensions']['dist_dim']['dist'] = True
+
+        field = rd.get()
+
+        # self.barrier_print(field['data'].get_value())
+
+        if vm.rank == 0:
+            actual_path = self.get_temporary_file_path('actual_mpi.nc')
+        else:
+            actual_path = None
+        actual_path = vm.bcast(actual_path)
+
+        # self.barrier_print('before field.write')
+        field.write(actual_path)
+        # self.barrier_print('after field.write')
+
+        if vm.rank == 0:
+            # self.ncdump(actual_path, header_only=False)
+            self.assertNcEqual(actual_path, path)
 
     @attr('mpi')
     def test_write_variable_collection_object_arrays(self):
@@ -294,6 +502,69 @@ class TestDriverNetcdf(TestBase):
 
 
 class TestDriverNetcdfCF(TestBase):
+    @property
+    def fixture_rotated_spherical_metadata(self):
+        ret = {'dimensions': {u'rlon': {'name': u'rlon', 'isunlimited': False, 'size': 194},
+                              u'bnds': {'name': u'bnds', 'isunlimited': False, 'size': 2},
+                              u'rlat': {'name': u'rlat', 'isunlimited': False, 'size': 201},
+                              u'time': {'name': u'time', 'isunlimited': True, 'size': 1826}},
+               'global_attributes': {u'model_id': u'DMI-HIRHAM5', u'rcm_version_id': u'v2', u'project_id': u'CORDEX',
+                                     u'driving_experiment_name': u'rcp45',
+                                     u'institution': u'Danish Meteorological Institute',
+                                     u'driving_experiment': u'ICHEC-EC-EARTH,rcp45,r3i1p1',
+                                     u'CDO': u'Climate Data Operators version 1.4.0.1 (http://www.mpimet.mpg.de/cdo)',
+                                     u'CDI': u'Climate Data Interface version 1.4.0.1', u'contact': u'obc@dmi.dk',
+                                     u'product': u'output', u'CORDEX_domain': u'AFR-44',
+                                     u'experiment': u'Scenario experiment with ICHEC-EC-EARTH forcing',
+                                     u'frequency': u'day', u'driving_model_ensemble_member': u'r3i1p1',
+                                     u'experiment_id': u'rcp45', u'NCO': u'4.0.9',
+                                     u'creation_date': u'2014-07-11 23:53:54', u'Conventions': u'CF-1.6',
+                                     u'driving_model_id': u'ICHEC-EC-EARTH',
+                                     u'tracking_id': u'9f58ce4e-0956-11e4-a2fa-6c626dd8513d', u'institute_id': u'DMI'},
+               'variables': {
+                   u'time_bnds': {'dimensions': (u'time', u'bnds'), 'dtype': np.dtype('float64'), 'fill_value': 'auto',
+                                  'fill_value_packed': None, 'dtype_packed': None,
+                                  'attrs': {u'units': u'days since 1949-12-01 00:00:00',
+                                            u'calendar': u'proleptic_gregorian'}, 'name': u'time_bnds'},
+                   u'tas': {'dimensions': (u'time', u'rlat', u'rlon'), 'dtype': np.dtype('float32'),
+                            'fill_value': 1e+20,
+                            'fill_value_packed': None, 'dtype_packed': None,
+                            'attrs': {u'_FillValue': 1e+20, u'coordinates': u'lon lat height',
+                                      u'long_name': u'Near-Surface Air Temperature',
+                                      u'standard_name': u'air_temperature', u'cell_methods': u'time: mean',
+                                      u'units': u'K', u'missing_value': 1e+20}, 'name': u'tas'},
+                   u'rlon': {'dimensions': (u'rlon',), 'dtype': np.dtype('float64'), 'fill_value': 'auto',
+                             'fill_value_packed': None, 'dtype_packed': None,
+                             'attrs': {u'units': u'degrees', u'long_name': u'longitude in rotated pole grid',
+                                       u'standard_name': u'grid_longitude', u'axis': u'X'}, 'name': u'rlon'},
+                   u'lon': {'dimensions': (u'rlat', u'rlon'), 'dtype': np.dtype('float64'), 'fill_value': 'auto',
+                            'fill_value_packed': None, 'dtype_packed': None,
+                            'attrs': {u'units': u'degrees_east', u'_CoordinateAxisType': u'Lon',
+                                      u'standard_name': u'longitude', u'long_name': u'longitude'}, 'name': u'lon'},
+                   u'height': {'dimensions': (), 'dtype': np.dtype('float64'), 'fill_value': 'auto',
+                               'fill_value_packed': None, 'dtype_packed': None,
+                               'attrs': {u'units': u'm', u'long_name': u'height', u'standard_name': u'height',
+                                         u'axis': u'Z', u'positive': u'up'}, 'name': u'height'},
+                   u'rotated_pole': {'dimensions': (), 'dtype': np.dtype('S1'), 'fill_value': 'auto',
+                                     'fill_value_packed': None, 'dtype_packed': None,
+                                     'attrs': {u'grid_north_pole_latitude': 90.0, u'grid_north_pole_longitude': 360.0,
+                                               u'grid_mapping_name': u'rotated_latitude_longitude'},
+                                     'name': u'rotated_pole'},
+                   u'time': {'dimensions': (u'time',), 'dtype': np.dtype('float64'), 'fill_value': 'auto',
+                             'fill_value_packed': None, 'dtype_packed': None,
+                             'attrs': {u'units': u'days since 1949-12-01 00:00:00', u'calendar': u'proleptic_gregorian',
+                                       u'bounds': u'time_bnds'}, 'name': u'time'},
+                   u'lat': {'dimensions': (u'rlat', u'rlon'), 'dtype': np.dtype('float64'), 'fill_value': 'auto',
+                            'fill_value_packed': None, 'dtype_packed': None,
+                            'attrs': {u'units': u'degrees_north', u'_CoordinateAxisType': u'Lat',
+                                      u'standard_name': u'latitude', u'long_name': u'latitude'}, 'name': u'lat'},
+                   u'rlat': {'dimensions': (u'rlat',), 'dtype': np.dtype('float64'), 'fill_value': 'auto',
+                             'fill_value_packed': None, 'dtype_packed': None,
+                             'attrs': {u'units': u'degrees', u'long_name': u'latitude in rotated pole grid',
+                                       u'standard_name': u'grid_latitude', u'axis': u'Y'}, 'name': u'rlat'}},
+               'groups': {}, 'file_format': 'NETCDF4_CLASSIC'}
+        return ret
+
     def get_drivernetcdf(self, **kwargs):
         path = self.get_drivernetcdf_file_path()
         kwargs['uri'] = path
@@ -392,7 +663,7 @@ class TestDriverNetcdfCF(TestBase):
             path_out = self.get_temporary_file_path('foo.nc')
         else:
             path_out = None
-        path_out = MPI_COMM.bcast(path_out)
+        path_out = vm.bcast(path_out)
 
         rd = self.test_data.get_rd('cancm4_tas')
         rd.metadata['dimensions']['lat']['dist'] = True
@@ -406,7 +677,7 @@ class TestDriverNetcdfCF(TestBase):
             self.assertNcEqual(path_out, rd.uri, ignore_variables=['latitude_longitude'],
                                ignore_attributes=ignore_attributes)
 
-    def test_system_get_field_dimensioned_variables(self):
+    def test_system_create_field_dimensioned_variables(self):
         """Test data is appropriately tagged to identify dimensioned variables."""
 
         path = self.get_temporary_file_path('foo.nc')
@@ -432,6 +703,33 @@ class TestDriverNetcdfCF(TestBase):
         actual = get_variable_names(read_field.data_variables)
         self.assertEqual(actual, ('data2',))
 
+    def test_system_rotated_pole_spherical_subsetting(self):
+        """Test rotated pole coordinates are left alone during a subset (no mask applied)."""
+
+        def _run_mask_test_(target):
+            for vn in ['rlat', 'rlon']:
+                self.assertIsNone(target[vn].get_mask())
+
+        rd = RequestDataset(metadata=self.fixture_rotated_spherical_metadata)
+        field = rd.create_field()
+        _run_mask_test_(field)
+        for ctr, vn in enumerate(['lat', 'lon', 'rlat', 'rlon']):
+            var = field[vn]
+            var.get_value()[:] = np.arange(var.size).reshape(var.shape) + (ctr * 10)
+        path = self.get_temporary_file_path('foo.nc')
+        field.write(path)
+
+        new_field = RequestDataset(path).create_field()
+        _run_mask_test_(new_field)
+        subset_geom = box(*new_field.grid.extent)
+        subset_field = new_field.grid.get_intersects(subset_geom, optimized_bbox_subset=True).parent
+        _run_mask_test_(subset_field)
+
+        path2 = self.get_temporary_file_path('foo2.nc')
+        subset_field.write(path2)
+        in_subset_field = RequestDataset(path2).create_field()
+        _run_mask_test_(in_subset_field)
+
     def test_get_data_variable_names(self):
         driver = self.get_drivernetcdf()
         dvars = driver.get_data_variable_names(driver.rd.metadata, driver.rd.dimension_map)
@@ -452,12 +750,13 @@ class TestDriverNetcdfCF(TestBase):
         with self.assertRaises(NoDataVariablesFound):
             assert driver.rd.variable
 
-    def test_get_dimension_map(self):
+    def test_create_dimension_map(self):
         d = self.get_drivernetcdf()
-        dmap = d.get_dimension_map(d.metadata_source, strict=True)
+        dmap = d.create_dimension_map(d.metadata_source, strict=True)
         desired = {'crs': {'variable': 'latitude_longitude'},
                    'time': {'variable': 'time', 'bounds': 'time_bounds', DimensionMapKey.DIMENSION: ['time'],
-                            DimensionMapKey.ATTRS: {'axis': 'T'}}}
+                            DimensionMapKey.ATTRS: {'axis': 'T'}},
+                   'driver': DriverNetcdfCF.key}
         self.assertEqual(dmap.as_dict(), desired)
 
         def _run_():
@@ -466,7 +765,7 @@ class TestDriverNetcdfCF(TestBase):
                                             'attrs': {'axis': 'X', 'bounds': 'x_bounds'},
                                             'dimensions': ('xx',)}},
                         'dimensions': {'xx': {'name': 'xx', 'size': None}}}
-            d.get_dimension_map(metadata)
+            d.create_dimension_map(metadata)
 
         self.assertWarns(OcgWarning, _run_)
 
@@ -476,25 +775,18 @@ class TestDriverNetcdfCF(TestBase):
         actual = driver.rd.dimension_map
         self.assertEqual(actual.get_variable(DimensionMapKey.LEVEL), 'does_not_exist')
         # The driver dimension map always loads from the data.
-        self.assertNotEqual(dm, driver.get_dimension_map(driver.metadata_source))
-        with self.assertRaises(ValueError):
-            driver.get_field()
+        self.assertNotEqual(dm, driver.create_dimension_map(driver.metadata_source))
+        actual = driver.create_field().dimension_map
+        self.assertEqual(actual.get_variable(DMK.LEVEL), dm['level']['variable'])
 
         # Test a dimension name is converted to a list.
         dmap = {DimensionMapKey.TIME: {DimensionMapKey.VARIABLE: 'time', DimensionMapKey.DIMENSION: 'time'}}
         d = self.get_drivernetcdf(dimension_map=dmap)
-        f = d.get_field()
+        f = d.create_field()
         actual = f.dimension_map.get_dimension(DimensionMapKey.TIME)
         self.assertEqual(actual, ['time'])
 
-    def test_get_dimension_map_no_time_axis(self):
-        metadata = {'variables': {'time': {'name': 'time', 'attrs': {}, 'dimensions': ['time']}},
-                    'dimensions': {}}
-        d = self.get_drivernetcdf()
-        dmap = d.get_dimension_map(metadata)
-        self.assertEqual(dmap.get_variable(DimensionMapKey.TIME), 'time')
-
-    def test_get_dimension_map_2d_spatial_coordinates(self):
+    def test_create_dimension_map_2d_spatial_coordinates(self):
         grid = create_gridxy_global()
         grid.expand()
         path = self.get_temporary_file_path('foo.nc')
@@ -511,7 +803,28 @@ class TestDriverNetcdfCF(TestBase):
         actual = f.dimension_map.get_dimension(DimensionMapKey.X)
         self.assertEqual(actual, ['x'])
 
-    def test_get_dimension_map_with_spatial_mask(self):
+    def test_create_dimension_map_no_time_axis(self):
+        metadata = {'variables': {'time': {'name': 'time', 'attrs': {}, 'dimensions': ['time']}},
+                    'dimensions': {}}
+        d = self.get_drivernetcdf()
+        dmap = d.create_dimension_map(metadata)
+        self.assertEqual(dmap.get_variable(DimensionMapKey.TIME), 'time')
+
+    def test_create_dimension_map_rotated_spherical(self):
+        self.assertTrue(env.SET_GRID_AXIS_ATTRS)
+        self.assertIsNone(env.COORDSYS_ACTUAL)
+        rd = mock.create_autospec(RequestDataset)
+        rd._has_assigned_coordinate_system = False
+        rd.rotated_pole_priority = False
+
+        driver = DriverNetcdfCF(rd)
+        dmap = driver.create_dimension_map(self.fixture_rotated_spherical_metadata)
+        self.assertFalse(env.SET_GRID_AXIS_ATTRS)
+        self.assertIsInstance(env.COORDSYS_ACTUAL, CFRotatedPole)
+        self.assertEqual(dmap.get_variable(DMK.X), 'lon')
+        self.assertEqual(dmap.get_variable(DMK.Y), 'lat')
+
+    def test_create_dimension_map_with_spatial_mask(self):
         path = self.get_temporary_file_path('foo.nc')
         grid = create_gridxy_global()
         gmask = grid.get_mask(create=True)
@@ -520,7 +833,7 @@ class TestDriverNetcdfCF(TestBase):
         grid.parent.write(path)
         rd = RequestDataset(path)
         driver = DriverNetcdfCF(rd)
-        dmap = driver.get_dimension_map(driver.metadata_source)
+        dmap = driver.create_dimension_map(driver.metadata_source)
         self.assertIsNotNone(dmap.get_spatial_mask())
         field = rd.get()
         self.assertEqual(field.grid.get_mask().sum(), 1)
@@ -533,14 +846,39 @@ class TestDriverNetcdfCF(TestBase):
         field = rd.get()
         self.assertIsNone(field.grid.get_mask())
 
+    def test_get_crs(self):
+        group_metadata = self.fixture_rotated_spherical_metadata
+
+        rd = mock.create_autospec(RequestDataset)
+
+        keywords = Dict()
+        keywords.rpp = [True, False]
+        keywords.with_spherical = [True, False]
+
+        desired = Dict()
+        desired.rpp[True] = CFRotatedPole
+        desired.rpp[False] = CFSpherical
+
+        for k in self.iter_product_keywords(keywords):
+            cgm = deepcopy(group_metadata)
+            if not k.with_spherical:
+                cgm['variables'].pop('lat')
+                cgm['variables'].pop('lon')
+            rd.rotated_pole_priority = k.rpp
+            actual = DriverNetcdfCF(rd).get_crs(cgm)
+            if k.with_spherical:
+                self.assertIsInstance(actual, desired.rpp[k.rpp])
+            else:
+                self.assertIsInstance(actual, CFRotatedPole)
+
     def test_get_dump_report(self):
         d = self.get_drivernetcdf()
         r = d.get_dump_report()
         self.assertGreaterEqual(len(r), 24)
 
-    def test_get_field(self):
+    def test_create_field(self):
         driver = self.get_drivernetcdf()
-        field = driver.get_field(format_time=False)
+        field = driver.create_field(format_time=False)
         self.assertEqual(field.driver.key, driver.key)
         self.assertIsInstance(field.time, TemporalVariable)
         with self.assertRaises(CannotFormatTimeError):
@@ -557,7 +895,7 @@ class TestDriverNetcdfCF(TestBase):
         rd = RequestDataset(uri=path)
         driver = DriverNetcdfCF(rd)
         self.assertEqual(driver.get_crs(driver.metadata_source), CFSpherical())
-        self.assertEqual(driver.get_field().crs, CFSpherical())
+        self.assertEqual(driver.create_field().crs, CFSpherical())
 
         # Second, test the overloaded CRS is found.
         desired = CoordinateReferenceSystem(epsg=2136)
@@ -565,7 +903,7 @@ class TestDriverNetcdfCF(TestBase):
         self.assertEqual(rd.crs, desired)
         driver = DriverNetcdfCF(rd)
         self.assertEqual(driver.get_crs(driver.metadata_source), CFSpherical())
-        field = driver.get_field()
+        field = driver.create_field()
         self.assertEqual(field.crs, desired)
         # Test file coordinate system variable is removed.
         self.assertNotIn('latitude_longitude', field)
@@ -578,7 +916,7 @@ class TestDriverNetcdfCF(TestBase):
         driver = DriverNetcdfCF(rd)
         self.assertEqual(rd.crs, env.DEFAULT_COORDSYS)
         self.assertEqual(driver.get_crs(driver.rd.metadata), env.DEFAULT_COORDSYS)
-        self.assertEqual(driver.get_field().crs, env.DEFAULT_COORDSYS)
+        self.assertEqual(driver.create_field().crs, env.DEFAULT_COORDSYS)
 
     def test_get_field_write_target(self):
         # Test coordinate system names are added to attributes of dimensioned variables.
@@ -596,8 +934,8 @@ class TestDriverNetcdfCF(TestBase):
         self.assertEqual(field.x.units, 'degrees_east')
 
         # Test bounds units are removed when writing.
-        x = Variable(name='x', value=[1, 2, 3], dtype=float, dimensions='x', units='hours')
-        y = Variable(name='y', value=[1, 2, 3], dtype=float, dimensions='x', units='hours')
+        x = Variable(name='x', value=[1, 2, 3], dtype=float, dimensions='xdim', units='hours')
+        y = Variable(name='y', value=[1, 2, 3], dtype=float, dimensions='ydim', units='hours')
         grid = Grid(x, y)
         grid.set_extrapolated_bounds('x_bounds', 'y_bounds', 'bounds')
         self.assertEqual(x.bounds.units, x.units)
@@ -611,6 +949,20 @@ class TestDriverNetcdfCF(TestBase):
         self.assertEqual(x.bounds.units, x.units)
         self.assertEqual(y.bounds.units, y.units)
 
+        # Test actual coordinate system is triggered.
+        field = Field()
+        src = CFSpherical()
+        dst = WGS84()
+        field.set_crs(src)
+        self.assertEqual(field.crs, src)
+        self.assertIsNone(env.COORDSYS_ACTUAL)
+        env.COORDSYS_ACTUAL = dst
+        actual = DriverNetcdfCF._get_field_write_target_(field)
+        self.assertEqual(actual.crs, dst)
+        self.assertEqual(field.crs, src)
+        self.assertNotIn(src.name, actual)
+        self.assertIn(dst.name, actual)
+
     def test_metadata_raw(self):
         d = self.get_drivernetcdf()
         metadata = d.metadata_raw
