@@ -10,11 +10,11 @@ import numpy as np
 from shapely.geometry import Point, LineString
 
 import ocgis
-from ocgis import RequestDataset, vm
+from ocgis import RequestDataset, vm, Dimension
 from ocgis import constants
 from ocgis import env
 from ocgis.collection.field import Field
-from ocgis.constants import WrappedState, HeaderName, OutputFormatName, DMK
+from ocgis.constants import WrappedState, HeaderName, OutputFormatName, DMK, KeywordArgument
 from ocgis.exc import DefinitionValidationError
 from ocgis.ops.core import OcgOperations
 from ocgis.ops.parms import definition
@@ -23,28 +23,32 @@ from ocgis.spatial.geom_cabinet import GeomCabinetIterator, GeomCabinet
 from ocgis.spatial.grid import Grid
 from ocgis.test.base import TestBase, attr, create_gridxy_global, create_exact_field
 from ocgis.test.test_simple.test_dependencies import create_mftime_nc_files
-from ocgis.util.addict import Dict
-from ocgis.util.helpers import make_poly, create_exact_field_value
+from ocgis.util.helpers import make_poly
 from ocgis.variable.base import Variable
-from ocgis.variable.crs import Spherical, CoordinateReferenceSystem, WGS84
+from ocgis.variable.crs import Spherical, CoordinateReferenceSystem, WGS84, CRS
 from ocgis.variable.temporal import TemporalVariable
-from ocgis.vmachine.mpi import OcgDist, MPI_RANK, variable_collection_scatter, MPI_COMM, dgather, \
-    hgather, MPI_SIZE
+from ocgis.vmachine.mpi import OcgDist, MPI_RANK, variable_collection_scatter, MPI_COMM
 
 
 class TestOcgOperations(TestBase):
-    def setUp(self):
-        TestBase.setUp(self)
 
+    @property
+    def datasets(self):
         rds = [self.test_data.get_rd('cancm4_tasmin_2001'), self.test_data.get_rd('cancm4_tasmax_2011'),
                self.test_data.get_rd('cancm4_tas')]
 
         time_range = [dt(2000, 1, 1), dt(2000, 12, 31)]
         level_range = [2, 2]
 
-        self.datasets = [{'uri': rd.uri, 'variable': rd.variable, 'time_range': time_range, 'level_range': level_range}
-                         for rd in rds]
-        self.datasets_no_range = [{'uri': rd.uri, 'variable': rd.variable} for rd in rds]
+        return [{'uri': rd.uri, 'variable': rd.variable, 'time_range': time_range, 'level_range': level_range} for rd in
+                rds]
+
+    @property
+    def datasets_no_range(self):
+        rds = [self.test_data.get_rd('cancm4_tasmin_2001'), self.test_data.get_rd('cancm4_tasmax_2011'),
+               self.test_data.get_rd('cancm4_tas')]
+
+        return [{'uri': rd.uri, 'variable': rd.variable} for rd in rds]
 
     @attr('data')
     def test_init(self):
@@ -83,6 +87,117 @@ class TestOcgOperations(TestBase):
         self.assertTrue(str(ret).startswith('OcgOperations'))
         self.assertGreater(len(ret), 500)
 
+    def test_system_coordinates_with_time_dimension(self):
+        """Test a field built on coordinate variables with a time dimension."""
+
+        env.SET_GRID_AXIS_ATTRS = False
+
+        lat = Variable(name='XLAT', value=[1, 2, 3], dtype=float, dimensions='north_south')
+        lon = Variable(name='XLONG', value=[10, 20, 30, 40], dtype=float, dimensions='west_east')
+        grid = Grid(y=lat, x=lon)
+        grid.expand()
+
+        time = Dimension('Time', size_current=1)
+        dlat, dlon = lat.dimensions
+
+        for v in [lat, lon]:
+            v.reshape([time, dlat, dlon])
+
+        t2 = Variable(name='T2', dtype=float, dimensions=[time, dlat, dlon])
+        t2.get_value()[:] = np.arange(np.prod(grid.shape)).reshape(*grid.shape)
+        grid.parent.add_variable(t2)
+
+        self.assertEqual(grid.shape, (dlat.size, dlon.size))
+
+        path = self.get_temporary_file_path('foo.nc')
+        grid.parent.write(path, driver='netcdf')
+
+        lat_variable = 'XLAT'
+        lon_variable = 'XLONG'
+        lat_dimension = 'south_north'
+        lon_dimension = 'west_east'
+
+        env.SET_GRID_AXIS_ATTRS = True
+
+        # Load the field from disk and modify it before passing it to operations. Operations can take a request dataset
+        # a field.
+        field = RequestDataset(path, variable='T2').get()
+        # Remove the time dimension from the spatial coordinate variables.
+        for cv in [lat_variable, lon_variable]:
+            var = field[cv]
+            new_dimensions = var.dimensions[1:]
+            var.reshape(new_dimensions)
+        # Update the field's metadata so it knows where the x/y coordinate variables are.
+        field.set_x(field['XLONG'], dimension=lon_dimension)
+        field.set_y(field['XLAT'], dimension=lat_dimension)
+        # Create and set the coordinate system.
+        lcc = CRS(
+            proj4='+proj=lcc +a=6370000.0m +b=6370000.0m +lon_0=97.00w +lat_1=33.00n +lat_2=45.00n +lat_0=40.00n +units=m +no_defs',
+            name='rpolcc')
+        field.set_crs(lcc)
+
+        # Run operations and write to netCDF.
+        geom = [{'geom': Point(20, 2), 'crs': lcc}]
+        ops = OcgOperations(dataset=field, geom=geom, output_format='nc')
+        _ = ops.execute()
+
+    def test_system_data_model_is_consistent(self):
+        """Test data model does not change when running through operations."""
+
+        env.OVERWRITE = True
+
+        formats = [{0: 'NETCDF3_CLASSIC',
+                    1: 'NETCDF3_CLASSIC'},
+                   {0: 'NETCDF3_CLASSIC',
+                    1: 'NETCDF3_64BIT_OFFSET'}
+                   ]
+
+        grid = create_gridxy_global()
+        field = create_exact_field(grid, 'foo')
+        path1 = self.get_temporary_file_path('out1.nc')
+        path2 = self.get_temporary_file_path('out2.nc')
+
+        for ii_f, f in enumerate(formats):
+            output_paths = []
+
+            desired_file_format = 'NETCDF3_CLASSIC'
+
+            paths = [path1, path2]
+            for ii, path in enumerate(paths):
+                k = {KeywordArgument.DATASET_KWARGS: {KeywordArgument.FORMAT: f[ii]}}
+                field.write(path, **k)
+
+            for ii, path in enumerate(paths):
+                rd = RequestDataset(path)
+                self.assertEqual(rd.metadata['file_format'], f[ii])
+
+            for ii, path in enumerate(paths):
+                rd = RequestDataset(path)
+                if ii_f == 0:
+                    ofo = None
+                else:
+                    ofo = {'data_model': desired_file_format}
+                ops = OcgOperations(dataset=rd, geom=[-80, 40, -60, 60], output_format='nc', prefix=str(ii),
+                                    output_format_options=ofo)
+                ret = ops.execute()
+                output_paths.append(ret)
+                actual = RequestDataset(ret)
+                self.assertEqual(actual.metadata['file_format'], desired_file_format)
+
+            rd = RequestDataset(output_paths)
+            self.assertAsSetEqual(rd.metadata['file_format'], [desired_file_format])
+
+    def test_system_field_integrity_following_calculation(self):
+        """Test outgoing field from operations passes parent validation."""
+
+        field = self.get_field(ntime=365 * 2)
+        field._validate_()
+        ret = ocgis.OcgOperations(field, calc=[{'func': 'max', 'name': 'max'}],
+                                  calc_grouping=['year', 'unique']).execute()
+        field_max = ret.get_element()
+        self.assertEqual(id(field_max['max'].parent), id(field_max))
+        field_max._validate_()
+
     @attr('data')
     def test_system_scalar_level_dimension(self):
         """Test scalar level dimensions are not dropped in netCDF output."""
@@ -148,6 +263,24 @@ class TestOcgOperations(TestBase):
             ret = ops.get_base_request_size()
             self.assertTrue(ret['total'] > 1)
 
+    @attr('data')
+    def test_get_base_request_size_time_range(self):
+        path = self.get_temporary_file_path('foo.nc')
+        field = self.get_field(ntime=60)
+        time_range = [field.time.get_value()[0], field.time.get_value()[1]]
+        field.write(path)
+        rd = RequestDataset(path)
+
+        ops = OcgOperations(dataset=rd)
+        size_no_tr = ops.get_base_request_size()
+        time_size_no_tr = size_no_tr['field']['foo']['time']['kb']
+
+        ops = OcgOperations(dataset=rd, time_range=time_range)
+        size_with_tr = ops.get_base_request_size()
+        time_size_with_tr = size_with_tr['field']['foo']['time']['kb']
+
+        self.assertAlmostEqual(time_size_no_tr - time_size_with_tr, 0.453125)
+
     @attr('data')
     def test_get_base_request_size_with_calculation(self):
         rd = self.test_data.get_rd('cancm4_tas')
@@ -183,30 +316,34 @@ class TestOcgOperations(TestBase):
         with self.assertRaises(DefinitionValidationError):
             kk('pt')
 
-    @attr('data')
+    @attr('data', 'slow')
     def test_keyword_aggregate(self):
-        rd = self.test_data.get_rd('rotated_pole_cnrm_cerfacs')
+        for rpp in [False, True]:
+            kwds = {KeywordArgument.ROTATED_POLE_PRIORITY: rpp}
+            rd = self.test_data.get_rd('rotated_pole_cnrm_cerfacs', kwds=kwds)
 
-        ofield = rd.get().get_field_slice({'time': slice(0, 10), 'y': slice(0, 10), 'x': slice(0, 10)})
-        ovalue = ofield['pr'].get_value()
-        manual_mean = ovalue[4, :, :].mean()
+            ofield = rd.get().get_field_slice({'time': slice(0, 10), 'y': slice(0, 10), 'x': slice(0, 10)})
+            ovalue = ofield['pr'].get_value()
+            manual_mean = ovalue[4, :, :].mean()
 
-        slc = [None, [0, 10], None, [0, 10], [0, 10]]
-        for output_format in [OutputFormatName.OCGIS, OutputFormatName.CSV]:
-            ops = OcgOperations(dataset=rd, output_format=output_format, aggregate=True, slice=slc, melted=True)
-            # Spatial operations on rotated pole require the output be spherical.
-            self.assertEqual(ops.output_crs, Spherical())
-            ret = ops.execute()
-            if output_format == constants.OutputFormatName.OCGIS:
-                field = ret.get_element()
-                self.assertEqual(field['pr'].shape, (10, 1))
-                self.assertAlmostEqual(float(manual_mean), float(field['pr'].get_value()[4]))
-            else:
-                with open(ret, 'r') as f:
-                    reader = csv.DictReader(f)
-                    rows = list(reader)
-                self.assertEqual(len(rows), 10)
-                self.assertAlmostEqual(float(rows[4]['VALUE']), manual_mean)
+            slc = [None, [0, 10], None, [0, 10], [0, 10]]
+            for output_format in [OutputFormatName.OCGIS, OutputFormatName.CSV]:
+                ops = OcgOperations(dataset=rd, output_format=output_format, aggregate=True, slice=slc, melted=True,
+                                    prefix='rpp_{}'.format(rpp))
+                # Spatial operations on rotated pole require the output be spherical.
+                if rpp:
+                    self.assertEqual(ops.output_crs, Spherical())
+                ret = ops.execute()
+                if output_format == constants.OutputFormatName.OCGIS:
+                    field = ret.get_element()
+                    self.assertEqual(field['pr'].shape, (10, 1))
+                    self.assertAlmostEqual(float(manual_mean), float(field['pr'].get_value()[4]))
+                else:
+                    with open(ret, 'r') as f:
+                        reader = csv.DictReader(f)
+                        rows = list(reader)
+                    self.assertEqual(len(rows), 10)
+                    self.assertAlmostEqual(float(rows[4]['VALUE']), manual_mean)
 
     @attr('data')
     def test_keyword_calc_grouping_none_date_parts(self):
@@ -337,7 +474,7 @@ class TestOcgOperations(TestBase):
 
         from ocgis.regrid.base import get_esmf_field_from_ocgis_field
 
-        efield = get_esmf_field_from_ocgis_field(self.get_field(nrlz=0, nlevel=0))
+        efield = get_esmf_field_from_ocgis_field(self.get_field(nrlz=0, nlevel=0, crs=Spherical()))
         output_format = OutputFormat.iter_possible()
         for kk in output_format:
             # Geojson may only be written with a spherical coordinate system.
@@ -416,7 +553,7 @@ class TestOcgOperations(TestBase):
         ops.execute()
         self.assertEqual(len(os.listdir(self.current_dir_output)), 0)
 
-    @attr('esmf', 'data', 'slow')
+    @attr('esmf', 'data')
     def test_keyword_output_format_esmpy(self):
         """Test with the ESMPy output format."""
         import ESMF
@@ -424,22 +561,22 @@ class TestOcgOperations(TestBase):
         slc = [None, None, None, [0, 10], [0, 10]]
         kwds = dict(as_field=[False, True],
                     with_slice=[True, False])
-        for k in self.iter_product_keywords(kwds):
+        for ctr, k in enumerate(self.iter_product_keywords(kwds)):
             rd = self.test_data.get_rd('cancm4_tas')
             if k.as_field:
                 rd = rd.get()
             if k.with_slice:
-                slc = slc
+                the_slc = slc
             else:
-                slc = None
-            ops = OcgOperations(dataset=rd, output_format='esmpy', slice=slc)
+                the_slc = None
+            ops = OcgOperations(dataset=rd, output_format='esmpy', slice=the_slc)
             ret = ops.execute()
             self.assertIsInstance(ret, ESMF.Field)
             try:
-                self.assertEqual(ret.data.shape, (3650, 10, 10))
+                self.assertEqual(ret.data.shape, (10, 10, 3650))
             except AssertionError:
                 self.assertFalse(k.with_slice)
-                self.assertEqual(ret.data.shape, (3650, 64, 128))
+                self.assertEqual(ret.data.shape, (128, 64, 3650))
 
     @attr('data')
     def test_keyword_output_format_nc_package_validation_raised_first(self):
@@ -485,6 +622,9 @@ class TestOcgOperations(TestBase):
     def test_keyword_regrid_destination_to_nc(self):
         """Write regridded data to netCDF."""
 
+        ocgis.env.VERBOSE = True
+        ocgis.env.DEBUG = True
+
         rd1 = self.test_data.get_rd('cancm4_tas')
         rd2 = self.test_data.get_rd('cancm4_tas')
 
@@ -667,6 +807,16 @@ class TestOcgOperationsNoData(TestBase):
 
         return field
 
+    def test_system_calculation_variable_naming(self):
+        """Test the same data variable name may be used for the calculation."""
+
+        field = self.get_field()
+        calc = [{'func': 'mean', 'name': field.data_variables[0].name}]
+        ops = OcgOperations(dataset=field, calc=calc, calc_grouping=['month'])
+        ret = ops.execute()
+        out_field = ret.get_element()
+        self.assertIn(field.data_variables[0].name, out_field)
+
     def test_system_dataset_identifiers_on_variables(self):
         """Test dataset identifiers make it to output variables for iteration."""
 
@@ -749,6 +899,30 @@ class TestOcgOperationsNoData(TestBase):
         shp_field = RequestDataset(shp_path).get()
         self.assertIn(gid_name, list(shp_field.keys()))
 
+    def test_system_geometry_identifier_typed_appropriately(self):
+        """Test GID is typed appropriately according to the data model."""
+
+        ofo = {'data_model': 'NETCDF3_64BIT_OFFSET'}
+        grid = create_gridxy_global(resolution=3.0)
+        field = create_exact_field(grid, 'foo', crs=Spherical())
+        ops = OcgOperations(dataset=field, output_format_options=ofo, geom=[-100, 30, -90, 40], aggregate=True)
+        actual = ops.execute()
+        actual = actual.get_element(container_ugid=1)
+        self.assertEqual(actual.geom.ugid.dtype, np.int32)
+
+        # Test data model is retrieved appropriately from file.
+        ofo = {'data_model': 'NETCDF3_64BIT_OFFSET'}
+        grid = create_gridxy_global(resolution=3.0)
+        field = create_exact_field(grid, 'foo', crs=Spherical())
+        ops = OcgOperations(dataset=field, output_format_options=ofo, output_format='nc')
+        ret = ops.execute()
+        rd = RequestDataset(uri=ret)
+
+        ops = OcgOperations(dataset=rd, geom=[-100, 30, -90, 40], aggregate=True)
+        actual = ops.execute()
+        actual = actual.get_element(container_ugid=1)
+        self.assertEqual(actual.geom.ugid.dtype, np.int32)
+
     def test_system_line_subsetting(self):
         """Test subsetting with a line."""
 
@@ -822,6 +996,22 @@ class TestOcgOperationsNoData(TestBase):
                 actual = actual == (idx + 1) * 10
                 self.assertTrue(np.all(actual))
 
+    def test_system_netcdf_output_format(self):
+        path = self.get_temporary_file_path('foo.nc')
+        var = Variable('vec', value=[1, 2, 3, 4, 5], dimensions='dvec', dtype=np.int32)
+        var.write(path)
+
+        with self.nc_scope(path, 'r') as ds:
+            self.assertEqual(ds.data_model, 'NETCDF4')
+
+        rd = RequestDataset(uri=path)
+        ops = OcgOperations(dataset=rd, prefix='converted', output_format='nc',
+                            output_format_options={'data_model': 'NETCDF4_CLASSIC'})
+        ret = ops.execute()
+
+        with self.nc_scope(ret, 'r') as ds:
+            self.assertEqual(ds.data_model, 'NETCDF4_CLASSIC')
+
     @attr('cfunits')
     def test_system_request_dataset_modifiers(self):
         """
@@ -881,6 +1071,17 @@ class TestOcgOperationsNoData(TestBase):
         shp_field = RequestDataset(shp_path).get()
         self.assertIn(ugid_name, list(shp_field.keys()))
 
+    def test_system_user_geometry_identifier_typed_appropriately(self):
+        """Test UGID is typed appropriately according to the data model."""
+
+        ofo = {'data_model': 'NETCDF3_64BIT_OFFSET'}
+        grid = create_gridxy_global(resolution=3.0)
+        field = create_exact_field(grid, 'foo', crs=Spherical())
+        ops = OcgOperations(dataset=field, output_format_options=ofo, geom=[-100, 30, -90, 40], aggregate=True)
+        actual = ops.execute()
+        actual = actual[1]
+        self.assertEqual(actual.geom.ugid.dtype, np.int32)
+
     @attr('mpi')
     def test_system_spatial_averaging_through_operations(self):
         data_name = 'data'
@@ -918,134 +1119,6 @@ class TestOcgOperationsNoData(TestBase):
                 actual = out_field.data_variables[0].get_value()[0]
                 self.assertEqual(actual, desired)
 
-    @attr('release')
-    def test_system_spatial_averaging_through_operations_state_boundaries(self):
-        if MPI_SIZE != 8:
-            raise SkipTest('MPI_SIZE != 8')
-
-        ntime = 3
-        # Get the exact field value for the state's representative center.
-        with vm.scoped([0]):
-            if MPI_RANK == 0:
-                states = RequestDataset(self.path_state_boundaries, driver='vector').get()
-                states.update_crs(env.DEFAULT_COORDSYS)
-                fill = np.zeros((states.geom.shape[0], 2))
-                for idx, geom in enumerate(states.geom.get_value().flat):
-                    centroid = geom.centroid
-                    fill[idx, :] = centroid.x, centroid.y
-                exact_states = create_exact_field_value(fill[:, 0], fill[:, 1])
-                state_ugid = states['UGID'].get_value()
-                area = states.geom.area
-
-        keywords = {
-            'spatial_operation': [
-                'clip',
-                'intersects'
-            ],
-            'aggregate': [
-                True,
-                False
-            ],
-            'wrapped': [True, False],
-            'output_format': [
-                OutputFormatName.OCGIS,
-                'csv',
-                'csv-shp',
-                'shp'
-            ],
-        }
-
-        # total_iterations = len(list(self.iter_product_keywords(keywords)))
-
-        for ctr, k in enumerate(self.iter_product_keywords(keywords)):
-            # barrier_print(k)
-            # if ctr % 1 == 0:
-            #     if vm.is_root:
-            #         print('Iteration {} of {}...'.format(ctr + 1, total_iterations))
-
-            with vm.scoped([0]):
-                if vm.is_root:
-                    grid = create_gridxy_global(resolution=1.0, dist=False, wrapped=k.wrapped)
-                    field = create_exact_field(grid, 'foo', ntime=ntime)
-                    path = self.get_temporary_file_path('foo.nc')
-                    field.write(path)
-                else:
-                    path = None
-            path = MPI_COMM.bcast(path)
-
-            rd = RequestDataset(path)
-
-            ops = OcgOperations(dataset=rd, geom='state_boundaries', spatial_operation=k.spatial_operation,
-                                aggregate=k.aggregate, output_format=k.output_format, prefix=str(ctr),
-                                # geom_select_uid=[8]
-                                )
-            ret = ops.execute()
-
-            # Test area is preserved for a problem element during union. The union's geometry was not fully represented
-            # in the output.
-            if k.output_format == 'shp' and k.aggregate and k.spatial_operation == 'clip':
-                with vm.scoped([0]):
-                    if vm.is_root:
-                        inn = RequestDataset(ret).get()
-                        inn_ugid_idx = np.where(inn['UGID'].get_value() == 8)[0][0]
-                        ugid_idx = np.where(state_ugid == 8)[0][0]
-                        self.assertAlmostEqual(inn.geom.get_value()[inn_ugid_idx].area, area[ugid_idx], places=2)
-
-            # Test the overview geometry shapefile is written.
-            if k.output_format == 'shp':
-                directory = os.path.split(ret)[0]
-                contents = os.listdir(directory)
-                actual = ['_ugid.shp' in c for c in contents]
-                self.assertTrue(any(actual))
-            elif k.output_format == 'csv-shp':
-                directory = os.path.split(ret)[0]
-                directory = os.path.join(directory, 'shp')
-                contents = os.listdir(directory)
-                actual = ['_ugid.shp' in c for c in contents]
-                self.assertTrue(any(actual))
-                if not k.aggregate:
-                    actual = ['_gid.shp' in c for c in contents]
-                    self.assertTrue(any(actual))
-
-            if k.output_format == OutputFormatName.OCGIS:
-                geom_keys = ret.children.keys()
-                all_geom_keys = vm.gather(np.array(geom_keys))
-                if vm.is_root:
-                    all_geom_keys = hgather(all_geom_keys)
-                    self.assertEqual(len(np.unique(all_geom_keys)), 51)
-
-                if k.aggregate:
-                    actual = Dict()
-                    for field, container in ret.iter_fields(yield_container=True):
-                        if not field.is_empty:
-                            ugid = container.geom.ugid.get_value()[0]
-                            actual[ugid]['actual'] = field.data_variables[0].get_value()
-                            actual[ugid]['area'] = container.geom.area[0]
-
-                    actual = vm.gather(actual)
-
-                    if vm.is_root:
-                        actual = dgather(actual)
-
-                        ares = []
-                        actual_areas = []
-                        for ugid_key, v in actual.items():
-                            ugid_idx = np.where(state_ugid == ugid_key)[0][0]
-                            desired = exact_states[ugid_idx]
-                            actual_areas.append(v['area'])
-                            for tidx in range(ntime):
-                                are = np.abs((desired + ((tidx + 1) * 10)) - v['actual'][tidx, 0])
-                                ares.append(are)
-
-                        if k.spatial_operation == 'clip':
-                            diff = np.abs(np.array(area) - np.array(actual_areas))
-                            self.assertLess(np.max(diff), 1e-6)
-                            self.assertLess(np.mean(diff), 1e-6)
-
-                        # Test relative errors.
-                        self.assertLess(np.max(ares), 0.031)
-                        self.assertLess(np.mean(ares), 0.009)
-
     @attr('mpi', 'no-3.5')
     def test_system_spatial_wrapping_and_reorder(self):
         if sys.version_info.major == 3 and sys.version_info.minor == 5:
