@@ -8,11 +8,11 @@ from ocgis.ops.parms.base import AbstractParameter
 from ocgis.ops.parms.definition import *
 from ocgis.ops.query import QueryInterface
 from ocgis.spatial.geom_cabinet import GeomCabinet
-from ocgis.test.base import TestBase, attr
+from ocgis.test.base import TestBase, attr, get_geometry_dictionaries
 from ocgis.util.helpers import make_poly
 from ocgis.util.itester import itr_products_keywords
 from ocgis.util.units import get_units_object, get_are_units_equal
-from ocgis.variable.crs import WGS84
+from ocgis.variable.crs import WGS84, Spherical
 
 
 class Test(TestBase):
@@ -439,7 +439,7 @@ class TestDataset(TestBase):
     def test_init_esmf(self):
         from ocgis.regrid.base import get_esmf_field_from_ocgis_field
 
-        original_field = self.get_field()
+        original_field = self.get_field(crs=Spherical())
         dimensions = original_field.data_variables[0].dimensions
         efield = get_esmf_field_from_ocgis_field(original_field)
         dd = Dataset(efield, esmf_field_dimensions=dimensions)
@@ -447,7 +447,6 @@ class TestDataset(TestBase):
         self.assertIsInstance(ofield, Field)
         dimensioned = ofield.get_by_tag(TagName.DATA_VARIABLES)[0]
         self.assertTrue(np.may_share_memory(dimensioned.get_value(), efield.data))
-        self.assertNumpyAll(dimensioned.get_value(), efield.data)
 
     @attr('data')
     def test_from_query(self):
@@ -498,18 +497,6 @@ class TestDataset(TestBase):
 class TestGeom(TestBase):
     create_dir = False
 
-    @staticmethod
-    def get_geometry_dictionaries(uid='UGID'):
-        coordinates = [('France', [2.8, 47.16]),
-                       ('Germany', [10.5, 51.29]),
-                       ('Italy', [12.2, 43.4])]
-        geom = []
-        for ugid, coordinate in enumerate(coordinates, start=1):
-            point = Point(coordinate[1][0], coordinate[1][1])
-            geom.append({'geom': point,
-                         'properties': {uid: ugid, 'COUNTRY': coordinate[0]}})
-        return geom
-
     def test_init(self):
         geom = make_poly((37.762, 38.222), (-102.281, -101.754))
 
@@ -575,11 +562,34 @@ class TestGeom(TestBase):
         with self.assertRaises(DefinitionValidationError):
             Geom(tempfile.gettempdir())
 
+    def test_init_field(self):
+        """Test using a field as initial value."""
+
+        field = Field.from_records(GeomCabinetIterator(path=self.path_state_boundaries))
+        self.assertIsInstance(field.crs, env.DEFAULT_COORDSYS.__class__)
+        g = Geom(field)
+        self.assertEqual(len(g.value), 51)
+        for field in g.value:
+            self.assertIsInstance(field, Field)
+            self.assertEqual(field.geom.shape, (1,))
+
+    @attr('data')
+    def test_init_data_model(self):
+        """Test data models are used when creating fields."""
+
+        ofo = OutputFormatOptions({'data_model': 'NETCDF3_CLASSIC'})
+        g = Geom(self.path_state_boundaries, output_format_options=ofo)
+        for row in g.value:
+            desired = {'UGID': np.int32,
+                       'ID': np.int32}
+            for v in desired.keys():
+                self.assertEqual(row[v].get_value().dtype, desired[v])
+
     def test_geometry_dictionaries(self):
         """Test geometry dictionaries as input."""
 
         for crs in [None, WGS84(), CoordinateReferenceSystem(epsg=2136)]:
-            geom = self.get_geometry_dictionaries()
+            geom = get_geometry_dictionaries()
             if crs is not None:
                 for g in geom:
                     g['crs'] = crs
@@ -596,20 +606,9 @@ class TestGeom(TestBase):
                 self.assertEqual(field['UGID'].get_value()[0], gdict['properties']['UGID'])
                 self.assertEqual(field['COUNTRY'].get_value()[0], gdict['properties']['COUNTRY'])
 
-    def test_init_field(self):
-        """Test using a field as initial value."""
-
-        field = Field.from_records(GeomCabinetIterator(path=self.path_state_boundaries))
-        self.assertIsInstance(field.crs, env.DEFAULT_COORDSYS.__class__)
-        g = Geom(field)
-        self.assertEqual(len(g.value), 51)
-        for field in g.value:
-            self.assertIsInstance(field, Field)
-            self.assertEqual(field.geom.shape, (1,))
-
     def test_parse(self):
         keywords = dict(geom_uid=[None, 'ID'],
-                        geom=[None, self.get_geometry_dictionaries(), self.get_geometry_dictionaries(uid='ID')])
+                        geom=[None, get_geometry_dictionaries(), get_geometry_dictionaries(uid='ID')])
         for k in self.iter_product_keywords(keywords):
             g = Geom(k.geom, geom_uid=k.geom_uid)
             ret = g.parse(k.geom)
