@@ -1,14 +1,24 @@
 from datetime import datetime as dt, datetime
+from unittest.case import SkipTest
 
+from ocgis import Variable, vm, Dimension, GeometryVariable
 from ocgis.constants import OCGIS_UNIQUE_GEOMETRY_IDENTIFIER
 from ocgis.spatial.geom_cabinet import GeomCabinetIterator
 from ocgis.test.base import TestBase
 from ocgis.test.base import attr
 from ocgis.util.helpers import *
-from ocgis.variable.crs import Spherical, CoordinateReferenceSystem
+from ocgis.variable.crs import Spherical, create_crs
+from ocgis.vmachine.mpi import OcgDist, variable_scatter, hgather
 
 
 class Test1(TestBase):
+    def test_create_ocgis_corners_from_esmf_corners(self):
+        ecorners = np.array([[0, 0.1],
+                             [0.2, 0.3]])
+        actual = create_ocgis_corners_from_esmf_corners(ecorners)
+        desired = [[[0.0, 0.1, 0.3, 0.2]]]
+        self.assertEqual(actual.tolist(), desired)
+
     def test_get_bounds_from_1d(self):
         across_180 = np.array([-180, -90, 0, 90, 180], dtype=float)
 
@@ -207,7 +217,8 @@ class Test2(TestBase):
         records = list(sci)
         self.assertAsSetEqual([1, 2], [xx['properties'][OCGIS_UNIQUE_GEOMETRY_IDENTIFIER] for xx in records])
         self.assertAsSetEqual([6, 60], [xx['properties']['fid'] for xx in records])
-        self.assertEqual(CoordinateReferenceSystem(records[0]['meta']['crs']), crs)
+        actual_crs = create_crs(records[0]['meta']['crs'])
+        self.assertEqual(actual_crs, crs)
 
         # test it works for the current working directory
         cwd = os.getcwd()
@@ -232,6 +243,104 @@ class Test2(TestBase):
             records = list(sci)
         self.assertAsSetEqual([6, 60], [xx['properties']['new_id'] for xx in records])
 
+    def test_arange_from_bool_ndarray(self):
+        arr = np.array([False, True, True, False, True])
+        actual = arange_from_bool_ndarray(arr, start=2)
+        actual = actual.tolist()
+        self.assertEqual(actual, [3, 4, 6])
+
+        arr = np.array([False, False, False])
+        actual = arange_from_bool_ndarray(arr)
+        self.assertEqual(actual.shape, (0,))
+
+    @attr('mpi')
+    def test_arange_from_dimension(self):
+        dist = OcgDist()
+        dim = dist.create_dimension('dim', size=7, dist=True)
+        dist.update_dimension_bounds()
+
+        actual = arange_from_dimension(dim, start=2, dtype=np.int64)
+        actual = vm.gather(actual)
+        if vm.rank == 0:
+            actual = hgather(actual)
+            desired = np.arange(2, 9, dtype=np.int64)
+            self.assertNumpyAll(actual, desired)
+
+    def test_create_index_array(self):
+        np.random.seed(1)
+        dst = np.random.rand(33)
+        src = np.random.choice(dst, size=11)
+
+        actual = create_index_array(src, dst)
+        self.assertNumpyAll(dst[actual], src)
+
+    @attr('mpi')
+    def test_create_index_variable_global(self):
+        raise SkipTest('not implemented')
+
+        dsrc_size = 5
+        ddst_size = 7
+
+        dsrc = Dimension('dsrc', dsrc_size, dist=True)
+        src_dist = OcgDist()
+        src_dist.add_dimension(dsrc)
+        src_dist.update_dimension_bounds()
+
+        ddst = Dimension('ddst', ddst_size, dist=True)
+        dst_dist = OcgDist()
+        dst_dist.add_dimension(ddst)
+        dst_dist.update_dimension_bounds()
+
+        if vm.rank == 0:
+            np.random.seed(1)
+            dst = np.random.rand(ddst_size)
+            src = np.random.choice(dst, size=dsrc_size, replace=False)
+
+            src = Variable(name='src', value=src, dimensions=dsrc.name)
+            # TODO: move create_ugid_global to create_global_index on a standard variable object
+            dst = GeometryVariable(name='dst', value=dst, dimensions=ddst.name)
+        else:
+            src, dst = [None] * 2
+
+        src = variable_scatter(src, src_dist)
+        dst = variable_scatter(dst, dst_dist)
+
+        actual = create_index_variable_global('index_array', src, dst)
+
+        self.assertNumpyAll(dst.get_value()[actual.get_value()], src.get_value())
+
+    @attr('mpi')
+    def test_create_unique_global_array(self):
+        dist = OcgDist()
+        dist.create_dimension('dim', 9, dist=True)
+        dist.update_dimension_bounds()
+
+        values = [
+            [4, 2, 1, 2, 1, 4, 1, 4, 2],
+            [44, 25, 16, 27, 18, 49, 10, 41, 22],
+            [44, 25, 16, 27, 44, 49, 10, 41, 44],
+            [1, 1, 1, 1, 1, 1, 1, 1, 1]
+        ]
+
+        for v in values:
+            if vm.rank == 0:
+                index = Variable(name='cindex', value=v, dimensions='dim')
+                desired = np.unique(index.get_value())
+                desired_length = len(desired)
+            else:
+                index = None
+            index = variable_scatter(index, dist)
+
+            with vm.scoped_by_emptyable('not empty', index):
+                if not vm.is_null:
+                    uvar = create_unique_global_array(index.get_value())
+                    uvar_gathered = vm.gather(uvar)
+
+                    if vm.rank == 0:
+                        uvar_gathered = hgather(uvar_gathered)
+                        self.assertEqual(len(uvar_gathered), desired_length)
+                        self.assertEqual(set(uvar_gathered), set(desired))
+
     def test_get_iter(self):
         element = 'hi'
         ret = list(get_iter(element))
@@ -512,6 +621,10 @@ class Test2(TestBase):
         ret = get_formatted_slice(slc, 1)
         self.assertNumpyAll(ret[0], slc)
 
+        slc = np.array([True], dtype=bool)
+        ret = get_formatted_slice(slc, 1)
+        self.assertNumpyAll(ret[0], slc)
+
     def test_set_name_attributes(self):
 
         class Foo(object):
