@@ -8,17 +8,29 @@ from numpy.core.multiarray import ndarray
 from numpy.testing.utils import assert_equal
 from shapely.geometry import Point
 
-from ocgis import RequestDataset, vm
+from ocgis import RequestDataset, vm, env
+from ocgis.base import get_variable_names
 from ocgis.collection.field import Field
 from ocgis.constants import HeaderName
 from ocgis.exc import VariableInCollectionError, EmptySubsetError, NoUnitsError, PayloadProtectedError, \
     DimensionsRequiredError
-from ocgis.test.base import attr, nc_scope, AbstractTestInterface
+from ocgis.ops.core import OcgOperations
+from ocgis.test.base import attr, nc_scope, AbstractTestInterface, TestBase
 from ocgis.util.units import get_units_object, get_are_units_equal
-from ocgis.variable.base import Variable, SourcedVariable, VariableCollection, ObjectType, init_from_source
+from ocgis.variable.base import Variable, SourcedVariable, VariableCollection, ObjectType, init_from_source, \
+    create_typed_variable_from_data_model
 from ocgis.variable.dimension import Dimension
 from ocgis.variable.geom import GeometryVariable
-from ocgis.vmachine.mpi import MPI_SIZE, MPI_RANK, OcgDist, MPI_COMM, hgather
+from ocgis.vmachine.mpi import MPI_SIZE, MPI_RANK, OcgDist, MPI_COMM, hgather, variable_scatter, variable_gather
+
+
+class Test(TestBase):
+    def test_create_typed_variable_from_data_model(self):
+        netcdf_file_format = 'NETCDF3_64BIT_OFFSET'
+        string_name = 'int'
+        actual = create_typed_variable_from_data_model(string_name, data_model=netcdf_file_format,
+                                                       name='test', value=[1, 2, 3, 4], dimensions='dim')
+        self.assertEqual(actual.dtype, np.int32)
 
 
 class TestVariable(AbstractTestInterface):
@@ -162,6 +174,15 @@ class TestVariable(AbstractTestInterface):
         var = Variable('woot', uid=50)
         self.assertEqual(var.uid, 50)
 
+        # Test setting is_empty to True.
+        v = Variable(is_empty=True)
+        self.assertTrue(v.is_empty)
+
+    def test_init_dimensions_named_the_same(self):
+        dim = Dimension('same', 7)
+        with self.assertRaises(ValueError):
+            Variable(name='samed', dimensions=(dim, dim))
+
     def test_init_dtype(self):
         """Test automatic data type detection."""
 
@@ -188,6 +209,11 @@ class TestVariable(AbstractTestInterface):
         var = Variable(name='b', value=np.array([1, 2, 3], dtype=object), dimensions='foo')
         self.assertIsNone(var.fill_value)
 
+    def test_init_hang_with_dimension(self):
+        # Test passing dimension to value.
+        with self.assertRaises(ValueError):
+            _ = Variable(name='data', value=Dimension('foo', 5), dimensions='blah')
+
     def test_init_object_array(self):
         value = [[1, 3, 5],
                  [7, 9],
@@ -287,6 +313,36 @@ class TestVariable(AbstractTestInterface):
         invar = RequestDataset(path).get()['strings']
         self.assertEqual(invar.dtype, 'S1')
 
+        value = [['2', '0', '1', '3', '-', '0', '3', '-', '0', '1', 'T', '0', '0', ':', '0', '0', ':', '0', '0'],
+                 ['2', '0', '1', '3', '-', '0', '3', '-', '0', '1', 'T', '0', '0', ':', '3', '0', ':', '0', '0']]
+        var = Variable(name='foo', value=value, dtype='S1', dimensions=['time', 'date_len'])
+        self.assertTrue(var.is_string_object)
+        var.set_string_max_length_global()
+        self.assertEqual(var.string_max_length_global, 19)
+
+    @attr('mpi')
+    def test_system_string_data_parallel(self):
+        if vm.size > 2:
+            raise SkipTest('vm.size > 2')
+
+        dist = OcgDist()
+        sdim = dist.create_dimension('sdim', 2, dist=True)
+        dist.update_dimension_bounds(min_elements=1)
+
+        if vm.rank == 0:
+            var = Variable(name='strings', value=['peas', 'peas please'], dimensions=sdim.name)
+            desired = len(var.get_value()[1])
+        else:
+            var = None
+            desired = None
+        desired = vm.bcast(desired)
+        var = variable_scatter(var, dist)
+        self.assertTrue(var.is_string_object)
+
+        self.assertIsNone(var.string_max_length_global)
+        var.set_string_max_length_global()
+        self.assertEqual(var.string_max_length_global, desired)
+
     @attr('mpi')
     def test_system_with_distributed_dimensions_from_file_netcdf(self):
         """Test a distributed read from file."""
@@ -367,6 +423,23 @@ class TestVariable(AbstractTestInterface):
         var = Variable(name='host', value=[1.5, 3.5], dimensions='a', bounds=bounds, parent=parent)
         self.assertEqual(list(var.parent.keys()), ['remember', 'host', 'time_value', 'the_bounds'])
 
+    def test_system_subsetting_single_value(self):
+        # Test using data loaded from file.
+        field = self.get_field(nlevel=1)
+        level_value = field.level.get_value()[0]
+        path = self.get_temporary_file_path('foo.nc')
+        field.write(path)
+        infield = RequestDataset(path).get()
+        infield.level.get_between(level_value - 1, level_value + 1)
+        self.assertEqual(infield.level.get_value()[0], level_value)
+
+        # Test through operations.
+        level_range = [int(level_value - 1), int(level_value + 1)]
+        ops = OcgOperations(dataset={'uri': path}, level_range=level_range, output_format='nc')
+        actual = ops.execute()
+        actual = RequestDataset(actual).get().level.get_value()[0]
+        self.assertEqual(actual, level_value)
+
     def test_allocate_value(self):
         dim = Dimension('foo', 5)
         var = Variable(name='tester', dimensions=dim, dtype=np.int8)
@@ -379,7 +452,7 @@ class TestVariable(AbstractTestInterface):
                        repeat_record=OrderedDict([('key', 'min'), ('src', 'foo_origin')]))
 
         desired = OrderedDict([('DID', None), ('foo', 1), ('key', 'min'), ('src', 'foo_origin')])
-        actual = var.as_record()
+        actual = var._as_record_()
         self.assertDictEqual(actual, desired)
 
     def test_bounds(self):
@@ -484,7 +557,8 @@ class TestVariable(AbstractTestInterface):
         self.assertNotIn('overload', var.parent.dimensions)
 
         # Test this is not a deepcopy.
-        var = Variable(value=[10, 11, 12], dimensions=Dimension('three', 3, src_idx='auto'), name='no deepcopies!')
+        var = Variable(value=[10, 11, 12], dimensions=Dimension('three', 3, src_idx=np.arange(3)),
+                       name='no deepcopies!')
         cvar = var.copy()
         self.assertNumpyMayShareMemory(var.get_value(), cvar.get_value())
         self.assertNumpyMayShareMemory(var.dimensions[0]._src_idx, cvar.dimensions[0]._src_idx)
@@ -528,7 +602,7 @@ class TestVariable(AbstractTestInterface):
         ancillary = Variable('ancillary')
         src = Variable('src', parent=ancillary.parent)
         csrc = src.copy()
-        csrc.extract()
+        csrc = csrc.extract()
 
         self.assertNotIn(ancillary.name, csrc.parent)
         self.assertIn(ancillary.name, src.parent)
@@ -542,72 +616,26 @@ class TestVariable(AbstractTestInterface):
         self.assertNotIn(v1.name, vc)
         self.assertEqual(list(vc.keys()), ['two', 'three'])
 
+        # Test dimensions match the extracted variable only.
+        var1 = Variable('var1', value=[1, 2], dimensions='dim1')
+        var2 = Variable('var2', value=[1, 2, 3], dimensions='dim2')
+        _ = VariableCollection(variables=[var1, var2])
+        var_extract = var1.extract()
+        self.assertEqual(len(var_extract.parent.dimensions), 1)
+        self.assertEqual(var_extract.parent.dimensions[var1.dimensions[0].name].name, var1.dimensions[0].name)
+
     def test_get_between(self):
         bv = Variable('foo', value=[0], dimensions='uni')
         with self.assertRaises(EmptySubsetError):
             bv.get_between(100, 200)
 
-        bv = Variable('foo', value=[100, 200, 300, 400], dimensions='a')
+        dim = Dimension('a', 4, src_idx='auto')
+        bv = Variable('foo', value=[100, 200, 300, 400], dimensions=dim)
         vdim_between = bv.get_between(100, 200)
         self.assertEqual(vdim_between.shape[0], 2)
-
-    def test_getitem(self):
-        var = Variable(value=[1, 2, 3], name='three', dimensions='three')
-        sub = var[1]
-        self.assertEqual(var.shape, (3,))
-        self.assertEqual(sub.shape, (1,))
-
-        # Test a dictionary slice.
-        var = Variable(name='empty')
-        dslc = {'one': slice(2, 5), 'two': np.array([False, True, False, True]), 'three': 0}
-        with self.assertRaises(DimensionsRequiredError):
-            var[dslc]
-        value = np.ma.arange(5 * 4 * 7 * 10, fill_value=100).reshape(5, 4, 10, 7)
-        var = Variable(value=value, name='range', dimensions=['one', 'two', 'four', 'three'])
-        sub = var[dslc]
-        self.assertEqual(sub.shape, (3, 2, 10, 1))
-        sub_value = value[2:5, np.array([False, True, False, True], dtype=bool), slice(None), slice(0, 1)]
-        self.assertNumpyAll(sub.get_masked_value(), sub_value)
-
-        # Test with a parent.
-        var = Variable(name='a', value=[1, 2, 3], dimensions=['one'])
-        parent = VariableCollection(variables=[var])
-        var2 = Variable(name='b', value=[11, 22, 33], dimensions=['one'], parent=parent)
-        self.assertIn('b', var2.parent)
-        sub = var2[1]
-        self.assertEqual(var2.parent.shapes, OrderedDict([('a', (3,)), ('b', (3,))]))
-        self.assertEqual(sub.parent.shapes, OrderedDict([('a', (1,)), ('b', (1,))]))
-
-        # Test slicing a bounded variabled.
-        bv = self.get_boundedvariable()
-        sub = bv[1]
-        self.assertEqual(sub.bounds.shape, (1, 2))
-        self.assertNumpyAll(sub.bounds.get_value(), bv.bounds[1, :].get_value())
-
-        # Test with a boolean array.
-        var = Variable(value=[1, 2, 3, 4, 5], name='five', dimensions='five')
-        sub = var[[False, True, True, True, False]]
-        self.assertEqual(sub.shape, (3,))
-
-        # Test filling the value on a sliced variable.
-        var = Variable(name='foo', dimensions=Dimension('one', 3), dtype=object)
-        self.assertIsNone(var._value)
-        var.allocate_value()
-        sub = var[{'one': 1}]
-        sub.get_value()[:] = 100.
-        self.assertEqual(var.get_value()[1], 100.)
-
-        # Test dimension bounds are updated.
-        var = Variable(name='foo', value=np.zeros((31, 360, 720)), dimensions=['time', 'row', 'col'])
-        desired_bounds_global = [(0, 31), (0, 360), (0, 720)]
-        bounds_global = [d.bounds_global for d in var.dimensions]
-        self.assertEqual(bounds_global, desired_bounds_global)
-        slc = {'time': 1, 'row': slice(0, 180), 'col': slice(0, 360)}
-        desired_bounds_global_slice = [(0, 1), (0, 180), (0, 360)]
-        sub = var[slc]
-        bounds_global = [d.bounds_global for d in sub.dimensions]
-        self.assertEqual(desired_bounds_global_slice, bounds_global)
-        self.assertEqual(desired_bounds_global_slice, [d.bounds_local for d in sub.dimensions])
+        self.assertNumpyMayShareMemory(bv.get_value(), vdim_between.get_value())
+        actual = vdim_between.dimensions[0]._src_idx
+        self.assertEqual(actual, (0, 2))
 
     def test_get_between_bounds(self):
         value = [0., 5., 10.]
@@ -665,12 +693,7 @@ class TestVariable(AbstractTestInterface):
 
         var = Variable(name='data', value=[1, 2, 3, 4], dtype=float, dimensions='dim')
         var.set_extrapolated_bounds('data_bounds', 'bounds')
-        # print var.get_value()
-        # print var.bounds.get_value()
-
         sub = var.get_between(2., 3.)
-        # print sub.get_value()
-        # print sub.bounds.get_value()
 
     def test_get_between_use_bounds(self):
         value = [3., 5.]
@@ -681,6 +704,12 @@ class TestVariable(AbstractTestInterface):
         self.assertNumpyAll(ret.get_masked_value(), np.ma.array([3.]))
         self.assertNumpyAll(ret.bounds.get_masked_value(), np.ma.array([[2., 4.]]))
 
+    def test_get_between_single_value(self):
+        var = Variable(name='tester', value=[3], dimensions='level')
+        sub = var.get_between(1, 5)
+        self.assertEqual(var.shape, sub.shape)
+        self.assertNumpyAll(var.get_value(), sub.get_value())
+
     @attr('mpi')
     def test_get_distributed_slice(self):
         if MPI_RANK == 0:
@@ -709,6 +738,56 @@ class TestVariable(AbstractTestInterface):
                 svar = SourcedVariable('var', request_dataset=rd)
                 self.assertEqual(svar.shape, (10, 2, 1))
 
+    @attr('mpi')
+    def test_get_distributed_slice_fancy_indexing(self):
+        """Test using a fancy indexing slice in parallel."""
+
+        if vm.size != 2:
+            raise SkipTest('vm.size != 2')
+
+        dist = OcgDist()
+        dim1 = dist.create_dimension('dim1', size=7, dist=True, src_idx='auto')
+        dim2 = dist.create_dimension('dim2', size=4)
+        dist.update_dimension_bounds()
+
+        if vm.rank == 0:
+            value = np.zeros((7, 4))
+            for idx in range(value.shape[0]):
+                value[idx, :] = idx
+            var = Variable(name='test', dimensions=['dim1', 'dim2'], value=value)
+        else:
+            var = None
+        var = variable_scatter(var, dist)
+
+        slices = {0: np.array([False, True, False, True]),
+                  1: np.array([False, True, False])}
+        for k, v in slices.items():
+            v = [v, slice(None)]
+            slices[k] = v
+
+        sub = var.get_distributed_slice(slices[vm.rank])
+
+        gvar = variable_gather(sub)
+
+        if vm.rank == 0:
+            desired = [[1.0, 1.0, 1.0, 1.0], [3.0, 3.0, 3.0, 3.0], [5.0, 5.0, 5.0, 5.0]]
+            actual = gvar.get_value().tolist()
+            self.assertEqual(actual, desired)
+
+        # Test when all are False.
+        slices = {0: np.array([False, False, False, False]),
+                  1: np.array([True, True, True])}
+        for k, v in slices.items():
+            v = [v, slice(None)]
+            slices[k] = v
+        sub = var.get_distributed_slice(slices[vm.rank])
+        if vm.rank == 0:
+            self.assertTrue(sub.is_empty)
+        else:
+            self.assertFalse(sub.is_empty)
+            self.assertEqual(sub.get_value().sum(), 60)
+            self.assertEqual(sub.dimensions[0]._src_idx.tolist(), [4, 5, 6])
+
     @attr('mpi')
     def test_get_distributed_slice_parent_variables(self):
         """Test distributed slicing on a variable with sibling variables."""
@@ -749,6 +828,7 @@ class TestVariable(AbstractTestInterface):
 
     @attr('mpi')
     def test_get_distributed_slice_simple(self):
+        self.add_barrier = False
         if MPI_RANK == 0:
             path = self.get_temporary_file_path('out.nc')
         else:
@@ -861,6 +941,82 @@ class TestVariable(AbstractTestInterface):
             self.assertTrue(var.get_mask()[1])
             self.assertTrue(var._mask[1])
 
+    def test_getitem(self):
+        var = Variable(value=[1, 2, 3], name='three', dimensions='three')
+        sub = var[1]
+        self.assertEqual(var.shape, (3,))
+        self.assertEqual(sub.shape, (1,))
+
+        # Test a dictionary slice.
+        var = Variable(name='empty')
+        dslc = {'one': slice(2, 5), 'two': np.array([False, True, False, True]), 'three': 0}
+        with self.assertRaises(DimensionsRequiredError):
+            var[dslc]
+        value = np.ma.arange(5 * 4 * 7 * 10, fill_value=100).reshape(5, 4, 10, 7)
+        var = Variable(value=value, name='range', dimensions=['one', 'two', 'four', 'three'])
+        sub = var[dslc]
+        self.assertEqual(sub.shape, (3, 2, 10, 1))
+        sub_value = value[2:5, np.array([False, True, False, True], dtype=bool), slice(None), slice(0, 1)]
+        self.assertNumpyAll(sub.get_masked_value(), sub_value)
+
+        # Test with a parent.
+        var = Variable(name='a', value=[1, 2, 3], dimensions=['one'])
+        parent = VariableCollection(variables=[var])
+        var2 = Variable(name='b', value=[11, 22, 33], dimensions=['one'], parent=parent)
+        self.assertIn('b', var2.parent)
+        sub = var2[1]
+        self.assertEqual(var2.parent.shapes, OrderedDict([('a', (3,)), ('b', (3,))]))
+        self.assertEqual(sub.parent.shapes, OrderedDict([('a', (1,)), ('b', (1,))]))
+
+        # Test slicing a bounded variable.
+        bv = self.get_boundedvariable()
+        sub = bv[1]
+        self.assertEqual(sub.bounds.shape, (1, 2))
+        self.assertNumpyAll(sub.bounds.get_value(), bv.bounds[1, :].get_value())
+
+        # Test with a boolean array.
+        var = Variable(value=[1, 2, 3, 4, 5], name='five', dimensions='five')
+        sub = var[[False, True, True, True, False]]
+        self.assertEqual(sub.shape, (3,))
+
+        # Test filling the value on a sliced variable.
+        var = Variable(name='foo', dimensions=Dimension('one', 3), dtype=object)
+        self.assertIsNone(var._value)
+        var.allocate_value()
+        sub = var[{'one': 1}]
+        sub.get_value()[:] = 100.
+        self.assertEqual(var.get_value()[1], 100.)
+
+        # Test dimension bounds are updated.
+        var = Variable(name='foo', value=np.zeros((31, 360, 720)), dimensions=['time', 'row', 'col'])
+        desired_bounds_global = [(0, 31), (0, 360), (0, 720)]
+        bounds_global = [d.bounds_global for d in var.dimensions]
+        self.assertEqual(bounds_global, desired_bounds_global)
+        slc = {'time': 1, 'row': slice(0, 180), 'col': slice(0, 360)}
+        desired_bounds_global_slice = [(0, 1), (0, 180), (0, 360)]
+        sub = var[slc]
+        bounds_global = [d.bounds_global for d in sub.dimensions]
+        self.assertEqual(desired_bounds_global_slice, bounds_global)
+        self.assertEqual(desired_bounds_global_slice, [d.bounds_local for d in sub.dimensions])
+
+    def test_getitem_index_slice(self):
+        # Test with an index slice.
+        d_coords = Dimension('d_coords', 7, src_idx='auto')
+        coords = Variable(name='coords', value=[0, 11, 22, 33, 44, 55, 66], dimensions=d_coords)
+        d_cindex = Dimension('d_cindex', 9, src_idx='auto')
+        index = Variable(name='cindex', value=[4, 2, 1, 2, 1, 4, 1, 4, 2], dimensions=d_cindex, parent=coords.parent)
+
+        sub_coords = coords[index.get_value()]
+        self.assertEqual(sub_coords.dimensions[0].size, 9)
+
+        actual = sub_coords.get_value().tolist()
+        desired = [44, 22, 11, 22, 11, 44, 11, 44, 22]
+        self.assertEqual(actual, desired)
+
+        actual = sub_coords.dimensions[0]._src_idx.tolist()
+        desired = index.get_value().tolist()
+        self.assertEqual(actual, desired)
+
     def test_group(self):
         var = Variable(name='lonely')
         self.assertEqual(var.group, [None])
@@ -872,6 +1028,24 @@ class TestVariable(AbstractTestInterface):
         vc3.add_variable(var)
         self.assertEqual(var.group, ['one', 'two', 'three'])
 
+    def test_set_bounds(self):
+
+        for l in ['__default__', False]:
+            if l == '__default__':
+                pass
+            else:
+                env.CLOBBER_UNITS_ON_BOUNDS = l
+
+            var = Variable(name='hi', value=[2], dimensions='one', units='unique')
+            bnds = Variable(name='hi_bounds', value=[[1, 3]], dimensions=['one', 'bounds'])
+
+            var.set_bounds(bnds)
+
+            if l == '__default__':
+                self.assertEqual(var.units, bnds.units)
+            else:
+                self.assertIsNone(bnds.units)
+
     def test_set_extrapolated_bounds(self):
         bv = self.get_boundedvariable(mask=[False, True, False])
         self.assertIsNotNone(bv.bounds)
@@ -902,6 +1076,16 @@ class TestVariable(AbstractTestInterface):
         self.assertEqual(len(bounds_dimensions), 3)
         self.assertEqual(bounds_dimensions[2].name, 'bounds_dimension')
 
+    def test_set_value(self):
+        # Test setting value to None only allowed for dimensionless.
+        var = Variable(name='tester', value=[1, 2, 3], mask=[False, True, False], dimensions=['foo'])
+        with self.assertRaises(ValueError):
+            var.set_value(None)
+
+        var = Variable(name='tester2', value=4, dimensions=())
+        var.set_value(None)
+        self.assertIsNone(var.get_value())
+
     def test_setitem(self):
         var = Variable(value=[10, 10, 10, 10, 10], name='tens', dimensions='dim')
         var2 = Variable(value=[2, 3, 4], mask=[True, True, False], name='var2', dimensions='three')
@@ -1117,6 +1301,22 @@ class TestSourcedVariable(AbstractTestInterface):
         self.assertLess(sv.get_value().sum(), 200)
         self.assertEqual(sv.units, 'celsius')
 
+    def test_system_get_between_memory(self):
+        # Test memory usage is lower when subsetting from file.
+        def _get_kb_(dtype, elements):
+            nbytes = np.array([1], dtype=dtype).nbytes
+            return float((elements * nbytes) / 1024.0)
+
+        var = Variable(name='levels', value=range(100000), dimensions='level')
+        original_kb = _get_kb_(var.dtype, var.get_value().size)
+        path = self.get_temporary_file_path('foo.nc')
+        var.parent.write(path)
+        rd = RequestDataset(path)
+        field = rd.create_raw_field()
+        sub = field['levels'].get_between(5, 100)
+        actual_kb = _get_kb_(sub.dtype, sub.get_value().size)
+        self.assertLess(actual_kb, original_kb - 100)
+
     def test_system_using_source_name_from_netcdf(self):
         path = self.get_temporary_file_path('foo.nc')
         with nc_scope(path, mode='w') as ds:
@@ -1182,13 +1382,6 @@ class TestSourcedVariable(AbstractTestInterface):
 
         self.assertNumpyAll(sub.get_value(), actual)
 
-        # Test value may be set to None and value is not reloaded.
-        sv = self.get_sourcedvariable()
-        self.assertIsNone(sv._value)
-        self.assertIsNotNone(sv.get_value())
-        sv.set_value(None)
-        self.assertIsNone(sv.get_value())
-
     @attr('data')
     def test_init_from_source(self):
         sv = self.get_sourcedvariable()
@@ -1222,7 +1415,7 @@ class TestSourcedVariable(AbstractTestInterface):
         vc.write(path)
 
         rd = RequestDataset(path)
-        vc_ff = rd.get_variable_collection()
+        vc_ff = rd.create_raw_field()
         svar = vc_ff[var1.name]
         svar2 = vc_ff[var2.name]
         svar.protected = True
@@ -1297,6 +1490,21 @@ class TestVariableCollection(AbstractTestInterface):
         vc = VariableCollection(uid=45)
         self.assertEqual(vc.uid, 45)
 
+    def test_find_by_attribute(self):
+        v1 = Variable(attrs={'a': 5}, name='1')
+        v2 = Variable(attrs={'b': 5}, name='2')
+        vc = VariableCollection(variables=[v1, v2])
+
+        actual = vc.find_by_attribute('c', value=10)
+        self.assertEqual(len(actual), 0)
+
+        actual = vc.find_by_attribute('b', 5)
+        self.assertEqual(actual[0].name, v2.name)
+
+        pred = lambda x: x == 5
+        actual = vc.find_by_attribute(pred=pred)
+        self.assertEqual(get_variable_names(actual), (v1.name, v2.name))
+
     def test_getitem(self):
         vc = self.get_variablecollection()
         slc = {'y': slice(1, 2)}
@@ -1439,6 +1647,29 @@ class TestVariableCollection(AbstractTestInterface):
         self.assertNotEqual(id(vc['b']), id(vc_copy['b']))
         self.assertNotEqual(id(vc['b'].get_value()), id(vc_copy['b'].get_value()))
 
+    def test_groups_to_variable(self):
+
+        np.random.seed(5)
+
+        def _make_child_(parent, ii):
+            child = VariableCollection(name=ii)
+            var = Variable(name='cvar', value=np.random.rand(3, 1) + ii, dimensions=['time', 'to_agg'])
+            self.assertEqual(var.shape, (3, 1))
+            dummy = Variable(name='dummy', dimensions=[])
+            child.add_variable(var)
+            child.add_variable(dummy)
+            parent.add_child(child)
+
+        lead = VariableCollection(name='lead')
+        for ii in range(4):
+            _make_child_(lead, ii)
+
+        actual = lead.groups_to_variable(name='concat', dimensions='to_agg')
+
+        self.assertAlmostEqual(actual['cvar'].v().sum(), 23.906239731796518)
+        self.assertEqual(list(actual.keys()), ['dummy', 'cvar', 'concat'])
+        self.assertEqual(actual['concat'].v().tolist(), list(range(4)))
+
     def test_remove_variable(self):
         v1 = Variable(name='vone', value=[1, 2, 3], dimensions='three')
         v1.set_extrapolated_bounds('vone_bounds', 'bounds')
@@ -1450,7 +1681,7 @@ class TestVariableCollection(AbstractTestInterface):
         self.assertNotIn(v2.name, vc)
         self.assertNotIn('two', vc.dimensions)
 
-        vc.remove_variable(v1)
+        v1 = vc.remove_variable(v1)
         self.assertNotIn(v1.name, vc)
         self.assertNotIn(v1.bounds.name, vc)
         self.assertNotIn(v1.bounds.dimensions[1].name, vc.dimensions)
@@ -1461,3 +1692,12 @@ class TestVariableCollection(AbstractTestInterface):
         self.assertNotIn(v4.name, vc)
 
         self.assertEqual(len(vc.dimensions), 0)
+
+    def test_rename_dimension(self):
+        parent = VariableCollection()
+        var = Variable(name='foo', value=[0], dimensions='one', parent=parent)
+        vc = var.parent
+        self.assertIsInstance(vc, VariableCollection)
+        self.assertEqual(id(parent), id(vc))
+        vc.rename_dimension('one', 'one_renamed')
+        self.assertEqual(var.dimensions[0].name, 'one_renamed')
