@@ -1,6 +1,6 @@
 import os
 import sys
-from copy import deepcopy
+from copy import deepcopy, copy
 from unittest import SkipTest
 
 import netCDF4 as nc
@@ -11,8 +11,9 @@ from shapely.geometry.multipolygon import MultiPolygon
 import ocgis
 from ocgis import constants, vm
 from ocgis.collection.field import Field
-from ocgis.constants import WrapAction, WrappedState, ConversionFactor, OcgisUnits
-from ocgis.exc import CRSNotEquivalenError
+from ocgis.constants import WrapAction, WrappedState, ConversionFactor, OcgisUnits, CFName
+from ocgis.driver.request.core import RequestDataset
+from ocgis.exc import CRSNotEquivalenError, CRSDepthNotImplemented
 from ocgis.spatial.grid import Grid
 from ocgis.test.base import TestBase, nc_scope, create_gridxy_global
 from ocgis.test.base import attr
@@ -20,10 +21,22 @@ from ocgis.util.helpers import make_poly
 from ocgis.util.itester import itr_products_keywords
 from ocgis.variable.base import Variable
 from ocgis.variable.crs import CoordinateReferenceSystem, CFAlbersEqualArea, CFLambertConformal, \
-    CFRotatedPole, WGS84, Spherical, CFSpherical, Tripole, Cartesian
+    CFRotatedPole, WGS84, Spherical, CFSpherical, Tripole, Cartesian, AbstractProj4CRS, create_crs
+from ocgis.variable.geom import GeometryVariable
 from ocgis.vmachine.mpi import OcgDist, MPI_RANK, variable_scatter
 
 
+class Test(TestBase):
+    def test_create_crs(self):
+        value = {'a': 6370997, 'no_defs': True, 'b': 6370997, 'proj': 'longlat'}
+        actual = create_crs(value)
+        self.assertIsInstance(actual, Spherical)
+
+        value = {'no_defs': True, 'datum': 'WGS84', 'proj': 'longlat'}
+        actual = create_crs(value)
+        self.assertIsInstance(actual, WGS84)
+
+
 class TestCoordinateReferenceSystem(TestBase):
     def test_init(self):
         keywords = dict(
@@ -63,7 +76,7 @@ class TestCoordinateReferenceSystem(TestBase):
 
         # test with a name parameter
         crs = CoordinateReferenceSystem(epsg=4326)
-        self.assertEqual(crs.name, constants.DEFAULT_COORDINATE_SYSTEM_NAME)
+        self.assertEqual(crs.name, constants.OcgisConvention.Name.COORDSYS)
         crs = CoordinateReferenceSystem(epsg=4326, name='foo')
         self.assertEqual(crs.name, 'foo')
 
@@ -71,6 +84,17 @@ class TestCoordinateReferenceSystem(TestBase):
         value = {'init': 'epsg:4326'}
         self.assertEqual(CoordinateReferenceSystem(value=value), WGS84())
 
+    def test_eq(self):
+        value_lhs = {'a': 6370997, 'b': 6370997, 'no_defs': True, 'proj': 'longlat', 'wktext': True,
+                     'towgs84': '0,0,0,0,0,0,0'}
+        lhs = CoordinateReferenceSystem(value=value_lhs)
+
+        value_rhs = value_lhs.copy()
+        value_rhs.pop('wktext')
+        rhs = CoordinateReferenceSystem(value=value_rhs)
+
+        self.assertEqual(lhs, rhs)
+
     def test_ne(self):
         crs1 = CoordinateReferenceSystem(epsg=4326)
         crs2 = CoordinateReferenceSystem(epsg=2136)
@@ -80,7 +104,6 @@ class TestCoordinateReferenceSystem(TestBase):
         self.assertNotEqual(crs1, None)
         self.assertNotEqual(None, crs1)
 
-        # try nonetype and string
         self.assertNotEqual(None, crs1)
         self.assertNotEqual('input', crs1)
 
@@ -90,8 +113,6 @@ class TestCoordinateReferenceSystem(TestBase):
         with nc_scope(path, 'w') as ds:
             variable = crs.write_to_rootgrp(ds)
             self.assertIsInstance(variable, nc.Variable)
-            with self.assertRaises(AttributeError):
-                variable.proj4
 
     def test_get_wrap_action(self):
         c = CoordinateReferenceSystem
@@ -113,7 +134,7 @@ class TestCoordinateReferenceSystem(TestBase):
             else:
                 self.assertIsNone(ret)
 
-    @attr('mpi')
+    @attr('mpi', 'no-3.5')
     def test_get_wrapped_state(self):
         if sys.version_info.major == 3 and sys.version_info.minor == 5:
             raise SkipTest('undefined behavior with Python 3.5')
@@ -158,6 +179,14 @@ class TestCoordinateReferenceSystem(TestBase):
             else:
                 self.assertEqual(wrapped_state, k.values['desired'])
 
+        # Test with masked geometries.
+        values = [Point(350, 2), Point(-90, 5), Point(340, 5)]
+        mask = [True, False, True]
+        gvar = GeometryVariable(name='geom', value=values, mask=mask, dimensions='ngeom')
+        crs = Spherical()
+        wrapped_state = crs.get_wrapped_state(gvar)
+        self.assertEqual(wrapped_state, WrappedState.WRAPPED)
+
     def test_get_wrapped_state_from_array(self):
 
         def _run_(arr, actual_wrapped_state):
@@ -221,6 +250,16 @@ class TestSpherical(TestBase):
                                          'towgs84': '0,0,0,0,0,0,0'})
         self.assertEqual(crs.name, 'latitude_longitude')
 
+    def test_copy(self):
+        crs = Spherical()
+        for _ in range(10):
+            copy(crs)
+
+    def test_deepcopy(self):
+        crs = Spherical()
+        for _ in range(10):
+            deepcopy(crs)
+
     def test_place_prime_meridian_array(self):
         arr = np.array([123, 180, 200, 180], dtype=float)
         ret = Spherical._place_prime_meridian_array_(arr)
@@ -259,6 +298,19 @@ class TestWGS84(TestBase):
         self.assertNotIsInstance(WGS84(), Spherical)
         self.assertIn('towgs84', WGS84().value)
 
+    def test_system_writing_to_netcdf(self):
+        """Test coordinate system is retrievable from netCDF format."""
+
+        path = self.get_temporary_file_path('crs.nc')
+        with nc.Dataset(path, 'w') as rootgrp:
+            WGS84().write_to_rootgrp(rootgrp)
+        rd = RequestDataset(path)
+        infield = rd.get()
+        actual = infield.first()
+        self.assertEqual(actual, WGS84())
+        actual_crs = AbstractProj4CRS.load_from_metadata(rd.metadata)
+        self.assertEqual(actual_crs, WGS84())
+
 
 class TestCFAlbersEqualArea(TestBase):
     def test_init(self):
@@ -398,7 +450,8 @@ class TestCFRotatedPole(TestBase):
 
     @attr('data')
     def test_load_from_metadata(self):
-        rd = self.test_data.get_rd('rotated_pole_ichec')
+        rd = self.test_data.get_rd('rotated_pole_ichec', kwds={'rotated_pole_priority': True})
+        self.assertTrue(rd.rotated_pole_priority)
         self.assertIsInstance(rd.get().crs, CFRotatedPole)
 
         # Test without the grid_mapping attribute attached to a variable.
@@ -409,7 +462,7 @@ class TestCFRotatedPole(TestBase):
 
     @attr('data')
     def test_write_to_rootgrp(self):
-        rd = self.test_data.get_rd('narccap_rotated_pole')
+        rd = self.test_data.get_rd('narccap_rotated_pole', kwds={'rotated_pole_priority': True})
         path = os.path.join(self.current_dir_output, 'foo.nc')
 
         with nc_scope(path, 'w') as ds:
@@ -419,6 +472,26 @@ class TestCFRotatedPole(TestBase):
             self.assertEqual(variable.proj4_transform, rd.crs._trans_proj)
 
 
+class TestCFSpherical(TestBase):
+    def test_eq(self):
+        other = CFSpherical()
+        sph = Spherical()
+        self.assertTrue(sph == other)
+
+    def test_load_from_metadata(self):
+        # Test with greater depth which checks variable attributes.
+        lat = {'attrs': {CFName.STANDARD_NAME: CFName.StandardName.LATITUDE}}
+        lon = {'attrs': {CFName.STANDARD_NAME: CFName.StandardName.LONGITUDE}}
+        var = {'attrs': {}}
+        meta = {'variables': {'lat': lat, 'lon': lon, 'var': var}}
+        actual = CFSpherical.load_from_metadata('var', meta, depth=2)
+        self.assertIsInstance(actual, CFSpherical)
+
+        # Test a depth check is not supported.
+        with self.assertRaises(CRSDepthNotImplemented):
+            CFSpherical.load_from_metadata('var', meta, depth=-1)
+
+
 class TestTripole(TestBase):
     def test_init(self):
         Tripole()
