@@ -4,29 +4,17 @@ from unittest import SkipTest
 
 import numpy as np
 
-from ocgis import vm
+from ocgis import vm, RequestDataset
 from ocgis.constants import DataType
 from ocgis.test.base import attr, AbstractTestInterface
-from ocgis.util.helpers import get_local_to_global_slices
 from ocgis.variable.base import Variable, VariableCollection
 from ocgis.variable.dimension import Dimension
 from ocgis.vmachine.mpi import MPI_SIZE, MPI_COMM, create_nd_slices, hgather, \
     get_optimal_splits, get_rank_bounds, OcgDist, get_global_to_local_slice, MPI_RANK, variable_scatter, \
-    variable_collection_scatter, variable_gather, gather_ranks, get_standard_comm_state, \
-    get_nonempty_ranks, barrier_ranks, bcast_ranks
+    variable_collection_scatter, variable_gather, get_standard_comm_state, get_nonempty_ranks, redistribute_by_src_idx
 
 
 class Test(AbstractTestInterface):
-    @attr('mpi')
-    def test_barrier_ranks(self):
-        if MPI_SIZE != 3 and MPI_SIZE != 1:
-            raise SkipTest('serial or mpi-3 only')
-
-        barrier_ranks([0])
-
-        ranks = [0, 2]
-        if MPI_RANK != 1:
-            barrier_ranks(ranks)
 
     @attr('mpi')
     def test_groups(self):
@@ -49,28 +37,6 @@ class Test(AbstractTestInterface):
             sub_group.Free()
             new_comm.Free()
 
-    @attr('mpi')
-    def test_gather_ranks(self):
-        if MPI_SIZE != 3 and MPI_SIZE != 1:
-            raise SkipTest('serial or mpi-3 only')
-
-        value = 33
-        for root in range(MPI_SIZE):
-            gathered = gather_ranks([0], value, root=root)
-            if MPI_RANK == root:
-                self.assertEqual(gathered, [33])
-            else:
-                self.assertIsNone(gathered)
-
-        if MPI_SIZE == 3:
-            for root in range(MPI_SIZE):
-                ranks = [1, 2]
-                gathered = gather_ranks(ranks, MPI_RANK + 10, root=root)
-                if MPI_RANK == root:
-                    self.assertEqual(gathered, [11, 12])
-                else:
-                    self.assertIsNone(gathered)
-
     @attr('mpi')
     def test_get_nonempty_ranks(self):
         from ocgis.variable.dimension import Dimension
@@ -179,14 +145,6 @@ class Test(AbstractTestInterface):
         res = get_rank_bounds(6, size=5, rank=6)
         self.assertIsNone(res)
 
-    def test_get_local_to_global_slices(self):
-        # tdk: consider removing this function
-        slices_global = (slice(2, 4, None), slice(0, 2, None))
-        slices_local = (slice(0, 1, None), slice(0, 2, None))
-
-        lm = get_local_to_global_slices(slices_global, slices_local)
-        self.assertEqual(lm, (slice(2, 3, None), slice(0, 2, None)))
-
     def test_get_global_to_local_slice(self):
         start_stop = (1, 4)
         bounds_local = (0, 3)
@@ -230,26 +188,64 @@ class Test(AbstractTestInterface):
             _ = get_global_to_local_slice(start_stop, bounds_local)
 
     @attr('mpi')
-    def test_bcast_ranks(self):
-        if MPI_SIZE != 3 and MPI_SIZE != 1:
-            raise SkipTest('serial or mpi-3 only')
+    def test_redistribute_by_src_idx(self):
+        if vm.size != 4:
+            raise SkipTest('vm.size != 4')
 
-        actual = bcast_ranks([0], 50)
+        dist = OcgDist()
+        dim1 = dist.create_dimension('dim1', 5 * vm.size, dist=True)
+        dim2 = dist.create_dimension('dim2', 2, dist=False)
+        dist.update_dimension_bounds()
 
-        if MPI_RANK == 0:
-            self.assertEqual(actual, 50)
+        rank_value = np.arange(5) + (10 * (vm.rank + 1))
+        var1 = Variable(name='dvar1', value=rank_value, dimensions=dim1)
+        var2 = Variable(name='dvar2', dimensions=[dim1, dim2])
+        var1.parent.add_variable(var2)
+        path = self.get_temporary_file_path('out.nc')
+        var1.parent.write(path)
+
+        desired_idx = np.array([1, 7, 9, 10, 14])
+        vdesired_value = variable_gather(var1)
+        if vm.rank == 0:
+            desired_value = vdesired_value.get_value()[desired_idx]
+
+        desired_idx_ranks = {0: slice(1, 2),
+                             1: [2, 4],
+                             2: [0, 4]}
+
+        rd = RequestDataset(path)
+        rd.metadata['dimensions'][dim1.name]['dist'] = True
+        field = rd.create_field()
+
+        indvar = field[var1.name]
+        field[var2.name].load()
+
+        try:
+            rank_slice = desired_idx_ranks[vm.rank]
+        except KeyError:
+            sub = Variable(is_empty=True)
         else:
-            self.assertIsNone(actual)
+            sub = indvar[rank_slice]
 
-        actual = bcast_ranks([0, 2], 50)
-        if MPI_RANK == 1:
-            self.assertEqual(actual, None)
-        else:
-            self.assertEqual(actual, 50)
+        self.barrier_print(sub.is_empty)
 
-        if MPI_RANK != 1:
-            actual = bcast_ranks([0, 2], 50)
-            self.assertEqual(actual, 50)
+        redistribute_by_src_idx(indvar, dim1.name, sub.dimensions_dict.get(dim1.name))
+
+        with vm.scoped_by_emptyable('gather for test', indvar):
+            if vm.is_null:
+                self.assertIn(vm.rank_global, [2, 3])
+            else:
+                self.assertIn(vm.rank_global, [0, 1])
+                for v in [indvar, indvar.parent[var2.name]]:
+                    self.assertIsNone(v._value)
+                    self.assertIsNone(v._mask)
+                    self.assertIsNone(v._is_empty)
+                    self.assertFalse(v._has_initialized_value)
+                self.rank_print(indvar)
+                actual_value = variable_gather(indvar)
+                if vm.rank == 0:
+                    actual_value = actual_value.get_value()
+                    self.assertNumpyAll(actual_value, desired_value)
 
     @attr('mpi')
     def test_variable_collection_scatter(self):
@@ -301,7 +297,7 @@ class Test(AbstractTestInterface):
     def test_variable_gather(self):
         dist = OcgDist()
         three = dist.create_dimension('three', 3, src_idx=np.arange(3) * 10)
-        four = dist.create_dimension('four', 4, src_idx='auto', dist=True)
+        four = dist.create_dimension('four', 4, src_idx=np.arange(4, dtype=np.int32), dist=True)
         dist.create_variable('four', dimensions=[three, four])
         dist.update_dimension_bounds()
 
@@ -328,17 +324,44 @@ class Test(AbstractTestInterface):
         else:
             self.assertIsNone(mask_gather)
 
+    @attr('mpi')
+    def test_variable_gather_bounded_source_index(self):
+        self.add_barrier = False
+        dist = OcgDist()
+        dist.create_dimension('long', 7, dist=True, src_idx='auto')
+        dist.create_dimension('short', 3, src_idx='auto')
+        dist.update_dimension_bounds()
+
+        if vm.rank == 0:
+            var = Variable(name='test', value=np.arange(21).reshape(7, 3), dimensions=['long', 'short'])
+        else:
+            var = None
+        svar = variable_scatter(var, dist)
+
+        with vm.scoped_by_emptyable('gather test', svar):
+            if vm.is_null:
+                return
+            gvar = variable_gather(svar)
+
+            if vm.rank == 0:
+                actual = gvar.dimensions[0]._src_idx
+                desired = (0, 7)
+                self.assertEqual(actual, desired)
+            else:
+                self.assertIsNone(gvar)
+
     @attr('mpi')
     def test_variable_scatter(self):
         var_value = np.arange(5, dtype=float) + 50
         var_mask = np.array([True, True, False, True, False])
 
         dest_dist = OcgDist()
-        five = dest_dist.create_dimension('five', 5, src_idx='auto', dist=True)
+        five = dest_dist.create_dimension('five', 5, src_idx=np.arange(5), dist=True)
+        bounds = dest_dist.create_dimension('bounds', 2)
         dest_dist.update_dimension_bounds()
 
         if MPI_RANK == 0:
-            local_dim = Dimension('local', 5, src_idx='auto')
+            local_dim = Dimension('local', 5, src_idx=np.arange(5))
             dim_src_idx = local_dim._src_idx.copy()
 
             var = Variable('the_five', value=var_value, mask=var_mask, dimensions=five.name)
@@ -437,7 +460,7 @@ class TestOcgDist(AbstractTestInterface):
     @attr('mpi')
     def test(self):
         ompi = OcgDist()
-        src_idx = [2, 3, 4, 5, 6]
+        src_idx = np.array([2, 3, 4, 5, 6])
         dim = ompi.create_dimension('foo', size=5, group='subroot', dist=True, src_idx=src_idx)
         self.assertEqual(dim, ompi.get_dimension(dim.name, group='subroot'))
         self.assertEqual(dim.bounds_local, (0, len(dim)))
@@ -460,7 +483,7 @@ class TestOcgDist(AbstractTestInterface):
                 self.assertTrue(dim.is_empty)
 
         # Test with multiple dimensions.
-        d1 = Dimension('d1', size=5, dist=True, src_idx='auto')
+        d1 = Dimension('d1', size=5, dist=True, src_idx=np.arange(5, dtype=np.int32))
         d2 = Dimension('d2', size=10, dist=False)
         d3 = Dimension('d3', size=3, dist=True)
         dimensions = [d1, d2, d3]
@@ -590,3 +613,17 @@ class TestOcgDist(AbstractTestInterface):
                 self.assertTrue(actual.is_empty)
                 self.assertEqual(actual.bounds_local, (0, 0))
                 self.assertNotEqual(id(d1), id(actual))
+
+    def test_update_dimension_bounds_with_source_indexing(self):
+        dist_size = 5
+        dist = OcgDist(size=dist_size)
+        dist.create_dimension('dim', 11, dist=True, src_idx='auto')
+        dist.update_dimension_bounds()
+
+        actual = []
+        for rank in range(dist_size):
+            rank_dim = dist.get_dimension('dim', rank=rank)
+            actual.append(rank_dim._src_idx)
+
+        desired = [(0, 3), (3, 5), (5, 7), (7, 9), (9, 11)]
+        self.assertEqual(actual, desired)
