@@ -25,7 +25,7 @@ from ocgis import osr
 from ocgis.base import get_variable_names
 from ocgis.collection.field import Field
 from ocgis.collection.spatial import SpatialCollection
-from ocgis.constants import WrappedState, KeywordArgument, TagName, HeaderName
+from ocgis.constants import WrappedState, KeywordArgument, TagName, HeaderName, GridAbstraction, OutputFormatName
 from ocgis.exc import ExtentError, DefinitionValidationError
 from ocgis.ops.core import OcgOperations
 from ocgis.ops.interpreter import OcgInterpreter
@@ -68,11 +68,13 @@ class TestSimpleBase(TestBase):
             self.nc_factory().write()
         MPI_COMM.Barrier()
 
-    def get_dataset(self, time_range=None, level_range=None, time_region=None):
+    def get_dataset(self, time_range=None, level_range=None, time_region=None, **kwargs):
         uri = os.path.join(env.DIR_OUTPUT, self.fn)
-        return ({'uri': uri, 'variable': self.var,
-                 'time_range': time_range, 'level_range': level_range,
-                 'time_region': time_region})
+        ret = {'uri': uri, 'variable': self.var,
+               'time_range': time_range, 'level_range': level_range,
+               'time_region': time_region}
+        ret.update(kwargs)
+        return ret
 
     def get_request_dataset(self, *args, **kwargs):
         if KeywordArgument.URI not in kwargs:
@@ -374,7 +376,8 @@ class TestSimple(TestSimpleBase):
         self.assertEqual(ret.get_element().geom.shape, (4, 4))
         self.assertEqual(len(ret.children), 1)
 
-        ops = OcgOperations(dataset=self.get_dataset(), geom=geom, output_format='nc', prefix='nc')
+        ops = OcgOperations(dataset=self.get_dataset(), geom=geom,
+                            agg_selection=True, output_format='nc', prefix='nc')
         ret = ops.execute()
         with nc_scope(ret) as ds:
             ref = ds.variables['foo']
@@ -631,7 +634,8 @@ class TestSimple(TestSimpleBase):
             self.assertEqual(ref.shape, (2, 2, 1))
             self.assertEqual(ref.get_value().flatten().mean(), 2.5)
             actual = ret.children[None].children['foo']['my_mean'].attrs
-            self.assertDictEqual(actual, {'long_name': 'Mean', 'standard_name': 'mean', 'units': 'K'})
+            self.assertDictEqual(actual, {'long_name': 'Mean', 'standard_name': 'mean', 'units': 'K',
+                                          'grid_mapping': 'latitude_longitude'})
 
     def test_calc_multivariate(self):
         rd1 = self.get_dataset()
@@ -655,9 +659,11 @@ class TestSimple(TestSimpleBase):
         calc = 'foo2=foo+4'
         ocgis.env.OVERWRITE = True
         for of in OutputFormat.iter_possible():
+            if of == OutputFormatName.ESMPY_GRID:
+                continue
             ops = ocgis.OcgOperations(dataset=rd, calc=calc, output_format=of)
             ret = ops.execute()
-            if of == 'nc':
+            if of in ['nc']:
                 with nc_scope(ret) as ds:
                     self.assertEqual(ds.variables['foo2'][:].mean(), 6.5)
 
@@ -775,6 +781,154 @@ class TestSimple(TestSimpleBase):
         finally:
             ds.close()
 
+    def test_nc_discrete_geometry_simple(self):
+        env.OVERWRITE = True
+        rd = self.get_dataset()
+        geom = [{'geom': Point(-104., 38.), 'properties': {'Name': 'A'}},
+                {'geom': Point(-103., 39.), 'properties': {'Name': 'B'}}]
+
+        ops = OcgOperations(dataset=rd,
+                            calc=[{'func': 'mean', 'name': 'mean'}, ],
+                            calc_grouping='year',
+                            geom=geom, aggregate=True, search_radius_mult=.01,
+                            output_format='nc')
+
+        ops = self.get_ops(kwds={'geom': geom, 'aggregate': True,
+                                 'search_radius_mult': 0.01,
+                                 'output_format': 'nc'})
+        ret = ops.execute()
+
+        with self.nc_scope(ret) as ds:
+            self.assertEqual(ds.file_format,
+                             constants.NETCDF_DEFAULT_DATA_MODEL)
+            self.assertEqual(ds.featureType, 'timeSeries')
+            self.assertEqual(ds.variables[self.var].dimensions, (
+                constants.DimensionName.TEMPORAL, constants.NAME_DIMENSION_LEVEL,
+                constants.DimensionName.UNIONED_GEOMETRY))
+            self.assertEqual(ds.dimensions[constants.DimensionName.UNIONED_GEOMETRY].size, 2)
+            self.assertNumpyAllClose(ds.variables['longitude'][:], np.array([-104., -103.]))
+            self.assertNumpyAllClose(ds.variables['latitude'][:], np.array([38., 39.]))
+
+            expected = {'time': 'T', 'level': 'Z', 'latitude': 'Y',
+                        'longitude': 'X'}
+            for k, v in expected.items():
+                var = ds.variables[k]
+                self.assertEqual(var.axis, v)
+
+    def test_nc_discrete_geometry_dim_name(self):
+        # Test changing the geom dimension name
+        geom = [{'geom': Point(-104., 38.), 'properties': {'Name': 'A'}}, ]
+
+        ops = self.get_ops(kwds={'geom': geom, 'aggregate': True,
+                                 'search_radius_mult': 0.01,
+                                 'output_format': 'nc',
+                                 'output_format_options': {'geom_dim': 'region'}})
+        ret = self.get_ret(ops)
+        with self.nc_scope(ret) as ds:
+            self.assertEqual(ds.variables[self.var].dimensions, (
+                constants.DimensionName.TEMPORAL, constants.NAME_DIMENSION_LEVEL,
+                'region'))
+
+    def test_nc_discrete_geometry_validate(self):
+
+        with self.assertRaises(DefinitionValidationError):
+            ops = self.get_ops(kwds={'aggregate': True,
+                                     'output_format': 'nc'})
+
+        geom = [{'geom': Point(-104., 38.), 'properties': {'Name': 'A'}},
+                {'geom': Point(-103., 39.), 'properties': {'Name': 'B'}}]
+
+        with self.assertRaises(DefinitionValidationError):
+            ops = self.get_ops(kwds={'geom': geom,
+                                     'aggregate': False,
+                                     'agg_selection': False,
+                                     'output_format': 'nc'})
+
+        with self.assertRaises(DefinitionValidationError):
+            ops = self.get_ops(kwds={'geom': geom,
+                                     'spatial_operation': 'intersects',
+                                     'aggregate': False,
+                                     'output_format': 'nc'})
+        # This should be ok
+        ops = self.get_ops(kwds={'geom': geom[:1],
+                                 'aggregate': False,
+                                 'agg_selection': False,
+                                 'output_format': 'nc'})
+
+    def test_nc_discrete_geometry_compatibility(self):
+        """Check that the output in the discrete geometry format can be read
+        and operated on."""
+        env.OVERWRITE = True
+        rd = self.get_dataset()
+        geom = [{'geom': Point(-104., 38.)},
+                {'geom': Point(-103., 39.)}]
+
+        ops1 = OcgOperations(dataset=rd,
+                             geom=geom, aggregate=True, search_radius_mult=.01,
+                             output_format='nc')
+        ret = ops1.execute()
+
+        rd = RequestDataset(ret, variable='foo')
+        ops2 = OcgOperations(dataset=rd,
+                             calc=[{'func': 'mean', 'name': 'mean'}, ],
+                             calc_grouping='year')
+        ops2.execute()
+
+    def test_nc_discrete_geometry_conversion_calc(self):
+        calc_grouping = ['month']
+        calc = [{'func': 'mean', 'name': 'my_mean'},
+                {'func': 'std', 'name': 'my_stdev'}]
+        geom = make_poly((38, 39), (-104, -103))
+        kwds = dict(calc_grouping=calc_grouping, calc=calc, geom=geom, aggregate=True, output_format='nc')
+        ret = self.get_ret(kwds=kwds)
+
+        ds = nc.Dataset(ret)
+        try:
+            for alias in ['my_mean', 'my_stdev']:
+                self.assertEqual(ds.variables[alias].shape, (2, 2, 1))
+
+            # Time variable bounds attribute should be named to account for climatology.
+            with self.assertRaises(AttributeError):
+                assert ds.variables['time'].bounds
+
+            self.assertEqual(ds.variables['time'].climatology,
+                             'climatology_bounds')
+            self.assertEqual(ds.variables['climatology_bounds'].shape, (2, 2))
+        finally:
+            ds.close()
+
+    def test_nc_discrete_geometry_conversion_multiple_request_datasets(self):
+        rd1 = self.get_dataset()
+        rd2 = self.get_dataset()
+        rd2[KeywordArgument.FIELD_NAME] = 'foo2'
+        geom = make_poly((38, 39), (-104, -103))
+        with self.assertRaises(DefinitionValidationError):
+            OcgOperations(dataset=[rd1, rd2], geom=geom, aggregate=True,
+                          output_format='nc')
+
+    def test_nc_discrete_geometry_conversion_multivariate_calculation(self):
+        rd1 = self.get_dataset()
+        rd2 = self.get_dataset()
+        rd2[KeywordArgument.RENAME_VARIABLE] = 'foo2'
+        calc = [{'func': 'divide', 'name': 'my_divide',
+                 'kwds': {'arr1': 'foo', 'arr2': 'foo2'}}]
+        calc_grouping = ['month']
+        geom = make_poly((38, 39), (-104, -103))
+        ops = OcgOperations(dataset=[rd1, rd2], calc=calc,
+                            geom=geom,
+                            aggregate=True,
+                            calc_grouping=calc_grouping,
+                            output_format='nc')
+        ret = ops.execute()
+
+        ds = nc.Dataset(ret)
+        try:
+            ref = ds.variables['my_divide'][:]
+            self.assertEqual(ref.shape, (2, 2, 1))
+            self.assertEqual(np.unique(ref)[0], 1.)
+        finally:
+            ds.close()
+
     def test_shp_projection(self):
         output_crs = CoordinateReferenceSystem(epsg=2163)
         ret = self.get_ret(kwds=dict(output_crs=output_crs, output_format='shp', snippet=True))
@@ -947,7 +1101,7 @@ class TestSimple(TestSimpleBase):
 
             with open(ret, 'r') as f:
                 reader = csv.DictReader(f)
-                rows = list(reader)
+                _ = list(reader)
 
     def test_csv_calc_conversion(self):
         calc = [{'func': 'mean', 'name': 'my_mean'}]
@@ -969,8 +1123,8 @@ class TestSimple(TestSimpleBase):
                 else:
                     desired = {'SRC_VAR': 'foo', 'LB_LEVEL': '0', 'LEVEL': '50', 'DID': '1',
                                'TIME': '2000-03-16 00:00:00', 'MONTH': '3', 'my_mean': '1.0', 'UB_LEVEL': '100',
-                               'LB_TIME': '2000-03-01 00:00:00', 'YEAR': '2000', 'CALC_KEY': 'mean',
-                               'UB_TIME': '2000-04-01 00:00:00', 'DAY': '16'}
+                               'LB_TIME': '2000-03-01 00:00:00', 'YEAR': '2000', 'UB_TIME': '2000-04-01 00:00:00',
+                               'DAY': '16'}
                 self.assertDictEqual(row, desired)
 
     def test_csv_calc_conversion_two_calculations(self):
@@ -1017,7 +1171,7 @@ class TestSimple(TestSimpleBase):
                     row = next(reader)
                     desired = {'LB_LEVEL': '0', 'LEVEL': '50', 'DID': '1', 'TIME': '2000-03-16 00:00:00', 'MONTH': '3',
                                'UB_LEVEL': '100', 'LB_TIME': '2000-03-01 00:00:00', 'YEAR': '2000',
-                               'CALC_KEY': 'divide', 'UB_TIME': '2000-04-01 00:00:00', 'DAY': '16', 'divide': '1.0'}
+                               'UB_TIME': '2000-04-01 00:00:00', 'DAY': '16', 'divide': '1.0'}
                     if o == constants.OutputFormatName.CSV_SHAPEFILE:
                         # This will have a unique geometry identifier to link with the shapefile.
                         desired = desired.copy()
@@ -1290,8 +1444,6 @@ class TestSimpleMPI(TestSimpleBase):
         # ocgis.env.VERBOSE = True
         # ocgis.env.DEBUG = True
         for ctr, k in enumerate(self.iter_product_keywords(keywords)):
-            # if ctr != 4: continue
-            # barrier_print(ctr, k)
             output_format = getattr(k, KeywordArgument.OUTPUT_FORMAT)
 
             if output_format in [constants.OutputFormatName.OCGIS, constants.OutputFormatName.NETCDF]:
@@ -1300,7 +1452,8 @@ class TestSimpleMPI(TestSimpleBase):
                 snippet = True
 
             rd = RequestDataset(path)
-            ops = OcgOperations(dataset=rd, geom=geom, prefix=output_format, output_format=output_format,
+            ops = OcgOperations(dataset=rd, geom=geom, agg_selection=True,
+                                prefix=output_format, output_format=output_format,
                                 snippet=snippet)
 
             dir_outputs = comm.gather(ops.dir_output)
@@ -1315,6 +1468,7 @@ class TestSimpleMPI(TestSimpleBase):
                 with vm.scoped_by_emptyable('global_extent', actual_field):
                     if not vm.is_null:
                         actual_extent = actual_field.grid.extent_global
+                        self.assertEqual(actual_field.grid.abstraction, GridAbstraction.POLYGON)
                         self.assertEqual(actual_extent, desired_extent)
 
                 data_in = actual_field.data_variables[0]
@@ -1341,12 +1495,8 @@ class TestSimpleMultivariate(TestSimpleBase):
         return RequestDataset(**ds)
 
     def get_multiple_request_datasets(self):
-        rd1 = self.get_request_dataset()
-        rd1._field_name = 'rd1'
-        rd1._rename_variable = ['f1', 'f2']
-        rd2 = self.get_request_dataset()
-        rd2._field_name = 'rd2'
-        rd2._rename_variable = ['ff1', 'ff2']
+        rd1 = self.get_request_dataset(rename_variable=['f1', 'f2'], field_name='rd1')
+        rd2 = self.get_request_dataset(rename_variable=['ff1', 'ff2'], field_name='rd2')
         return [rd1, rd2]
 
     def run_field_tst(self, field):
