@@ -16,11 +16,12 @@ from netCDF4 import netcdftime
 from numpy.core.multiarray import ndarray
 from numpy.ma import MaskedArray
 from shapely.geometry import Point
-from shapely.geometry.base import BaseGeometry
+from shapely.geometry.base import BaseGeometry, BaseMultipartGeometry
 from shapely.geometry.geo import mapping
 from shapely.geometry.polygon import Polygon
 from shapely.wkb import loads as wkb_loads
 
+from ocgis.constants import MPITag
 from ocgis.exc import SingleElementError, ShapeError, AllElementsMaskedError
 
 
@@ -87,6 +88,71 @@ def format_bool(value):
     return ret
 
 
+def arange_from_bool_ndarray(arr, start=0):
+    """
+    Create a collapsed range from ``start`` to the sum of ``True`` elements in ``arr`` plus one.
+
+    :param arr: The boolean array to use as a filter for the range creation.
+    :type arr: :class:`numpy.ndarray`
+    :param int start: The start index for the range array.
+    :return: An integer range array.
+    :rtype: :class:`numpy.ndarray`
+    """
+
+    npw = np.where(arr)[0]
+    npw += start
+    return npw
+
+
+def arange_from_dimension(dim, start=0, dtype=None, dist=True):
+    """
+    Create a sequential integer range similar to ``numpy.arange``. Call is collective across the current
+    :class:`~ocgis.OcgVM` if ``dist=True`` (the default).
+
+    :param dim: The dimension to use for creating the range.
+    :type dim: :class:`~ocgis.Dimension`
+    :param int start: The starting value for the range.
+    :param dtype: The data type for the output array.
+    :param bool dist: If ``True``, create range as a distributed array with a collective VM call. If ``False``, create
+     the array locally.
+    :rtype: :class:`numpy.ndarray`
+    """
+
+    if dtype is None:
+        from ocgis import env
+        dtype = env.NP_INT
+
+    local_size = len(dim)
+    if dist:
+        from ocgis import vm
+        for rank in vm.ranks:
+            dest_rank = rank + 1
+            if dest_rank == vm.size:
+                break
+            else:
+                if vm.rank == rank:
+                    data = np.array([start + local_size], dtype=dtype)
+                    buf = [data, vm.get_mpi_type(dtype)]
+                    vm.comm.Send(buf, dest=dest_rank, tag=MPITag.ARANGE_FROM_DIMENSION)
+                elif vm.rank == dest_rank:
+                    data = np.zeros(1, dtype=dtype)
+                    buf = [data, vm.get_mpi_type(dtype)]
+                    vm.comm.Recv(buf, source=rank, tag=MPITag.ARANGE_FROM_DIMENSION)
+                    start = data[0]
+                else:
+                    pass
+        vm.barrier()
+
+    ret = np.arange(start, start + local_size, dtype=dtype)
+
+    return ret
+
+
+def dict_first(target):
+    for element in target:
+        return element
+
+
 def get_added_slice(slice1, slice2):
     '''
     :param slice slice1:
@@ -245,6 +311,28 @@ def create_exact_field_value(longitude, latitude):
     return exact
 
 
+def create_ocgis_corners_from_esmf_corners(ecorners):
+    """
+    :param ecorners: An array of ESMF corners.
+    :type ecorners: :class:`numpy.ndarray`
+    :return: :class:`numpy.ndarray`
+    """
+    assert ecorners.ndim == 2
+
+    # ESMF corners have an extra row and column.
+    base_shape = [xx - 1 for xx in ecorners.shape]
+    grid_corners = np.zeros(base_shape + [4], dtype=ecorners.dtype)
+    # Upper left, upper right, lower right, lower left - this is an ideal order. Actual order may differ later.
+    slices = [(0, 0), (0, 1), (1, 1), (1, 0)]
+    for ii, jj in itertools.product(list(range(base_shape[0])), list(range(base_shape[1]))):
+        row_slice = slice(ii, ii + 2)
+        col_slice = slice(jj, jj + 2)
+        corners = ecorners[row_slice, col_slice]
+        for kk, slc in enumerate(slices):
+            grid_corners[ii, jj, kk] = corners[slc]
+    return grid_corners
+
+
 def create_zero_padded_integer(integer, nzeros):
     sint = str(integer)
     nstr_zeros = nzeros - len(sint)
@@ -349,7 +437,11 @@ def get_formatted_slice(slc, n_dims):
     def _format_singleton_(single_slc):
         # Avoid numpy typing by attempting to convert to an integer. Floats will be mangled, but whatever.
         try:
-            single_slc = int(single_slc)
+            do_int_conversion = True
+            if hasattr(single_slc, 'dtype') and single_slc.dtype == bool:
+                do_int_conversion = False
+            if do_int_conversion:
+                single_slc = int(single_slc)
         except:
             pass
 
@@ -506,12 +598,13 @@ def get_esmf_corners_from_ocgis_corners(ocorners, fill=None):
     """
     :param ocorners: Corners array with dimension (m, n, 4).
     :type ocorners: :class:`numpy.ma.core.MaskedArray`
-    :returns: An ESMF corners array with dimension (m + 1, n + 1).
+    :returns: An ESMF corners array with dimension (n + 1, m + 1).
     :rtype: :class:`numpy.ndarray`
     """
 
     if fill is None:
-        fill = np.zeros([element + 1 for element in ocorners.shape[0:2]], dtype=ocorners.dtype)
+        shp = np.flipud(ocorners.shape[0:2])
+        fill = np.zeros([element + 1 for element in shp], dtype=ocorners.dtype)
     range_row = list(range(ocorners.shape[0]))
     range_col = list(range(ocorners.shape[1]))
 
@@ -521,39 +614,14 @@ def get_esmf_corners_from_ocgis_corners(ocorners, fill=None):
         _corners = ocorners
 
     for ii, jj in itertools.product(range_row, range_col):
-        ref = fill[ii:ii + 2, jj:jj + 2]
+        ref = fill[jj:jj + 2, ii:ii + 2]
         ref[0, 0] = _corners[ii, jj, 0]
-        ref[0, 1] = _corners[ii, jj, 1]
+        ref[1, 0] = _corners[ii, jj, 1]
         ref[1, 1] = _corners[ii, jj, 2]
-        ref[1, 0] = _corners[ii, jj, 3]
+        ref[0, 1] = _corners[ii, jj, 3]
     return fill
 
 
-def get_ocgis_corners_from_esmf_corners(ecorners):
-    """
-    :param ecorners: An array of ESMF corners.
-    :type ecorners: :class:`numpy.ndarray`
-    :returns: A masked array of OCGIS corners.
-    :rtype: :class:`~numpy.ma.core.MaskedArray`
-    """
-
-    assert ecorners.ndim == 2
-
-    # ESMF corners have an extra row and column.
-    base_shape = [xx - 1 for xx in ecorners.shape]
-    grid_corners = np.zeros(base_shape + [4], dtype=ecorners.dtype)
-    # Upper left, upper right, lower right, lower left
-    slices = [(0, 0), (0, 1), (1, 1), (1, 0)]
-    for ii, jj in itertools.product(list(range(base_shape[0])), list(range(base_shape[1]))):
-        row_slice = slice(ii, ii + 2)
-        col_slice = slice(jj, jj + 2)
-        corners = ecorners[row_slice, col_slice]
-        for kk, slc in enumerate(slices):
-            grid_corners[ii, jj, kk] = corners[slc]
-    grid_corners = np.ma.array(grid_corners, mask=False)
-    return grid_corners
-
-
 def get_optimal_slice_from_array(arr, check_diff=True):
     if arr.dtype == bool:
         ret = arr
@@ -656,7 +724,6 @@ def get_trimmed_array_by_mask(arr, return_adjustments=False):
     :raises NotImplementedError:
     :returns: :class:`numpy.ma.MaskedArray` or (:class:`numpy.ma.MaskedArray', {'row':slice(...),'col':slice(...)})
     """
-
     assert arr.ndim <= 2
 
     has_col = False
@@ -674,52 +741,38 @@ def get_trimmed_array_by_mask(arr, return_adjustments=False):
 
     # Row 0 to end.
     start_row = 0
-    masked_rows = [start_row]
     for idx_row in range(arr.shape[0]):
-        if _mask[idx_row, ...].all():
-            start_row += 1
-            masked_rows.append(start_row)
-        else:
+        if not _mask[idx_row, ...].all():
+            start_row = idx_row
             break
-    start_row = max(masked_rows)
 
     # Row end to 0.
-    stop_row = _mask.shape[0]
-    masked_rows = [stop_row]
-    for adjust in range(arr.shape[0]):
-        adjust += 1
-        row_to_test = stop_row - adjust
-        if _mask[row_to_test, ...].all():
-            masked_rows.append(row_to_test)
-        else:
+    row_count = _mask.shape[0]
+    stop_row = row_count
+    for idx_row in range(row_count):
+        row_to_test = row_count - (idx_row + 1)
+        if not _mask[row_to_test, ...].all():
+            stop_row = row_to_test + 1
             break
-    stop_row = min(masked_rows)
 
     if arr.ndim == 2:
         has_col = True
 
         # col 0 to end.
         start_col = 0
-        masked_cols = [start_col]
         for idx_col in range(arr.shape[1]):
-            if _mask[:, idx_col].all():
-                start_col += 1
-                masked_cols.append(start_col)
-            else:
+            if not _mask[:, idx_col].all():
+                start_col = idx_col
                 break
-        start_col = max(masked_cols)
 
         # col end to 0.
-        stop_col = _mask.shape[1]
-        masked_cols = [stop_col]
-        for adjust in range(arr.shape[1]):
-            adjust += 1
-            col_to_test = stop_col - adjust
-            if _mask[:, col_to_test].all():
-                masked_cols.append(col_to_test)
-            else:
+        col_count = _mask.shape[1]
+        stop_col = col_count
+        for idx_col in range(col_count):
+            col_to_test = col_count - (idx_col + 1)
+            if not _mask[:, col_to_test].all():
+                stop_col = col_to_test + 1
                 break
-        stop_col = min(masked_cols)
 
     slc = [slice(start_row, stop_row)]
 
@@ -895,6 +948,80 @@ def reduce_multiply(sequence):
     return ret
 
 
+def reindex_and_minimize_variables_global(idxvar, datavar, start_index=0):
+    """
+    Reindex ``idxvar`` to start-based index ``start_index`` and minimize elements in ``datavar`` based on its index
+    presence in ``idxvar``.
+
+    :param idxvar: N-dimensional, integer variable containing indices.
+    :type idxvar: :class:`~ocgis.Variable`
+    :param datavar: One-dimensional variable containing data for indices in ``idxvar``.
+    :param int start_index: The start index to use for the first element in ``idxvar``.
+    :return: A two-element tuple with the first element being the re-indexed ``idxvar`` and the second element being
+     the minimized data indexed by the new ``idxvar``.
+    :rtype: tuple
+    """
+
+    raise NotImplementedError('work in progress')
+
+
+def create_index_array(src, dst, dtype=int, indices=None):
+    """
+    Index the values in ``src`` inside ``dst``. Returns an integer index array that will create the values in ``src``
+    if used to slice ``dst``.
+    
+    :param src: Array containing source values to index.
+    :type src: :class:`numpy.ndarray`
+    :param dst: Array containing the value database to index into.
+    :type dst: :class:`numpy.ndarray`
+    :param dtype: Output data type for the index array.
+    :type dtype: `NumPy integer type`
+    :rtype: :class:`numpy.ndarray`
+    """
+
+    src = np.array(src)
+    dst = np.array(dst)
+    ret = np.zeros_like(src, dtype=dtype)
+
+    for idx, src_value in enumerate(src.flat):
+        select = src_value == dst
+        idx_dst = np.where(select)[0][0]
+        if indices is not None:
+            a = indices[idx_dst]
+        else:
+            a = idx_dst
+        ret[idx] = a
+
+    return ret
+
+
+def create_index_variable_global(name, src, dst, dtype=int, start=0):
+    """
+    Create an index variable in parallel.
+    
+    Similar to :meth:`~ocgis.util.helpers.create_index_array` except :class:`~ocgis.Variable` objects are used and 
+    returned instead of :class:`numpy.ndarray` objects. This method will accept empty objects for both ``src`` and
+    ``dst``. If ``src`` is empty on the calling rank, then ``None`` will be returned.
+    
+    :param str name: Name of the output global index variable.
+    """
+
+    raise NotImplementedError('work in progress')
+
+    global_index = dst.create_ugid_global('global_index', start=start)
+
+    for src_value in src.get_value().flat:
+        tkk
+
+    # barrier_print(global_index.get_value())
+
+    # tkk
+
+    index_array = create_index_array(src.get_value(), dst.get_value(), dtype=dtype, indices=global_index.get_value())
+    ret = src.__class__(name=name, value=index_array, dimensions=src.dimensions)
+    return ret
+
+
 def set_name_attributes(name_mapping):
     """
     Set the name attributes on the keys of ``name_mapping``.
@@ -907,6 +1034,85 @@ def set_name_attributes(name_mapping):
             target.name = name
 
 
+def create_unique_global_array(arr):
+    """
+    Create a distributed NumPy array containing unique elements. If the rank has no unique items, an array with zero
+    elements will be returned. This call is collective across the current VM.
+
+    :param arr: Input array for unique operation.
+    :type arr: :class:`numpy.ndarray`
+    :rtype: :class:`numpy.ndarray`
+    :raises: ValueError
+    """
+
+    from ocgis import vm
+
+    if arr is None:
+        raise ValueError('Input must be a NumPy array.')
+
+    unique_local = np.unique(arr)
+    vm.barrier()
+
+    local_bounds = min(unique_local), max(unique_local)
+    lb_global = vm.gather(local_bounds)
+    lb_global = vm.bcast(lb_global)
+
+    # Find the vm ranks the local rank cares about. It cares if unique values have overlapping unique bounds.
+    overlaps = []
+    for rank, lb in enumerate(lb_global):
+        if rank == vm.rank:
+            continue
+        contains = []
+        for lb2 in local_bounds:
+            if lb[0] <= lb2 <= lb[1]:
+                to_app = True
+            else:
+                to_app = False
+            contains.append(to_app)
+        if any(contains) or (local_bounds[0] <= lb[0] and local_bounds[1] >= lb[1]):
+            overlaps.append(rank)
+
+    # Send out the overlapping sources.
+    tag_overlap = MPITag.OVERLAP_CHECK
+    tag_select_send_size = MPITag.SELECT_SEND_SIZE
+    vm.barrier()
+
+    # NumPy and MPI types.
+    np_type = unique_local.dtype
+    mpi_type = vm.get_mpi_type(np_type)
+
+    for o in overlaps:
+        if vm.rank != o and vm.rank < o:
+            dest_rank_bounds = lb_global[o]
+            select_send = np.logical_and(unique_local >= dest_rank_bounds[0], unique_local <= dest_rank_bounds[1])
+            u_src = unique_local[select_send]
+            select_send_size = u_src.size
+            _ = vm.comm.Isend([np.array([select_send_size], dtype=np_type), mpi_type], dest=o, tag=tag_select_send_size)
+            _ = vm.comm.Isend([u_src, mpi_type], dest=o, tag=tag_overlap)
+
+    # Receive and process conflicts to reduce the unique local values.
+    if vm.rank != 0:
+        for o in overlaps:
+            if vm.rank != o and vm.rank > o:
+                select_send_size = np.array([0], dtype=np_type)
+                req_select_send_size = vm.comm.Irecv([select_send_size, mpi_type], source=o, tag=tag_select_send_size)
+                req_select_send_size.wait()
+                select_send_size = select_send_size[0]
+
+                u_src = np.zeros(select_send_size.astype(int), dtype=np_type)
+                req = vm.comm.Irecv([u_src, mpi_type], source=o, tag=tag_overlap)
+                req.wait()
+
+                utokeep = np.ones_like(unique_local, dtype=bool)
+                for uidx, u in enumerate(unique_local.flat):
+                    if u in u_src:
+                        utokeep[uidx] = False
+                unique_local = unique_local[utokeep]
+
+    vm.barrier()
+    return unique_local
+
+
 def update_or_pass(target, key, value):
     """
     :param dict target: The target dictionary to update.
@@ -1069,14 +1275,6 @@ def get_temp_path(suffix='', name=None, nest=False, only_dir=False, wd=None, dir
     return str(ret)
 
 
-def get_local_to_global_slices(slices_global, slices_local):
-    # tdk: optimize: remove np.arange
-    ga = [np.arange(s.start, s.stop) for s in slices_global]
-    lm = [get_optimal_slice_from_array(ga[idx][slices_local[idx]]) for idx in range(len(slices_local))]
-    lm = tuple(lm)
-    return lm
-
-
 def get_group(ddict, keyseq, has_root=True, create=False, last=None):
     keyseq = deepcopy(keyseq)
     if last is None:
@@ -1116,3 +1314,51 @@ def get_or_create_dict(dct, key, default):
     if key not in dct:
         dct[key] = default
     return dct[key]
+
+
+def find_index(seqs, to_find):
+    """
+    Find the first matching index of a sequence of scalar values in ``to_find`` that match the first unique index
+    in the sequence of arrays in ``seq``.
+
+    :param seqs: A sequence of :class:`numpy.ndarray` objects with the same length as ``to_find``.
+    :type seqs: sequence
+    :param to_find: A sequence of scalar objects to search ``seqs`` for.
+    :type to_find: sequence
+    :return: The integer index value for ``to_find``'s location in ``seqs``.
+    :rtype: int
+    """
+    seqs = [np.array(s) for s in seqs]
+
+    selects = []
+    for seq_idx, seq in enumerate(seqs):
+        selects.append(seq == to_find[seq_idx])
+
+    reduced = None
+    for idx, curr_select in enumerate(selects):
+        if reduced is None:
+            reduced = curr_select
+        else:
+            reduced = np.logical_and(reduced, curr_select)
+
+    try:
+        found_idx = np.where(reduced)[0][0]
+    except IndexError:
+        found_idx = None
+
+    return found_idx
+
+
+def iter_exploded_geometries(geom):
+    """Yield single-part Shapely geometry objects."""
+
+    if isinstance(geom, BaseMultipartGeometry):
+        itr = iter(geom)
+    else:
+        itr = [geom]
+    for element1 in itr:
+        if isinstance(element1, BaseMultipartGeometry):
+            for element2 in iter_exploded_geometries(element1):
+                yield element2
+        else:
+            yield element1
