@@ -10,9 +10,9 @@ def itr_row(key, sequence):
 
 def itr_products_keywords(keywords, as_namedtuple=False):
     if as_namedtuple:
-        yld_tuple = namedtuple('ITesterKeywords', keywords.keys())
+        yld_tuple = namedtuple('ITesterKeywords', list(keywords.keys()))
 
-    iterators = [itr_row(ki, vi) for ki, vi in keywords.iteritems()]
+    iterators = [itr_row(ki, vi) for ki, vi in keywords.items()]
     for dictionaries in itertools.product(*iterators):
         yld = {}
         for dictionary in dictionaries:
