@@ -1,20 +1,22 @@
 import abc
 import itertools
 import tempfile
-from copy import copy, deepcopy
+from copy import deepcopy
 
 import numpy as np
 import six
 from fiona.crs import from_string, to_string
-from shapely.geometry import Point, Polygon
+from shapely.geometry import Point, Polygon, box
 from shapely.geometry.base import BaseMultipartGeometry
+from shapely.geometry.multipolygon import MultiPolygon
 
 from ocgis import constants
 from ocgis.base import AbstractInterfaceObject, raise_if_empty, AbstractOcgisObject
 from ocgis.constants import MPIWriteMode, WrappedState, WrapAction, KeywordArgument, CFName, OcgisUnits, \
-    ConversionFactor, DimensionMapKey
+    ConversionFactor, DimensionMapKey, DMK, OcgisConvention
 from ocgis.environment import osr
-from ocgis.exc import ProjectionCoordinateNotFound, ProjectionDoesNotMatch, CRSNotEquivalenError
+from ocgis.exc import ProjectionCoordinateNotFound, ProjectionDoesNotMatch, CRSNotEquivalenError, \
+    WrappedStateEvalTargetMissing, CRSDepthNotImplemented
 from ocgis.spatial.wrap import GeometryWrapper, CoordinateArrayWrapper
 from ocgis.util.helpers import iter_array, get_iter
 
@@ -22,7 +24,7 @@ SpatialReference = osr.SpatialReference
 
 
 @six.add_metaclass(abc.ABCMeta)
-class AbstractCRS(AbstractOcgisObject):
+class AbstractCRS(AbstractInterfaceObject):
     """
     Base class for all OCGIS coordinate systems. Intended to allow differentiation between standard PROJ.4 coordinate
     systems and specialized OCGIS-supported coordinate systems.
@@ -34,6 +36,9 @@ class AbstractCRS(AbstractOcgisObject):
     """
     _cf_attributes = None
     _cf_attributes_to_remove = ('units', 'standard_name')
+    _default_towgs84 = None
+    _fuzzy_value = None
+    _string_max_length_global = None
 
     def __init__(self, angular_units=OcgisUnits.DEGREES, linear_units=None):
         self._angular_units = angular_units
@@ -43,6 +48,11 @@ class AbstractCRS(AbstractOcgisObject):
     def angular_units(self):
         return self._angular_units
 
+    @property
+    def dist(self):
+        """Here for variable compatibility."""
+        return False
+
     @property
     def linear_units(self):
         return self._linear_units
@@ -51,12 +61,6 @@ class AbstractCRS(AbstractOcgisObject):
     def attrs(self):
         return {}
 
-    def copy(self):
-        return copy(self)
-
-    def deecopy(self):
-        return deepcopy(self)
-
     @property
     def has_initialized_parent(self):
         return False
@@ -100,32 +104,34 @@ class AbstractCRS(AbstractOcgisObject):
     def convert_to_empty(self):
         pass
 
-    def format_field(self, field, is_transform=False):
-        """
-        :param field: The field to format.
-        :type field: :class:`~ocgis.Field`
-        :param bool is_transform: If ``True``, this is a coordinate system tranformation format. CF attributes should be
-         removed. If ``False``, this is for a write and attributes should be left as is or overwritten if explicitly
-         defined by the coordinate system.
-        """
+    def create_dimension_map(self, group_metadata, strict=False):
+        from ocgis import DimensionMap
+        return DimensionMap()
 
+    def format_spatial_object(self, spatial_obj, is_transform=False):
+        """
+        :param spatial_obj: The spatial object to format.
+        :type spatial_obj: <varying>
+        :param bool is_transform: If ``True``, this is a coordinate system transformation format call. CF attributes
+         should be removed. If ``False``, this is for a write and attributes should be left as is or overwritten if
+         explicitly defined by the coordinate system.
+        """
         # No changes to geometry variables.
         from ocgis.variable.geom import GeometryVariable
-        if isinstance(field, GeometryVariable):
+        if isinstance(spatial_obj, GeometryVariable):
             return
 
         # Allows grids to be modified in addition to fields.
-        if hasattr(field, 'data_variables'):
-            for data_variable in field.data_variables:
-                field[data_variable.name].attrs[CFName.GRID_MAPPING] = self.name
+        if hasattr(spatial_obj, 'data_variables'):
+            for data_variable in spatial_obj.data_variables:
+                data_variable.attrs[CFName.GRID_MAPPING] = self.name
 
         # Add CF attributes to coordinate variables.
         if self._cf_attributes is None and is_transform:
-            targets = [field.x, field.y]
+            targets = spatial_obj.coordinate_variables
             for target in targets:
-                if target is not None:
-                    for attr_key in self._cf_attributes_to_remove:
-                        target.attrs.pop(attr_key, None)
+                for attr_key in self._cf_attributes_to_remove:
+                    target.attrs.pop(attr_key, None)
         elif self._cf_attributes is not None:
 
             def _update_attr_(target, name, value, clobber=False):
@@ -133,14 +139,43 @@ class AbstractCRS(AbstractOcgisObject):
                 if name not in attrs or (name in attrs and clobber):
                     attrs[name] = value
 
-            updates = {field.x: self._cf_attributes[DimensionMapKey.X],
-                       field.y: self._cf_attributes[DimensionMapKey.Y]}
+            updates = {}
+            for c in spatial_obj.coordinate_variables:
+                i = spatial_obj.dimension_map.inquire_is_xyz(c)
+                if i != DMK.LEVEL:
+                    updates[c] = self._cf_attributes[i]
 
             for target, name_values in updates.items():
                 if target is not None:
                     for k, v in name_values.items():
                         _update_attr_(target, k, v)
 
+    @classmethod
+    def fuzzy_check(cls, value):
+        """
+        Coordinate system definitions are often changing in interpreters. This function allows coordinate system
+        definitions to vary slightly, but still be created appropriately.
+
+        This returns ``True`` if there is a fuzzy match.
+
+        :param dict value: The coordinate system dictionary definition.
+        :return: bool
+        """
+        req = cls._fuzzy_value
+        ret = True
+        for k, v in req.items():
+            if k not in value:
+                ret = False
+                break
+            else:
+                if v != value[k]:
+                    ret = False
+                    break
+        if ret:
+            if cls._default_towgs84 is not None and 'towgs84' in value:
+                ret = value['towgs84'] == cls._default_towgs84
+        return ret
+
     @classmethod
     def get_wrap_action(cls, state_src, state_dst):
         """
@@ -186,8 +221,9 @@ class AbstractCRS(AbstractOcgisObject):
         :type target: :class:`~ocgis.Field`
         """
         # TODO: Wrapped state should operate on the x-coordinate variable vectors or geometries only.
+        # TODO: This should be a method on grids and geometry variables.
         from ocgis.collection.field import Field
-        from ocgis.spatial.grid import Grid
+        from ocgis.spatial.base import AbstractXYZSpatialContainer
         from ocgis import vm
 
         raise_if_empty(self)
@@ -197,26 +233,30 @@ class AbstractCRS(AbstractOcgisObject):
             ret = None
         else:
             if isinstance(target, Field):
-                if target.grid is not None:
-                    target = target.grid
+                grid = target.grid
+                if grid is not None:
+                    target = grid
                 else:
                     target = target.geom
 
             if target is None:
-                raise ValueError('Target has no spatial information to evaluate.')
+                raise WrappedStateEvalTargetMissing
             elif target.is_empty:
                 ret = None
-            elif isinstance(target, Grid):
+            elif isinstance(target, AbstractXYZSpatialContainer):
                 ret = self._get_wrapped_state_from_array_(target.x.get_value())
             else:
                 stops = (WrappedState.WRAPPED, WrappedState.UNWRAPPED)
                 ret = WrappedState.UNKNOWN
-                geoms = target.get_value().flat
+                geoms = target.get_masked_value().flat
+                _is_masked = np.ma.is_masked
+                _get_ws = self._get_wrapped_state_from_geometry_
                 for geom in geoms:
-                    flag = self._get_wrapped_state_from_geometry_(geom)
-                    if flag in stops:
-                        ret = flag
-                        break
+                    if not _is_masked(geom):
+                        flag = _get_ws(geom)
+                        if flag in stops:
+                            ret = flag
+                            break
 
         rets = vm.gather(ret)
         if vm.rank == 0:
@@ -233,36 +273,73 @@ class AbstractCRS(AbstractOcgisObject):
 
         return ret
 
-    def wrap_or_unwrap(self, action, target):
+    def prepare_geometry_variable(self, subset_geom, rhs_tol=10.0, inplace=True):
+        """
+        Prepared a geometry variable for subsetting. This method:
+
+        * Appropriately wraps subset geometries for spherical coordinate systems.
+
+        :param subset_geom: The geometry variable to prepare.
+        :type subset_geom: :class:`~ocgis.GeometryVariable`
+        :param float rhs_tol: The amount, in matching coordinate system units, to buffer the right hand selection
+         geometry.
+        :param bool inplace: If ``True``, modify the object in-place.
+        """
+        # TODO: This should work with arbitrary geometries not just bounding boxes.
+        assert subset_geom.size == 1
+        if self.is_wrappable and subset_geom.is_bbox and subset_geom.wrapped_state == WrappedState.UNWRAPPED:
+            if not inplace:
+                subset_geom = subset_geom.deepcopy()
+            svalue = subset_geom.get_value()
+            buffered_bbox = svalue[0].bounds
+            if buffered_bbox[0] < 0.:
+                bbox_rhs = list(deepcopy(buffered_bbox))
+                bbox_rhs[0] = buffered_bbox[0] + 360.
+                bbox_rhs[2] = 360. + rhs_tol
+
+                bboxes = [buffered_bbox, bbox_rhs]
+            else:
+                bboxes = [buffered_bbox]
+            bboxes = [box(*b) for b in bboxes]
+            if len(bboxes) > 1:
+                svalue[0] = MultiPolygon(bboxes)
+
+        return subset_geom
+
+    def set_string_max_length_global(self, value=None):
+        """Here for variable compatibility."""
+
+    def wrap_or_unwrap(self, action, target, force=False):
         from ocgis.variable.geom import GeometryVariable
         from ocgis.spatial.grid import Grid
 
         if action not in (WrapAction.WRAP, WrapAction.UNWRAP):
             raise ValueError('"action" not recognized: {}'.format(action))
 
-        if action == WrapAction.WRAP:
-            attr = 'wrap'
-        else:
-            attr = 'unwrap'
+        if target.wrapped_state != action or force:
+            if action == WrapAction.WRAP:
+                attr = 'wrap'
+            else:
+                attr = 'unwrap'
 
-        if isinstance(target, GeometryVariable):
-            w = GeometryWrapper()
-            func = getattr(w, attr)
-            target_value = target.get_value()
-            for idx, target_geom in iter_array(target_value, use_mask=True, return_value=True,
-                                               mask=target.get_mask()):
-                target_value.__setitem__(idx, func(target_geom))
-        elif isinstance(target, Grid):
-            ca = CoordinateArrayWrapper()
-            func = getattr(ca, attr)
-            func(target.x.get_value())
-            target.remove_bounds()
-            if target.has_allocated_point:
-                getattr(target.get_point(), attr)()
-            if target.has_allocated_polygon:
-                getattr(target.get_polygon(), attr)()
-        else:
-            raise NotImplementedError(target)
+            if isinstance(target, GeometryVariable):
+                w = GeometryWrapper()
+                func = getattr(w, attr)
+                target_value = target.get_value()
+                for idx, target_geom in iter_array(target_value, use_mask=True, return_value=True,
+                                                   mask=target.get_mask()):
+                    target_value.__setitem__(idx, func(target_geom))
+            elif isinstance(target, Grid):
+                ca = CoordinateArrayWrapper()
+                func = getattr(ca, attr)
+                func(target.x.get_value())
+                target.remove_bounds()
+                if target.has_allocated_point:
+                    getattr(target.get_point(), attr)()
+                if target.has_allocated_polygon:
+                    getattr(target.get_polygon(), attr)()
+            else:
+                raise NotImplementedError(target)
 
     @classmethod
     def _get_wrapped_state_from_array_(cls, arr):
@@ -338,6 +415,32 @@ class AbstractProj4CRS(AbstractCRS):
     Base class for coordinate systems that may be transformed using PROJ.4.
     """
 
+    @classmethod
+    def load_from_metadata(cls, group_metadata):
+        """
+        Create a coordinate system object using group metadata.
+
+        :param dict group_metadata: The metdata to interpret. This should be for the current group only.
+        :return: :class:`~ocgis.CRS`
+        """
+        names = []
+        vmeta = group_metadata['variables']
+        for k, v in vmeta.items():
+            vattrs = v.get('attrs', {})
+            if vattrs.get(OcgisConvention.Name.OCGIS_ROLE) == OcgisConvention.Value.ROLE_COORDSYS:
+                if vattrs.get(OcgisConvention.Name.PROJ4) is not None:
+                    names.append(k)
+
+        len_names = len(names)
+        if len_names > 1:
+            raise ValueError('Multiple coordinate system variables found: {}'.format(names))
+        elif len_names == 0:
+            raise ProjectionDoesNotMatch
+        else:
+            v = vmeta[names[0]]
+            ret = CRS(name=v['name'], proj4=v['attrs'][OcgisConvention.Name.PROJ4])
+        return ret
+
 
 class CoordinateReferenceSystem(AbstractProj4CRS, AbstractInterfaceObject):
     """
@@ -353,7 +456,7 @@ class CoordinateReferenceSystem(AbstractProj4CRS, AbstractInterfaceObject):
     :type name: str
     """
 
-    def __init__(self, value=None, proj4=None, epsg=None, name=constants.DEFAULT_COORDINATE_SYSTEM_NAME):
+    def __init__(self, value=None, proj4=None, epsg=None, name=OcgisConvention.Name.COORDSYS):
         self.name = name
         # Allows operations on data variables to look through an empty dimension list. Alleviates instance checking.
         self.dimensions = tuple()
@@ -408,7 +511,15 @@ class CoordinateReferenceSystem(AbstractProj4CRS, AbstractInterfaceObject):
             if self.sr.IsSame(other.sr) == 1:
                 ret = True
             else:
-                ret = False
+                # Try a value comparison.
+                if self.value == other.value:
+                    ret = True
+                else:
+                    # Try without the "wktext" flag.
+                    new_values = [self.value.copy(), other.value.copy()]
+                    for n in new_values:
+                        n.pop('wktext', None)
+                    ret = new_values[0] == new_values[1]
         except AttributeError:
             # likely a nonetype of other object type
             if other is None or not isinstance(other, self.__class__):
@@ -420,8 +531,14 @@ class CoordinateReferenceSystem(AbstractProj4CRS, AbstractInterfaceObject):
     def __ne__(self, other):
         return not self.__eq__(other)
 
-    def __str__(self):
-        return str(self.value)
+    # def __repr__(self):
+    #     msg = [str(type(self))]
+    #     elements = []
+    #     elements.append("name  = {}".format(self.name))
+    #     elements.append("value = {}".format(self.value))
+    #     elements = [' {}'.format(e) for e in elements]
+    #     msg.extend(elements)
+    #     return '\n'.join(msg)
 
     @property
     def dtype(self):
@@ -474,7 +591,7 @@ class CoordinateReferenceSystem(AbstractProj4CRS, AbstractInterfaceObject):
     def convert_to_empty(self):
         pass
 
-    def extract(self):
+    def extract(self, *args, **kwargs):
         return self
 
     def get_mask(self):
@@ -484,7 +601,7 @@ class CoordinateReferenceSystem(AbstractProj4CRS, AbstractInterfaceObject):
         """Compatibility with variable."""
         pass
 
-    def write_to_rootgrp(self, rootgrp, with_proj4=False):
+    def write_to_rootgrp(self, rootgrp, with_proj4=True, **kwargs):
         """
         Write the coordinate system to an open netCDF file.
 
@@ -497,16 +614,24 @@ class CoordinateReferenceSystem(AbstractProj4CRS, AbstractInterfaceObject):
         """
         variable = rootgrp.createVariable(self.name, 'S1')
         if with_proj4:
-            variable.proj4 = self.proj4
+            setattr(variable, OcgisConvention.Name.PROJ4, self.proj4)
+            setattr(variable, OcgisConvention.Name.OCGIS_ROLE, OcgisConvention.Value.ROLE_COORDSYS)
         return variable
 
     def write(self, *args, **kwargs):
         write_mode = kwargs.pop('write_mode', MPIWriteMode.NORMAL)
-        with_proj4 = kwargs.pop('with_proj4', False)
+
+        # Let subclasses determine if proj4 should be written.
+        with_proj4 = kwargs.pop('with_proj4', None)
+
         # Fill operations set values on variables. Coordinate system variables have no inherent values constructed only
         # from attributes.
         if write_mode != MPIWriteMode.FILL:
-            return self.write_to_rootgrp(*args, with_proj4=with_proj4)
+            if with_proj4 is not None:
+                ret = self.write_to_rootgrp(*args, **kwargs)
+            else:
+                ret = self.write_to_rootgrp(*args)
+            return ret
 
 
 class CRS(CoordinateReferenceSystem):
@@ -516,19 +641,19 @@ class CRS(CoordinateReferenceSystem):
 @six.add_metaclass(abc.ABCMeta)
 class AbstractSphericalCoordinateReferenceSystem(AbstractOcgisObject):
     _cf_attributes = {DimensionMapKey.X: {CFName.UNITS: 'degrees_east',
-                                          CFName.STANDARD_NAME: 'longitude'},
+                                          CFName.STANDARD_NAME: CFName.StandardName.LONGITUDE},
                       DimensionMapKey.Y: {CFName.UNITS: 'degrees_north',
-                                          CFName.STANDARD_NAME: 'latitude'}}
+                                          CFName.STANDARD_NAME: CFName.StandardName.LATITUDE}}
     is_wrappable = True
     is_geographic = True
 
-    def format_field(self, *args, **kwargs):
+    def format_spatial_object(self, *args, **kwargs):
         if self.angular_units == OcgisUnits.RADIANS:
             msg = '{} are not acceptable units for field formatting. Must be {}.'
             msg = msg.format(OcgisUnits.RADIANS, OcgisUnits.DEGREES)
             raise ValueError(msg)
 
-        super(AbstractSphericalCoordinateReferenceSystem, self).format_field(*args, **kwargs)
+        super(AbstractSphericalCoordinateReferenceSystem, self).format_spatial_object(*args, **kwargs)
 
     def transform_coordinates(self, other_crs, x, y, z, inverse=False):
         """
@@ -659,9 +784,11 @@ class Spherical(AbstractSphericalCoordinateReferenceSystem, CoordinateReferenceS
     :param units: Either degrees or radians.
     :type units: :class:`ocgis.constants.OcgisUnits`
     """
+    _default_towgs84 = '0,0,0,0,0,0,0'
+    _fuzzy_value = {'a': 6370997, 'b': 6370997, 'proj': 'longlat'}
 
     def __init__(self, semi_major_axis=6370997.0, angular_units=OcgisUnits.DEGREES):
-        value = {'proj': 'longlat', 'towgs84': '0,0,0,0,0,0,0', 'no_defs': '', 'a': semi_major_axis,
+        value = {'proj': 'longlat', 'towgs84': self._default_towgs84, 'no_defs': '', 'a': semi_major_axis,
                  'b': semi_major_axis}
         CoordinateReferenceSystem.__init__(self, value=value, name='latitude_longitude')
         AbstractCRS.__init__(self, angular_units=angular_units)
@@ -718,6 +845,8 @@ class WGS84(CoordinateReferenceSystem):
                                           CFName.STANDARD_NAME: 'longitude'},
                       DimensionMapKey.Y: {CFName.UNITS: 'degrees_north',
                                           CFName.STANDARD_NAME: 'latitude'}}
+    _default_towgs84 = '0,0,0,0,0,0,0'
+    _fuzzy_value = {'datum': 'WGS84', 'proj': 'longlat'}
 
     def __init__(self):
         value = {'no_defs': True, 'datum': 'WGS84', 'proj': 'longlat', 'towgs84': '0,0,0,0,0,0,0'}
@@ -836,8 +965,8 @@ class CFCoordinateReferenceSystem(CoordinateReferenceSystem):
             if not strict:
                 r_grid_mapping = None
                 fuzzy_names = cls.get_fuzzy_names()
-                for var_meta in list(meta['variables'].values()):
-                    if var_meta['name'] in fuzzy_names:
+                for var_name, var_meta in list(meta['variables'].items()):
+                    if var_meta.get('name', var_name) in fuzzy_names:
                         r_grid_mapping = var_meta
                         break
                 if r_grid_mapping is None:
@@ -857,9 +986,6 @@ class CFCoordinateReferenceSystem(CoordinateReferenceSystem):
         else:
             pc_x, pc_y = None, None
 
-        # this variable name is used by the netCDF converter
-        # meta['grid_mapping_variable_name'] = r_grid_mapping['name']
-
         kwds = r_grid_mapping['attrs'].copy()
         kwds.pop('grid_mapping_name', None)
         kwds['projection_x_coordinate'] = pc_x
@@ -876,7 +1002,7 @@ class CFCoordinateReferenceSystem(CoordinateReferenceSystem):
     def _load_from_metadata_finalize_(cls, kwds, var, meta):
         pass
 
-    def write_to_rootgrp(self, rootgrp, with_proj4=False):
+    def write_to_rootgrp(self, rootgrp, with_proj4=False, **kwargs):
         variable = super(CFCoordinateReferenceSystem, self).write_to_rootgrp(rootgrp, with_proj4=with_proj4)
         variable.grid_mapping_name = self.grid_mapping_name
         for k, v in self.map_parameters_values.items():
@@ -897,13 +1023,33 @@ class CFSpherical(Spherical, CFCoordinateReferenceSystem):
         Spherical.__init__(self, *args, **kwargs)
 
     @classmethod
-    def load_from_metadata(cls, var, meta):
-        r_grid_mapping = meta['variables'][var]['attrs'].get('grid_mapping')
-        r_grid_mapping_name = meta['variables'][var]['attrs'].get('grid_mapping_name')
-        if cls.grid_mapping_name in (r_grid_mapping, r_grid_mapping_name):
-            return cls()
+    def load_from_metadata(cls, var, meta, strict=False, depth=1):
+        variables = meta['variables']
+        if depth == 1:
+            variable_attrs = variables[var].get('attrs', {})
+            r_grid_mapping = variable_attrs.get('grid_mapping')
+            r_grid_mapping_name = variable_attrs.get('grid_mapping_name')
+            if cls.grid_mapping_name in (r_grid_mapping, r_grid_mapping_name):
+                return cls()
+            else:
+                raise ProjectionDoesNotMatch
+        elif depth == 2:
+            # Check for standard names on variables as spherical is often not defined in the metadata file.
+            found = {}
+            for k, v in cls._cf_attributes.items():
+                sn = v[CFName.STANDARD_NAME]
+                found[sn] = []
+                for vname, vmeta in variables.items():
+                    if vmeta['attrs'].get(CFName.STANDARD_NAME) == sn:
+                        found[sn].append(vname)
+            # There should be only one variable found for each standard name.
+            counts = [len(v) for v in found.values()]
+            if any([c > 1 for c in counts]) or any([c == 0 for c in counts]):
+                raise ProjectionDoesNotMatch
+            else:
+                return cls()
         else:
-            raise ProjectionDoesNotMatch
+            raise CRSDepthNotImplemented(depth)
 
 
 class CFAlbersEqualArea(CFCoordinateReferenceSystem):
@@ -988,9 +1134,9 @@ class CFRotatedPole(CFCoordinateReferenceSystem):
 
     def update_with_rotated_pole_transformation(self, grid, inverse=False):
         """
-        :type spatial: :class:`ocgis.interface.base.dimension.spatial.SpatialDimension`
+        :type grid: :class:`~ocgis.Grid`
         :param bool inverse: If ``True``, this is an inverse transformation.
-        :rtype: :class:`ocgis.interface.base.dimension.spatial.SpatialDimension`
+        :rtype: :class:`~ocgis.Grid`
         """
         assert grid.crs.is_geographic or isinstance(grid.crs, CFRotatedPole)
 
@@ -1015,6 +1161,34 @@ class CFRotatedPole(CFCoordinateReferenceSystem):
         return variable
 
 
+def create_crs(value, **kwargs):
+    """
+    Create a coordinate system object from a dictionary definition. This will create a spherical coordinate system
+    object, for example, if there is a fuzzy match with that coordinate system.
+
+    :param dict value: The coordinate system's dictionary definition.
+    :param dict kwargs: Optional keyword arguments to the coordinate system's creation.
+    :return: :class:`ocgis.variable.crs.AbstractCRS`
+    """
+    if value is None:
+        ret = None
+    else:
+        if isinstance(value, AbstractCRS):
+            value = value.value
+        to_check = np.array([Spherical, WGS84], dtype=object)
+        match = np.zeros(len(to_check), dtype=bool)
+        for idx, t in enumerate(to_check):
+            match[idx] = t.fuzzy_check(value)
+        found = to_check[match]
+        if found.size > 1:
+            raise ValueError('Multiple coordinate systems found: {}'.format(found))
+        elif found.size == 0:
+            ret = CoordinateReferenceSystem(value=value, **kwargs)
+        else:
+            ret = found[0]()
+    return ret
+
+
 def get_lonlat_rotated_pole_transform(lon, lat, transform, inverse=False, is_vectorized=False):
     """
     Transform longitude and latitude coordinates to/from their rotated pole representation.
