@@ -1,29 +1,32 @@
-import abc
-from abc import abstractmethod
 from collections import deque
 from copy import deepcopy
 from itertools import product
 
 import numpy as np
-import six
 from numpy.core.multiarray import ndarray
 from shapely import wkb
 from shapely.geometry import Point, Polygon, MultiPolygon, mapping, MultiPoint, box
 from shapely.geometry.base import BaseGeometry
+from shapely.geometry.polygon import orient
 from shapely.ops import cascaded_union
 from shapely.prepared import prep
 
-from ocgis import SourcedVariable, Variable, vm
+from ocgis import Variable, vm
 from ocgis import constants
 from ocgis import env
-from ocgis.base import AbstractInterfaceObject, get_dimension_names, get_variable_names, raise_if_empty
 from ocgis.base import AbstractOcgisObject
-from ocgis.constants import WrapAction, KeywordArgument, HeaderName
+from ocgis.base import get_dimension_names, get_variable_names, raise_if_empty
+from ocgis.constants import KeywordArgument, HeaderName, VariableName, DimensionName, ConversionTarget, DriverKey, \
+    WrappedState
 from ocgis.environment import ogr
-from ocgis.exc import EmptySubsetError
-from ocgis.util.helpers import iter_array, get_trimmed_array_by_mask, get_swap_chain
-from ocgis.variable.base import AbstractContainer, get_dimension_lengths
+from ocgis.exc import EmptySubsetError, RequestableFeature, NoInteriorsError
+from ocgis.spatial.base import AbstractSpatialVariable, create_split_polygons
+from ocgis.util.addict import Dict
+from ocgis.util.helpers import iter_array, get_trimmed_array_by_mask, get_swap_chain, find_index, \
+    iter_exploded_geometries, get_iter
+from ocgis.variable.base import get_dimension_lengths, ObjectType
 from ocgis.variable.crs import Cartesian
+from ocgis.variable.dimension import create_distributed_dimension, Dimension
 from ocgis.variable.iterator import Iterator
 
 CreateGeometryFromWkb, Geometry, wkbGeometryCollection, wkbPoint = ogr.CreateGeometryFromWkb, ogr.Geometry, \
@@ -32,139 +35,81 @@ CreateGeometryFromWkb, Geometry, wkbGeometryCollection, wkbPoint = ogr.CreateGeo
 GEOM_TYPE_MAPPING = {'Polygon': Polygon, 'Point': Point, 'MultiPoint': MultiPoint, 'MultiPolygon': MultiPolygon}
 
 
-class AbstractSpatialObject(AbstractInterfaceObject):
-    """
-    :keyword crs: (``=None``) Coordinate reference system for the spatial object.
-    :type crs: :class:`ocgis.variable.crs.AbstractCRS`
-    """
+class GeometrySplitter(AbstractOcgisObject):
+    _buffer_split = 1e-6
 
-    def __init__(self, *args, **kwargs):
-        self._crs_name = None
-        self.crs = kwargs.pop('crs', None)
-        super(AbstractInterfaceObject, self).__init__(*args, **kwargs)
+    def __init__(self, geometry):
+        self.geometry = geometry
+
+        if self.interior_count == 0:
+            raise NoInteriorsError
 
     @property
-    def crs(self):
-        if self.parent is not None and self._crs_name is not None:
-            ret = self.parent.get(self._crs_name)
-        else:
-            ret = None
+    def interior_count(self):
+        try:
+            ret = len(self.geometry.interiors)
+        except AttributeError:
+            ret = 0
+            for g in self.geometry:
+                ret += len(g.interiors)
         return ret
 
-    @crs.setter
-    def crs(self, value):
-        if value is None:
-            if self.crs is not None:
-                self.parent.pop(self._crs_name)
-                self._crs_name = None
-        else:
-            if self.parent is None:
-                self.initialize_parent()
-            if self._crs_name is not None:
-                self.parent.pop(self._crs_name)
-            self.parent.add_variable(value, force=True)
-            self._crs_name = value.name
-            value.format_field(self)
+    def create_split_vector_dict(self, interior):
+        minx, miny, maxx, maxy = self.geometry.buffer(self._buffer_split).bounds
+        col_key = 'cols'
+        row_key = 'rows'
+        ret = {}
 
-    @property
-    def wrapped_state(self):
-        raise_if_empty(self)
+        icx, icy = interior.centroid.x, interior.centroid.y
+
+        ret[col_key] = (minx, icx, maxx)
+        ret[row_key] = (miny, icy, maxy)
 
-        if self.crs is None:
-            ret = None
-        else:
-            ret = self.crs.get_wrapped_state(self)
         return ret
 
-    def unwrap(self):
-        raise_if_empty(self)
+    def create_split_polygons(self, interior):
+        split_dict = self.create_split_vector_dict(interior)
+        cols = split_dict['cols']
+        rows = split_dict['rows']
 
-        if self.crs is None or not self.crs.is_geographic:
-            raise ValueError("Only spherical coordinate systems may be wrapped/unwrapped.")
+        ul = box(cols[0], rows[0], cols[1], rows[1])
+        ur = box(cols[1], rows[0], cols[2], rows[1])
+        lr = box(cols[1], rows[1], cols[2], rows[2])
+        ll = box(cols[0], rows[1], cols[1], rows[2])
+
+        return ul, ur, lr, ll
+
+    def split(self, is_recursing=False):
+        geometry = self.geometry
+        if is_recursing:
+            assert isinstance(geometry, Polygon)
+            ret = []
+            for interior in self.iter_interiors():
+                split_polygon = self.create_split_polygons(interior)
+                for sp in split_polygon:
+                    ret.append(geometry.intersection(sp))
         else:
-            self.crs.wrap_or_unwrap(WrapAction.UNWRAP, self)
+            if isinstance(geometry, MultiPolygon):
+                itr = geometry
+            else:
+                itr = [geometry]
 
-    def wrap(self):
-        raise_if_empty(self)
+            ret = []
+            for geometry_part in itr:
+                try:
+                    geometry_part_splitter = self.__class__(geometry_part)
+                except NoInteriorsError:
+                    ret.append(geometry_part)
+                else:
+                    split = geometry_part_splitter.split(is_recursing=True)
+                    for element in split:
+                        ret.append(element)
 
-        if self.crs is None or not self.crs.is_geographic:
-            raise ValueError("Only spherical coordinate systems may be wrapped/unwrapped.")
-        else:
-            self.crs.wrap_or_unwrap(WrapAction.WRAP, self)
+        return MultiPolygon(ret)
 
-
-@six.add_metaclass(abc.ABCMeta)
-class AbstractOperationsSpatialObject(AbstractSpatialObject):
-    @property
-    def envelope(self):
-        return box(*self.extent)
-
-    @property
-    def extent(self):
-        return self._get_extent_()
-
-    @abstractmethod
-    def update_crs(self, to_crs):
-        """Update coordinate system in-place."""
-        raise_if_empty(self)
-
-        if self.crs is None:
-            msg = 'The current CRS is None and cannot be updated.'
-            raise ValueError(msg)
-
-    @abstractmethod
-    def _get_extent_(self):
-        """
-        :returns: A tuple with order (minx, miny, maxx, maxy).
-        :rtype: tuple
-        """
-
-    @abstractmethod
-    def get_intersects(self, *args, **kwargs):
-        """Perform an intersects operations."""
-
-    @abstractmethod
-    def get_nearest(self, target, return_indices=False):
-        """Get nearest element to target geometry."""
-
-    @abstractmethod
-    def get_spatial_index(self):
-        """Get the spatial index."""
-
-    @abstractmethod
-    def iter_records(self, use_mask=True):
-        """Generate fiona-compatible records."""
-
-
-@six.add_metaclass(abc.ABCMeta)
-class AbstractSpatialContainer(AbstractContainer, AbstractOperationsSpatialObject):
-    def __init__(self, **kwargs):
-        crs = kwargs.pop('crs', None)
-        parent = kwargs.pop('parent', None)
-        name = kwargs.pop('name', None)
-        AbstractContainer.__init__(self, name, parent=parent)
-        AbstractOperationsSpatialObject.__init__(self, crs=crs)
-
-
-@six.add_metaclass(abc.ABCMeta)
-class AbstractSpatialVariable(SourcedVariable, AbstractOperationsSpatialObject):
-    def __init__(self, **kwargs):
-        crs = kwargs.pop('crs', None)
-        SourcedVariable.__init__(self, **kwargs)
-        AbstractOperationsSpatialObject.__init__(self, crs=crs)
-
-    def deepcopy(self, eager=False):
-        ret = super(AbstractSpatialVariable, self).deepcopy(eager)
-        if ret.crs is not None:
-            ret.crs = ret.crs.deepcopy()
-        return ret
-
-    def extract(self, **kwargs):
-        crs = self.crs
-        ret = super(AbstractSpatialVariable, self).extract(**kwargs)
-        if crs is not None:
-            self.parent.add_variable(crs)
-        return ret
+    def iter_interiors(self):
+        for interior in self.geometry.interiors:
+            yield interior
 
 
 class GeometryProcessor(AbstractOcgisObject):
@@ -244,20 +189,22 @@ class GeometryVariable(AbstractSpatialVariable):
     :type crs: :class:`~ocgis.variable.crs.AbstractCRS`
     :param str geom_type: (``='auto'``) See http://toblerity.org/shapely/manual.html#object.geom_type. If ``'auto'``,
      the geometry type will be automatically determined from the object array. Providing a default prevents iterating
-     over the obejct array to identify the geometry type.
+     over the object array to identify the geometry type.
     :param ugid: (``=None``) An integer array with same shape as the geometry variable. This array will be converted to
      a :class:`~ocgis.Variable`.
     :type ugid: :class:`numpy.ndarray`
+    :param bool is_bbox: If ``True``, treat the polygon geometry as a bounding box geometry.
     """
 
     def __init__(self, **kwargs):
-        self._name_ugid = None
+        kwargs = kwargs.copy()
         self._geom_type = kwargs.pop(KeywordArgument.GEOM_TYPE, 'auto')
 
         if kwargs.get(KeywordArgument.NAME) is None:
-            kwargs[KeywordArgument.NAME] = 'geom'
+            kwargs[KeywordArgument.NAME] = VariableName.GEOMETRY_VARIABLE
 
         ugid = kwargs.pop(KeywordArgument.UGID, None)
+        self.is_bbox = kwargs.pop('is_bbox', False)
 
         super(GeometryVariable, self).__init__(**kwargs)
 
@@ -265,6 +212,11 @@ class GeometryVariable(AbstractSpatialVariable):
             ugid_var = Variable(name=HeaderName.ID_SELECTION_GEOMETRY, value=[ugid], dimensions=self.dimensions)
             self.set_ugid(ugid_var)
 
+        # If the parent field has no representative geometry, then this variable should be set as the representative
+        # geometry.
+        if self.parent.geom is None:
+            self.parent.set_geom(self)
+
     @property
     def area(self):
         """
@@ -329,15 +281,29 @@ class GeometryVariable(AbstractSpatialVariable):
             g = None
         return vm.bcast(g)
 
+    @property
+    def has_z(self):
+        """
+        Return ``True`` if the variable has a z-coordinate.
+
+        :rtype: bool
+        """
+
+        if self.is_empty:
+            ret = False
+        else:
+            ret = self.get_masked_value().compressed()[0].has_z
+        return ret
+
     @property
     def weights(self):
         """
         Weights are defined as:
-        
+
         >>> self.area / self.area.max()
-        
+
         Any geometries with zero area (points) are given an area of ``1.0`` for the purposes of weight calculation.
-        
+
         :return:  weights as a float masked array
         :rtype: :class:`numpy.ma.MaskedArray`
         """
@@ -346,67 +312,277 @@ class GeometryVariable(AbstractSpatialVariable):
         area.data[area.data == 0] = 1.0
         return area / area.max()
 
-    @property
-    def ugid(self):
+    def as_shapely(self):
         """
-        :return: unique identifier variable
-        :rtype: :class:`~ocgis.Variable`
+        Convert to a Shapely geometry provided this is a singleton geometry variable.
+
+        :rtype: :class:`shapely.geometry.base.BaseGeometry`
+        """
+        f = self.get_value().flatten()
+        assert f.size == 1
+        return f[0]
+
+    def convert_to(self, target=ConversionTarget.GEOMETRY_COORDS, **kwargs):
+        """
+        Convert to a target type. The returned object is orphaned (does not share a parent with the source).
+
+        Some common manipulations are shared between conversion targets.
+
+        * Always orients polygons CCW.
+
+        :param target: The target type.
+        :type target: :attr:`ocgis.constants.ConversionTarget`
+        :keyword dtype: ``(=None)`` Array data type for the coordinate variables.
+        :keyword xname: ``(=constants.DEFAULT_NAME_COL_COORDINATES)`` Name of the x-coordinate variable.
+        :keyword yname: ``(=constants.DEFAULT_NAME_ROW_COORDINATES)`` Name of the y-coordinate variable.
+        :keyword zname: ``(=constants.DEFAULT_NAME_LVL_COORDINATES)`` Name of the z-coordinate variable.
+        :keyword node_dim_name: ``(='n_node')`` Name of the node count dimension.
+        :keyword element_index_name: ``(='element_index')`` Name of the element node connectivity variable.
+        :keyword pack: ``(=True)`` If ``True``, de-duplicate coordinate vectors.
+        :keyword repeat_last_node: ``(=False)`` If ``False``, do not repeat the last node coordinate for polygon
+         geometries.
+        :keyword max_element_coords: ``(=None)`` If provided, the maximum number of coordinates across all polygon
+         geometries to convert. This fixes the column count for element node connectivity arrays. Otherwise, ragged
+         arrays are used.
+        :type max_element_coords: int
+        :keyword multi_break_value: ``(=constants.OcgisConvention.MULTI_BREAK_VALUE)`` Value to use for indicating a
+         multi-geometry break (indicates a separation of elements. Value must be negative.
+        :type multi_break_value: int
+        :keyword int node_threshold: ``(=None)`` Split polygons with nodes counts greater than this value into
+         multi-polygons.
+        :keyword bool split_interiors: ``(=False)`` If ``True``, split polygons with holes/interiors into
+         multi-polygons.
+        :keyword str driver: ``(driver=constants.DriverKey.NETCDF_UGRID)`` The driver to use for the output object.
+        :keyword bool use_geometry_iterator: ``(=False)`` If ``True``, use a geometry iterator instead of loading all
+         the geometries from source.
+        :keyword int start_index: ``(=0)`` The start index to use for coordinate indexing.
         """
 
-        if self._name_ugid is None:
-            return None
-        else:
-            return self.parent[self._name_ugid]
-
-    def create_ugid(self, name, start=1):
-        """
-        Create a unique identifier variable for the geometry variable. The returned variable will have the same
-        dimensions.
-        
-        :param str name: The name for the new geometry variable. 
-        :param int start: Starting value for the unique identifier. 
-        :rtype: :class:`~ocgis.Variable`
-        """
-
-        if self.is_empty:
-            value = None
-        else:
-            value = np.arange(start, start + self.size).reshape(self.shape)
-        ret = Variable(name=name, value=value, dimensions=self.dimensions, is_empty=self.is_empty)
-        self.set_ugid(ret)
-        return ret
-
-    def create_ugid_global(self, name, start=1):
-        """
-        Same as :meth:`~ocgis.GeometryVariable.create_ugid` but collective across the current :class:`~ocgis.OcgVM`.
-        
-        :raises: :class:`~ocgis.exc.EmptyObjectError`
-        """
+        # TODO: IMPLEMENT: Line conversion.
+        # TODO: IMPLEMENT: Storage method for holes/interiors. Interiors are currently only split not stored.
+        # TODO: OPTIMIZE: Append/extend array creation. Try to use pre-determined sizes for all arrays where possible.
 
         raise_if_empty(self)
 
-        sizes = vm.gather(self.size)
-        if vm.rank == 0:
-            start = start
-            for idx, n in enumerate(vm.ranks):
-                if n == vm.rank:
-                    rank_start = start
-                else:
-                    vm.comm.send(start, dest=n)
-                start += sizes[idx]
-        else:
-            rank_start = vm.comm.recv(source=0)
+        from ocgis.spatial.geomc import PointGC, PolygonGC
+        from ocgis.driver.registry import get_driver_class
 
-        return self.create_ugid(name, start=rank_start)
+        assert self.ndim == 1
+
+        kwargs = kwargs.copy()
+        dtype = kwargs.pop(KeywordArgument.DTYPE, None)
+        xname = kwargs.pop('xname', constants.DEFAULT_NAME_COL_COORDINATES)
+        yname = kwargs.pop('yname', constants.DEFAULT_NAME_ROW_COORDINATES)
+        zname = kwargs.pop('zname', constants.DEFAULT_NAME_LVL_COORDINATES)
+        node_dim_name = kwargs.pop('node_dim_name', 'n_node')
+        element_index_name = kwargs.pop('element_index_name', 'element_index')
+        pack = kwargs.pop('pack', True)
+        repeat_last_node = kwargs.pop('repeat_last_node', False)
+        max_element_coords = kwargs.pop('max_element_coords', None)
+        ocgis_convention = constants.OcgisConvention
+        name_mbv = ocgis_convention.Name.MULTI_BREAK_VALUE
+        multi_break_value = kwargs.pop(name_mbv, ocgis_convention.Value.MULTI_BREAK_VALUE)
+        node_threshold = kwargs.pop('node_threshold', None)
+        split_interiors = kwargs.pop('split_interiors', False)
+        driver = get_driver_class(kwargs.pop('driver', None), default=DriverKey.NETCDF_UGRID)
+        use_geometry_iterator = kwargs.pop('use_geometry_iterator', False)
+        to_crs = kwargs.pop('to_crs', None)
+        start_index = kwargs.pop('start_index', 0)
+        assert len(kwargs) == 0
+        if to_crs is not None and not use_geometry_iterator:
+            raise ValueError("'to_crs' only applies when using a geometry iterator")
+
+        polygon_types = ('Polygon', 'MultiPolygon')
+
+        geom_type = self.geom_type
+        # Flag to indicate if we are processing multi-geometries.
+        if geom_type.lower().startswith('multi'):
+            is_multi = True
+        else:
+            is_multi = False
+
+        if target == ConversionTarget.GEOMETRY_COORDS:
+            if use_geometry_iterator:
+                has_z_itr = self._request_dataset.driver.get_variable_value(self, as_geometry_iterator=True)
+                for g in has_z_itr:
+                    has_z = g.has_z
+                    break
+                geom_itr = self._request_dataset.driver.get_variable_value(self, as_geometry_iterator=True)
+            else:
+                has_z = self.has_z
+                geom_itr = self.get_value().flat
+            if geom_type == 'Point':
+                if has_z:
+                    ndim = 3
+                else:
+                    ndim = 2
+                fill = np.zeros([self.size, ndim], dtype=dtype)
+                for idx, geom in enumerate(geom_itr):
+                    fill[idx] = np.array(geom)
+                xv = fill[:, 0]
+                yv = fill[:, 1]
+                if has_z:
+                    zv = fill[:, 2]
+                else:
+                    zv = None
+            elif geom_type in polygon_types:
+                # This array holds indices pointing to coordinate arrays. Supplying "max_element_coords" sets the size
+                # of each element's coordinate count.
+                if max_element_coords is not None:
+                    element_index = np.zeros((self.size, max_element_coords), dtype=env.NP_INT)
+                    ocgis_dtype = env.NP_INT
+                else:
+                    element_index = np.zeros(self.size, dtype=object)
+                    ocgis_dtype = ObjectType(env.NP_INT)
+
+                # The start index for the element node connectivity array.
+                cidx = start_index
+
+                xv = deque()
+                yv = deque()
+                zv = deque()
+                seqs = [xv, yv]
+                if has_z:
+                    seqs.append(zv)
+
+                if to_crs is not None:
+                    from_crs = self._request_dataset.crs
+
+                for idx, geom in enumerate(geom_itr):
+                    if to_crs is not None:
+                        to_transform = GeometryVariable.from_shapely(geom, crs=from_crs)
+                        to_transform.update_crs(to_crs)
+                        geom = to_transform.get_value()[0]
+
+                    if geom.geom_type in polygon_types:
+                        try:
+                            gsplitter = GeometrySplitter(geom)
+                        except NoInteriorsError:
+                            pass
+                        else:
+                            if split_interiors:
+                                geom = gsplitter.split()
+                            else:
+                                raise ValueError('Interiors are not handled unless they are split.')
+
+                        if node_threshold is not None and get_node_count(geom) > node_threshold:
+                            geom = get_split_polygon_by_node_threshold(geom, node_threshold)
+
+                    fill_cidx = np.array([], dtype=env.NP_INT)
+
+                    for subidx, subgeom in enumerate(iter_exploded_geometries(geom)):
+                        # Insert a break value if we are on the second or greater component geometry of a
+                        # multi-geometry.
+                        if subidx > 0:
+                            fill_cidx = np.hstack((fill_cidx, np.array([multi_break_value], dtype=env.NP_INT)))
+
+                        subgeom = get_ccw_oriented_and_valid_shapely_polygon(subgeom)
+                        coords = np.array(subgeom.exterior.coords)
+
+                        if repeat_last_node:
+                            coords_shape = coords.shape[0]
+                        else:
+                            coords_shape = coords.shape[0] - 1
+
+                        if pack:
+                            curr_element_index = np.zeros(coords_shape, dtype=env.NP_INT)
+                            for coords_row_idx in range(coords_shape):
+                                coords_row = coords[coords_row_idx, :].flatten()
+                                found_index = find_index(seqs, coords_row)
+                                if found_index is None:
+                                    xv.append(coords_row[0])
+                                    yv.append(coords_row[1])
+                                    if has_z:
+                                        zv.append(coords_row[2])
+                                    found_index = cidx
+                                    cidx += 1
+                                else:
+                                    found_index += start_index
+                                curr_element_index[coords_row_idx] = found_index
+                            fill_cidx = np.hstack((fill_cidx, curr_element_index))
+                        else:
+                            xv.extend(coords[0:coords_shape, 0].flatten().tolist())
+                            yv.extend(coords[0:coords_shape, 1].flatten().tolist())
+                            if has_z:
+                                zv.extend(coords[0:coords_shape, 2].flatten().tolist())
+                            fill = np.arange(cidx, cidx + coords_shape, dtype=env.NP_INT)
+                            fill_cidx = np.hstack((fill_cidx, fill))
+                            cidx += coords_shape
+
+                    if max_element_coords is None:
+                        element_index[idx] = fill_cidx
+                    else:
+                        element_index[idx, :] = fill_cidx
+
+                if max_element_coords is None:
+                    element_index_dims = self.dimensions[0]
+                else:
+                    element_index_dims = [self.dimensions[0],
+                                          Dimension(name=DimensionName.UGRID_MAX_ELEMENT_COORDS,
+                                                    size=max_element_coords)]
+                element_index = Variable(name=element_index_name, value=element_index, dimensions=element_index_dims,
+                                         dtype=ocgis_dtype)
+
+                # Indicate there are multi-geometries in the coordinates objects.
+                if is_multi:
+                    element_index.attrs[name_mbv] = multi_break_value
+                else:
+                    element_index.attrs.pop(name_mbv, None)
+            else:
+                msg = "Conversion for this geometry type is not implemented: '{}'".format(geom_type)
+                raise RequestableFeature(message=msg)
+
+            names = [xname, yname, zname]
+            values = [xv, yv, zv]
+            variables = [None] * 3
+
+            if geom_type == 'Point':
+                dim = self.dimensions[0]
+            elif geom_type in polygon_types:
+                dim = create_distributed_dimension(len(xv), name=node_dim_name)
+            else:
+                raise NotImplementedError(geom_type)
+
+            for idx in range(len(names)):
+                if not has_z and idx == 2:
+                    break
+                variables[idx] = Variable(name=names[idx], value=values[idx], dimensions=dim)
+
+            if geom_type == 'Point':
+                klass = PointGC
+            elif geom_type in polygon_types:
+                klass = PolygonGC
+            else:
+                raise NotImplementedError(geom_type)
+
+            kwds = dict(z=variables[2], crs=self.crs, driver=driver, start_index=start_index)
+            if geom_type in polygon_types:
+                kwds['cindex'] = element_index
+                kwds['packed'] = pack
+
+            if self.has_mask:
+                kwds[KeywordArgument.MASK] = self.get_mask()
+            ret = klass(variables[0], variables[1], **kwds)
+        else:
+            raise RequestableFeature('This conversion target is not supported: {}'.format(target))
+        return ret
+
+    @classmethod
+    def from_shapely(cls, geom, **kwargs):
+        kwargs = kwargs.copy()
+        kwargs[KeywordArgument.VALUE] = [geom]
+        if KeywordArgument.DIMENSIONS not in kwargs:
+            kwargs[KeywordArgument.DIMENSIONS] = DimensionName.GEOMETRY_DIMENSION
+        return cls(**kwargs)
 
     def get_buffer(self, *args, **kwargs):
         """
         Return a shallow copy of the geometry variable with geometries buffered.
-        
+
         .. note:: Accepts all parameters to :meth:`shapely.geometry.base.BaseGeometry.buffer`.
-        
+
         An additional keyword argument is:
-        
+
         :keyword str geom_type: The geometry type for the new buffered geometry if known in advance.
         :rtype: :class:`~ocgis.GeometryVariable`
         :raises: :class:`~ocgis.exc.EmptyObjectError`
@@ -435,7 +611,7 @@ class GeometryVariable(AbstractSpatialVariable):
     def get_intersects(self, *args, **kwargs):
         """
         Perform an intersects spatial operations on the geometry variable.
-        
+
         :keyword bool return_slice: (``=False``) If ``True``, return the _global_ slice that will guarantee no masked
          elements outside the subset geometry as the second element in the return value.
         :keyword bool cascade: (``=True``) If ``True`` (the default), set the mask following the spatial operation on
@@ -478,9 +654,9 @@ class GeometryVariable(AbstractSpatialVariable):
 
         Additional arguments and/or keyword arguments are:
 
-        :keyword bool inplace: (``=False``) If ``False`` (the default), deep copy the geometry array on the output 
+        :keyword bool inplace: (``=False``) If ``False`` (the default), deep copy the geometry array on the output
          before executing an intersection. If ``True``, modify the geometries in-place.
-        :keyword bool intersects_check: (``=True``) If ``True`` (the default), first perform an intersects operation to 
+        :keyword bool intersects_check: (``=True``) If ``True`` (the default), first perform an intersects operation to
          limit the geometries tests for intersection. If ``False``, perform the intersection as is.
         """
 
@@ -560,7 +736,7 @@ class GeometryVariable(AbstractSpatialVariable):
         """
         :param target: The Shapely geometry to use for proximity.
         :type target: :class:`shapely.geometry.base.BaseGeometry`
-        :param bool return_indices: If ``True``, also return the indices used for slicing the geometry variable. 
+        :param bool return_indices: If ``True``, also return the indices used for slicing the geometry variable.
         :return: shallow copy of the geometry variable and optionally slices
         :rtype: :class:`~ocgis.GeometryVariable` | ``(<geometry variable>, <slice>)``
         """
@@ -614,11 +790,30 @@ class GeometryVariable(AbstractSpatialVariable):
 
         return si
 
+    def get_spatial_subset_operation(self, spatial_op, subset_geom, **kwargs):
+        if spatial_op == 'intersects':
+            ret = self.get_intersects(*[subset_geom], **kwargs)
+        elif spatial_op == 'intersection':
+            ret = self.get_intersection(*[subset_geom], **kwargs)
+        else:
+            raise NotImplementedError(spatial_op)
+        return ret
+
     def get_unioned(self, dimensions=None, union_dimension=None, spatial_average=None, root=0):
         """
-        Unions _unmasked_ geometry objects. Collective across the current :class:`~ocgis.OcgVM`.
+        Unions _unmasked_ geometry objects and applies spatial averaging weights to variables in the parent collection
+        if requested. Collective across the current :class:`~ocgis.OcgVM`.
+
+        :param dimensions: Dimensions to union. If ``None``, default to the object's dimensions.
+        :type dimensions: tuple(:class:`ocgis.Dimension`, ...) | tuple(str, ...)
+        :param union_dimension: The new dimension for the unioned geometry.
+        :type union_dimension: :class:`ocgis.Dimension` | str
+        :param spatial_average: The variables to spatially average. Other variables will be left untouched.
+        :type spatial_average: tuple(:class:`ocgis.Variable`, ...) | tuple(str, ...)
+        :param int root: If executing in parallel, the root rank to send all data. On non-root ranks, ``None`` will be
+         returned.
+        :rtype: :class:`ocgis.GeometryVariable`
         """
-        # TODO: optimize!
 
         # Get dimension names and lengths for the dimensions to union.
         if dimensions is None:
@@ -647,7 +842,7 @@ class GeometryVariable(AbstractSpatialVariable):
         if spatial_average is None:
             ret = ret.extract()
         ret.set_mask(None)
-        ret.set_value(None)
+        ret._value = None
         ret.set_dimensions(new_dimensions)
         ret.allocate_value()
 
@@ -717,7 +912,7 @@ class GeometryVariable(AbstractSpatialVariable):
                     # Prepare the spatially averaged variable.
                     target = ret.parent[var_to_weight.name]
                     target.set_mask(None)
-                    target.set_value(None)
+                    target._value = None
                     target.set_dimensions(new_dimensions)
                     target.allocate_value()
 
@@ -757,7 +952,7 @@ class GeometryVariable(AbstractSpatialVariable):
                     weighted_value = np.atleast_1d(np.ma.average(target_to_weight, weights=weights))
                     target = ret.parent[var_to_weight.name]
                     target.set_mask(None)
-                    target.set_value(None)
+                    target._value = None
                     target.set_dimensions(new_dimensions)
                     target.set_value(weighted_value)
 
@@ -811,6 +1006,51 @@ class GeometryVariable(AbstractSpatialVariable):
         else:
             return
 
+    def prepare(self, archetype=None):
+        """
+        Prepare the geometry variable for spatial operations by calling its coordinate system's :meth:`ocgis.variable.crs.AbstractCRS.prepare_geometry_variable`
+        method and returning a deep copy. If an archetype is provided, update the returned object's coordinate system and
+        wrapped state to match the archetype's. If the current object has no crs or no modifications are required by
+        the object, then a shallow copy is returned.
+
+        :param archetype: The object to use for spatial property matching.
+        :type archetype: :class:`ocgis.spatial.base.AbstractSpatialObject`
+        :return: :class:`~ocgis.GeometryVariable`
+        """
+
+        if self.size > 1:
+            raise RequestableFeature('Preparations only work on a single geometry.')
+
+        crs = self.crs
+        dced = False
+        if crs is not None or archetype is not None:
+            if crs is not None:
+                ret = self.deepcopy()
+                dced = True
+                ret = crs.prepare_geometry_variable(ret)
+
+            # Update the coordinate system if it differs from the archetype.
+            if archetype is not None:
+                acrs = archetype.crs
+                if acrs != crs:
+                    if not dced:
+                        ret = self.deepcopy()
+                        dced = True
+                    ret.update_crs(acrs)
+
+                # Update spatial wrapping if it is still applicable.
+                if acrs is not None:
+                    archetype_wrapped_state = archetype.wrapped_state
+                    if archetype_wrapped_state not in (WrappedState.UNKNOWN, None):
+                        if not dced:
+                            ret = self.deepcopy()
+                            dced = True
+                        acrs.wrap_or_unwrap(archetype_wrapped_state, ret)
+        else:
+            ret = self.copy()
+
+        return ret
+
     def update_crs(self, to_crs):
         """
         Update the coordinate system of the geometry variable in-place.
@@ -820,7 +1060,6 @@ class GeometryVariable(AbstractSpatialVariable):
         """
 
         super(GeometryVariable, self).update_crs(to_crs)
-
         members = [self.crs, to_crs]
         contains_cartesian = any([isinstance(ii, Cartesian) for ii in members])
 
@@ -859,21 +1098,12 @@ class GeometryVariable(AbstractSpatialVariable):
             feature = {'properties': {}, 'geometry': mapping(geom)}
             yield feature
 
-    def set_ugid(self, variable):
+    def set_ugid(self, variable, attr_link_name=constants.AttributeName.UNIQUE_GEOMETRY_IDENTIFIER):
         """
-        Set the unique identifier for the geometry variable.
-        
-        :param variable: The unqiue identifier variable.
-        :type variable: :class:`~ocgis.Variable` | ``None``
+        Same as :meth:`~ocgis.Variable.set_ugid`, except the unique identifier name has a default value.
         """
 
-        if variable is None:
-            self._name_ugid = None
-            self.attrs.pop(constants.AttributeName.UNIQUE_GEOMETRY_IDENTIFIER, None)
-        else:
-            self.parent.add_variable(variable, force=True)
-            self._name_ugid = variable.name
-            self.attrs[constants.AttributeName.UNIQUE_GEOMETRY_IDENTIFIER] = variable.name
+        super(GeometryVariable, self).set_ugid(variable, attr_link_name=attr_link_name)
 
     def set_value(self, value, **kwargs):
         if not isinstance(value, ndarray) and value is not None:
@@ -889,14 +1119,18 @@ class GeometryVariable(AbstractSpatialVariable):
         super(GeometryVariable, self).set_value(value, **kwargs)
 
     def write_vector(self, *args, **kwargs):
-        from ocgis.collection.field import Field
-        from ocgis.driver.vector import DriverVector
-        field = Field(geom=self, crs=self.crs)
-        kwargs[KeywordArgument.DRIVER] = DriverVector
-        field.write(*args, **kwargs)
+        kwargs = kwargs.copy()
+        lself = self.copy()
+        if lself.parent.geom is None:
+            lself.parent.set_geom(lself)
+        lself.parent.set_abstraction_geom(create_ugid=True, set_ugid_as_data=True)
+        kwargs[KeywordArgument.DRIVER] = DriverKey.VECTOR
+        lself.parent.write(*args, **kwargs)
 
     def _get_extent_(self):
-        raise NotImplementedError
+        if self.size > 1:
+            raise RequestableFeature('Extent not supported for more than one geometry.')
+        return self.get_value().flatten()[0].bounds
 
 
 def get_masking_slice(intersects_mask_value, target, apply_slice=True):
@@ -944,6 +1178,7 @@ def get_masking_slice(intersects_mask_value, target, apply_slice=True):
     else:
         global_slice = None
         raise_empty_subset = None
+
     raise_empty_subset = vm.bcast(raise_empty_subset)
     if raise_empty_subset:
         raise EmptySubsetError
@@ -990,6 +1225,82 @@ def get_grid_or_geom_attr(sc, attr):
     return ret
 
 
+def get_ccw_oriented_and_valid_shapely_polygon(geom):
+    try:
+        assert geom.is_valid
+    except AssertionError:
+        geom = geom.buffer(0)
+        assert geom.is_valid
+
+    if not geom.exterior.is_ccw:
+        geom = orient(geom)
+
+    return geom
+
+
+def get_node_schema(geom):
+    """Create a dictionary containing polygon metadata."""
+    ret = Dict()
+    for ctr, ii in enumerate(get_iter(geom, dtype=Polygon)):
+        ret[ctr].node_count = get_node_count(ii)
+        ret[ctr].area = ii.area
+        ret[ctr].geom = ii
+    return ret
+
+
+def get_node_count(geom):
+    node_count = 0
+    for ii in get_iter(geom, dtype=Polygon):
+        node_count += len(ii.exterior.coords)
+    return node_count
+
+
+def get_split_polygon_by_node_threshold(geom, node_threshold):
+    """Split a polygon by a node threshold."""
+    node_schema = get_node_schema(geom)
+
+    # Collect geometries with node counts higher than the threshold.
+    to_split = []
+    for k, v in node_schema.items():
+        if v['node_count'] > node_threshold:
+            to_split.append(k)
+
+    # Identify split parameters for an element exceeding the node threshold.
+    for ii in to_split:
+        n = node_schema[ii]
+        # Approximate number of splits need for each split element to be less than the node threshold.
+        n.n_splits = int(np.ceil(n['node_count'] / node_threshold))
+        # This is the shape of the polygon grid to use for splitting the target element.
+        n.split_shape = np.sqrt(n.n_splits)
+        # There should be at least two splits.
+        if n.split_shape == 1:
+            n.split_shape += 1
+        n.split_shape = tuple([int(np.ceil(ns)) for ns in [n.split_shape] * 2])
+
+        # Get polygons to use for splitting.
+        n.splitters = create_split_polygons(n['geom'], n.split_shape)
+
+        # Create the individual splits:
+        n.splits = []
+        for s in n.splitters:
+            if n.geom.intersects(s):
+                the_intersection = n.geom.intersection(s)
+                for ti in get_iter(the_intersection, dtype=Polygon):
+                    n.splits.append(ti)
+
+                    # write_fiona(n.splits, '01-splits')
+
+    # Collect the polygons to return as a multipolygon.
+    the_multi = []
+    for v in node_schema.values():
+        if 'splits' in v:
+            the_multi += v.splits
+        else:
+            the_multi.append(v.geom)
+
+    return MultiPolygon(the_multi)
+
+
 def geometryvariable_get_mask_from_intersects(gvar, geometry, use_spatial_index=env.USE_SPATIAL_INDEX,
                                               keep_touches=False, original_mask=None):
     # Create the fill array and reference the mask. This is the output geometry value array.
