@@ -9,7 +9,7 @@ import netcdftime
 import numpy as np
 import six
 
-from ocgis import constants, env
+from ocgis import constants, env, Dimension
 from ocgis.constants import HeaderName, KeywordArgument
 from ocgis.exc import EmptySubsetError, IncompleteSeasonError, CannotFormatTimeError, ResolutionError
 from ocgis.util.helpers import get_is_date_between, iter_array, get_none_or_slice
@@ -146,12 +146,12 @@ class TemporalVariable(SourcedVariable):
             ret = False
         return ret
 
-    def as_record(self, *args, **kwargs):
+    def _as_record_(self, *args, **kwargs):
         add_parts = kwargs.pop('add_parts', True)
         pytype_primitives = kwargs.get('pytype_primitives', False)
         if pytype_primitives:
             kwargs['formatter'] = str
-        ret = super(TemporalVariable, self).as_record(*args, **kwargs)
+        ret = super(TemporalVariable, self)._as_record_(*args, **kwargs)
         if add_parts and self.format_time:
             value = self._get_iter_value_().flatten()[0]
 
@@ -249,10 +249,18 @@ class TemporalVariable(SourcedVariable):
             new_bounds, date_parts, repr_dt, dgroups = self._get_grouping_other_(grouping)
 
         new_name = 'climatology_bounds'
+        time_dimension_name = self.dimensions[0].name
         if self.has_bounds:
             new_dimensions = [d.name for d in self.bounds.dimensions]
         else:
-            new_dimensions = [self.name, 'bounds']
+            new_dimensions = [time_dimension_name, 'bounds']
+
+        # Create the new time dimension as unlimited if the original time variable also has an unlimited dimension.
+        if self.dimensions[0].is_unlimited:
+            new_time_dimension = Dimension(name=time_dimension_name, size_current=len(repr_dt))
+        else:
+            new_time_dimension = time_dimension_name
+        new_dimensions[0] = new_time_dimension
 
         new_bounds = TemporalVariable(value=new_bounds, name=new_name, dimensions=new_dimensions)
         new_attrs = deepcopy(self.attrs)
@@ -1008,11 +1016,15 @@ def get_time_regions(seasons, dates, raise_if_incomplete=True):
     # extract the years from the data vector collapsing them to a unique set then sort in ascending order
     years = list(set([d.year for d in dates]))
     years.sort()
-    # determine if any of the seasons are interannual
-    interannual_check = list(map(get_is_interannual, seasons))
     # holds the return value
     time_regions = []
-    # the interannual cases requires two time region sequences to properly extract
+    # the interannual cases requires two time region sequences to properly extract. there must be at least two years to
+    # care about interannual seasons.
+    if len(years) > 1:
+        # determine if any of the seasons are interannual
+        interannual_check = list(map(get_is_interannual, seasons))
+    else:
+        interannual_check = [False]
     if any(interannual_check):
         # loop over years first to ensure each year is accounted for in the time region output
         for ii_year, year in enumerate(years):
@@ -1054,7 +1066,6 @@ def get_time_regions(seasons, dates, raise_if_incomplete=True):
     else:
         for year, season in itertools.product(years, seasons):
             time_regions.append([{'year': [year], 'month': season}])
-
     # ensure each time region is valid. if it is not, remove it from the returned list
     td = TemporalVariable(value=dates, dimensions=constants.DimensionName.TEMPORAL)
     remove = []
@@ -1078,13 +1089,12 @@ def iter_boolean_groups_from_time_regions(time_regions, tvar, yield_subset=False
     >>> [[{'month':[1,2],'year':[2024]},...],...]
 
     :param tvar: A temporal variable object.
-    :type tvar: :class:`ocgis.interface.base.dimension.temporal.TemporalDimension`
+    :type tvar: :class:`ocgis.TemporalVariable`
     :param bool yield_subset: If ``True``, yield a tuple with the subset of ``tvar``.
     :param bool raise_if_incomplete: If ``True``, raise an exception if the season is incomplete.
     :returns: boolean ndarray vector with yld.shape == tvar.shape
     :raises: IncompleteSeasonError
     """
-    # tdk: fix doc for tvar
     for sub_time_regions in time_regions:
         # incomplete seasons are searched for in the nested loop. this indicates if a time region group should be
         # considered a season.
