@@ -1,14 +1,16 @@
 import itertools
+from collections import OrderedDict
 from contextlib import contextmanager
-from functools import reduce
+from functools import reduce, partial
 
 import numpy as np
 import six
 
 from ocgis import constants
 from ocgis.base import AbstractOcgisObject, raise_if_empty
-from ocgis.constants import DataType, MPITag
+from ocgis.constants import DataType, SourceIndexType
 from ocgis.exc import DimensionNotFound
+from ocgis.util.addict import Dict
 from ocgis.util.helpers import get_optimal_slice_from_array, get_iter, get_group
 
 try:
@@ -16,14 +18,21 @@ try:
     from mpi4py.MPI import COMM_NULL
 except ImportError:
     MPI_ENABLED = False
+    MPI_TYPE_MAPPING = None
 else:
     if MPI.COMM_WORLD.Get_size() > 1:
         MPI_ENABLED = True
     else:
         MPI_ENABLED = False
+    MPI_TYPE_MAPPING = {np.int32: MPI.INT, np.int64: MPI.LONG_LONG,
+                        np.dtype('int32'): MPI.INT, np.dtype('float64'): MPI.LONG_LONG}
 
 
 class DummyMPIComm(object):
+
+    def __init__(self):
+        self._send_recv = Dict()
+
     def Barrier(self):
         pass
 
@@ -51,14 +60,38 @@ class DummyMPIComm(object):
     def Incl(self, *args, **kwargs):
         return self
 
-    def recv(self, *args, **kwargs):
-        pass
+    def reduce(self, *args, **kwargs):
+        return args[0]
 
     def scatter(self, *args, **kwargs):
         return args[0][0]
 
+    def Irecv(self, payload, source=0, tag=None):
+        def _irecv_callback_(ipayload, icomm, isource, itag, mode):
+            ret = False
+            if mode == 'test':
+                ret = itag in icomm._send_recv[isource]
+            elif mode == 'get' or ret:
+                stored = icomm._send_recv[isource].pop(itag)
+                ipayload[0] = stored
+                ret = True
+            else:
+                raise NotImplementedError(mode)
+            return ret
+
+        the_callback = partial(_irecv_callback_, payload, self, source, tag)
+
+        return DummyRequest(callback=the_callback)
+
+    def recv(self, *args, **kwargs):
+        return self.Irecv(*args, **kwargs)
+
+    def Isend(self, payload, dest=0, tag=None):
+        self._send_recv[dest][tag] = payload[0]
+        return DummyRequest()
+
     def send(self, *args, **kwargs):
-        pass
+        return self.Isend(*args, **kwargs)
 
 
 if MPI_ENABLED and MPI.COMM_WORLD.Get_size() > 1:
@@ -70,6 +103,28 @@ MPI_SIZE = MPI_COMM.Get_size()
 MPI_RANK = MPI_COMM.Get_rank()
 
 
+class DummyRequest(AbstractOcgisObject):
+
+    def __init__(self, callback=None):
+        self._callback = callback
+
+    def Test(self):
+        if self._callback is None:
+            ret = True
+        else:
+            ret = self._callback('test')
+        return ret
+
+    def wait(self):
+        if self._callback is None:
+            ret = True
+        else:
+            while not self.Test():
+                continue
+            ret = self._callback('get')
+        return ret
+
+
 class OcgDist(AbstractOcgisObject):
     """
     Computes the parallel distribution for variable dimensions.
@@ -91,7 +146,7 @@ class OcgDist(AbstractOcgisObject):
         self.size = size
         self.mapping = {}
         for rank in self.ranks:
-            self.mapping[rank] = get_template_rank_dict()
+            self.mapping[rank] = create_template_rank_dict()
 
         self.has_updated_dimensions = False
 
@@ -111,7 +166,7 @@ class OcgDist(AbstractOcgisObject):
                 msg = 'Dimension with name "{}" already in group "{}" and "force=False".'
                 raise ValueError(msg.format(to_add_dim.name, group))
             else:
-                the_group['dimensions'][to_add_dim.name] = to_add_dim
+                the_group['dimensions'][to_add_dim.source_name] = to_add_dim
 
     def add_dimensions(self, dims, **kwargs):
         for dim in dims:
@@ -130,7 +185,6 @@ class OcgDist(AbstractOcgisObject):
         """
         from ocgis.variable.base import Variable
         from ocgis.variable.dimension import Dimension
-
         if isinstance(name_or_variable, Variable):
             group = group or name_or_variable.group
             name = name_or_variable.name
@@ -187,11 +241,16 @@ class OcgDist(AbstractOcgisObject):
 
     def get_dimension(self, name, group=None, rank=MPI_RANK):
         group_data = self.get_group(group=group, rank=rank)
-        return group_data['dimensions'][name]
+        d = group_data['dimensions']
+        try:
+            ret = d[name]
+        except KeyError:
+            raise DimensionNotFound(name)
+        return ret
 
     def get_dimensions(self, names, **kwargs):
         ret = [self.get_dimension(name, **kwargs) for name in names]
-        return ret
+        return tuple(ret)
 
     def get_empty_ranks(self, group=None, inverse=False):
         ret = []
@@ -280,11 +339,9 @@ class OcgDist(AbstractOcgisObject):
                 # Use this to calculate the local bounds for a dimension.
                 bounds_local = get_rank_bounds(len(distributed_dimension), the_size, rank)
                 if bounds_local is not None:
+                    from ocgis.variable.dimension import slice_source_index
                     start, stop = bounds_local
-                    if distributed_dimension._src_idx is None:
-                        src_idx = None
-                    else:
-                        src_idx = distributed_dimension._src_idx[start:stop]
+                    src_idx = slice_source_index(slice(start, stop), distributed_dimension._src_idx)
                     distributed_dimension.set_size(stop - start, src_idx=src_idx)
                     distributed_dimension.bounds_local = bounds_local
                 else:
@@ -313,40 +370,27 @@ class OcgDist(AbstractOcgisObject):
         return ret
 
 
-def barrier_print(*args):
+def barrier_print(*args, **kwargs):
+    kwargs = kwargs.copy()
+    comm = kwargs.pop('comm', MPI_COMM)
+    assert len(kwargs) == 0
+
+    size = comm.Get_size()
+    rank = comm.Get_rank()
     if len(args) == 1:
         args = args[0]
-    ranks = list(range(MPI_SIZE))
-    MPI_COMM.Barrier()
-    for rank in ranks:
-        if rank == MPI_RANK:
+    ranks = list(range(size))
+    comm.Barrier()
+    for r in ranks:
+        if r == rank:
             print('')
-            print('(rank={}, barrier=True) {}'.format(MPI_RANK, args))
+            print('(rank={}, barrier=True) {}'.format(rank, args))
             print('')
-        MPI_COMM.Barrier()
-
-
-def barrier_ranks(ranks, comm=None):
-    comm, rank, _ = get_standard_comm_state(comm=comm)
-    if len(ranks) > 1:
-        if rank in ranks:
-            root = ranks[0]
-            if rank == root:
-                received = [root]
-                for irank in ranks:
-                    if irank != root:
-                        comm.recv(source=irank, tag=MPITag.BARRIER)
-                        received.append(irank)
-                for irank in ranks:
-                    if irank != root:
-                        comm.send(True, dest=irank, tag=MPITag.BARRIER)
-            else:
-                comm.send(True, dest=root, tag=MPITag.BARRIER)
-                comm.recv(source=root, tag=MPITag.BARRIER)
+        comm.Barrier()
 
 
 def create_slices(length, size):
-    # tdk: optimize: remove np.arange
+    # TODO: Optimize by removing numpy.arange
     r = np.arange(length)
     sections = np.array_split(r, size)
     sections = [get_optimal_slice_from_array(s, check_diff=False) for s in sections]
@@ -406,30 +450,6 @@ def dict_get_or_create(ddict, key, default):
     return ret
 
 
-def gather_ranks(ranks, value, root=0, comm=None):
-    comm, rank, size = get_standard_comm_state(comm=comm)
-
-    if rank == root:
-        gathered = [None] * len(ranks)
-    else:
-        gathered = None
-
-    for idx, curr_rank in enumerate(ranks):
-        if rank == root:
-            if rank == curr_rank:
-                gathered[idx] = value
-        elif rank == curr_rank:
-            comm.send(value, dest=root, tag=MPITag.GATHER)
-
-    if rank == root:
-        for idx, recv_rank in enumerate(ranks):
-            if recv_rank != root:
-                recv = comm.recv(source=recv_rank, tag=MPITag.GATHER)
-                gathered[idx] = recv
-
-    return gathered
-
-
 def get_rank_bounds(nelements, size, rank, esplit=None):
     """
     :param nelements: The number of elements in the sequence to split.
@@ -519,38 +539,6 @@ def mpi_group_scope(ranks, comm=None):
             sub_group.Free()
 
 
-def bcast_ranks(ranks, value, root=0, comm=None):
-    comm, rank, size = get_standard_comm_state(comm=comm)
-
-    if rank == root:
-        for target_rank in ranks:
-            if target_rank != root:
-                comm.send(value, dest=target_rank, tag=MPITag.BCAST)
-    elif rank in ranks:
-        value = comm.recv(source=root, tag=MPITag.BCAST)
-    else:
-        value = None
-
-    return value
-
-
-def scatter_ranks(ranks, value, root=0, comm=None):
-    comm, rank, size = get_standard_comm_state(comm=comm)
-
-    if rank == root:
-        for target_rank, target_value in zip(ranks, value):
-            if target_rank != root:
-                comm.send(target_value, dest=target_rank, tag=MPITag.SCATTER)
-            else:
-                ret_value = target_value
-    elif rank in ranks:
-        ret_value = comm.recv(source=root, tag=MPITag.SCATTER)
-    else:
-        ret_value = None
-
-    return ret_value
-
-
 def ogather(elements):
     ret = np.array(elements, dtype=object)
     return ret
@@ -578,6 +566,10 @@ def create_nd_slices(splits, shape):
     return tuple(ret)
 
 
+def create_template_rank_dict():
+    return {None: {'dimensions': {}, 'groups': {}, 'variables': {}}}
+
+
 def find_dimension_in_sequence(dimension_name, dimensions):
     ret = None
     for dim in dimensions:
@@ -630,10 +622,6 @@ def get_standard_comm_state(comm=None):
     return comm, comm.Get_rank(), comm.Get_size()
 
 
-def get_template_rank_dict():
-    return {None: {'dimensions': {}, 'groups': {}, 'variables': {}}}
-
-
 def rank_print(*args):
     if len(args) == 1:
         args = args[0]
@@ -641,6 +629,127 @@ def rank_print(*args):
     print(msg)
 
 
+def redistribute_by_src_idx(variable, dimname, dimension):
+    """
+    Redistribute values in ``variable`` using the source index associated with ``dimension``. The reloads the data from
+    source and does not do an in-memory redistribution using MPI.
+
+    This function is collective across the current `~ocgis.OcgVM`.
+
+    * Uses fancy indexing only.
+    * Gathers all source indices to a single processor.
+
+    :param variable: The variable to redistribute.
+    :type variable: :class:`~ocgis.Variable`
+    :param str dimname: The name of the dimension holding the source indices.
+    :param dimension: The dimension object.
+    :type dimension: :class:`~ocgis.Dimension`
+    """
+    from ocgis import SourcedVariable, Variable, vm
+    from ocgis.variable.dimension import create_src_idx
+
+    assert isinstance(variable, SourcedVariable)
+    assert dimname is not None
+
+    # If this is a serial operation just return. The rank should be fully autonomous in terms of its source information.
+    if vm.size == 1:
+        return
+
+    # There needs to be at least one rank to redistribute.
+    live_ranks = vm.get_live_ranks_from_object(variable)
+    if len(live_ranks) == 0:
+        raise ValueError('There must be at least one rank to redistribute by source index.')
+
+    # Remove relevant values from a variable.
+    def _reset_variable_(target):
+        target._is_empty = None
+        target._mask = None
+        target._value = None
+        target._has_initialized_value = False
+
+    # Gather the sliced dimensions. This dimension hold the source indices that are redistributed.
+    dims_global = vm.gather(dimension)
+
+    if vm.rank == 0:
+        # Filter any none-type dimensions to handle currently empty ranks.
+        dims_global = [d for d in dims_global if d is not None]
+        # Convert any bounds-type source indices to fancy type.
+        # TODO: Support bounds-type source indices.
+        for d in dims_global:
+            if d._src_idx_type == SourceIndexType.BOUNDS:
+                d._src_idx = create_src_idx(*d._src_idx, si_type=SourceIndexType.FANCY)
+        # Create variable to scatter that holds the new global source indices.
+        global_src_idx = hgather([d._src_idx for d in dims_global])
+        global_src_idx = Variable(name='global_src_idx', value=global_src_idx, dimensions=dimname)
+        # The new size is also needed to create a regular distribution for the variable scatter.
+        global_src_idx_size = global_src_idx.size
+    else:
+        global_src_idx, global_src_idx_size = [None] * 2
+
+    # Build the new distribution based on the gathered source indices.
+    global_src_idx_size = vm.bcast(global_src_idx_size)
+    dest_dist = OcgDist()
+    new_dim = dest_dist.create_dimension(dimname, global_src_idx_size, dist=True)
+    dest_dist.update_dimension_bounds()
+
+    # This variable holds the new source indices.
+    new_rank_src_idx = variable_scatter(global_src_idx, dest_dist)
+
+    if new_rank_src_idx.is_empty:
+        # Support new empty ranks following the scatter.
+        variable.convert_to_empty()
+    else:
+        # Reset the variable so everything can be loaded from source.
+        _reset_variable_(variable)
+        # Update the source index on the target dimension.
+        new_dim._src_idx = new_rank_src_idx.get_value()
+        # Add the dimension with the new source index to the collection.
+        variable.parent.dimensions[dimname] = new_dim
+
+    # All emptiness should be pushed back to the dimensions.
+    variable.parent._is_empty = None
+    for var in variable.parent.values():
+        var._is_empty = None
+
+    # Any variables that have a shared dimension should also be reset.
+    for var in variable.parent.values():
+        if dimname in var.dimension_names:
+            if new_rank_src_idx.is_empty:
+                var.convert_to_empty()
+            else:
+                _reset_variable_(var)
+
+
+def variable_collection_scatter(variable_collection, dest_dist):
+    from ocgis import vm
+
+    if vm.rank == 0:
+        for v in variable_collection.values():
+            if v.dist:
+                msg = "Only variables with no prior distribution may be scattered. '{}' is distributed."
+                raise ValueError(msg.format(v.name))
+        target = variable_collection.first()
+    else:
+        target = None
+    sv = variable_scatter(target, dest_dist)
+    svc = sv.parent
+
+    if vm.rank == 0:
+        n_children = len(variable_collection.children)
+    else:
+        n_children = None
+    n_children = vm.bcast(n_children)
+    if vm.rank == 0:
+        children = list(variable_collection.children.values())
+    else:
+        children = [None] * n_children
+    for child in children:
+        scattered_child = variable_collection_scatter(child, dest_dist)[0]
+        svc.add_child(scattered_child, force=True)
+
+    return svc
+
+
 def variable_gather(variable, root=0):
     from ocgis import vm
 
@@ -651,9 +760,9 @@ def variable_gather(variable, root=0):
         if vm.rank == root:
             new_variable = variable.copy()
             new_variable.dtype = variable.dtype
-            new_variable.set_mask(None)
-            new_variable.set_value(None)
-            new_variable.set_dimensions(None)
+            new_variable._mask = None
+            new_variable._value = None
+            new_variable._dimensions = None
             assert not new_variable.has_allocated_value
         else:
             new_variable = None
@@ -680,9 +789,16 @@ def variable_gather(variable, root=0):
                         has_src_idx = part._src_idx is not None
                         new_size += len(part)
                     if has_src_idx:
-                        new_src_idx = np.zeros(new_size, dtype=DataType.DIMENSION_SRC_INDEX)
-                        for part in parts:
-                            new_src_idx[part.bounds_local[0]: part.bounds_local[1]] = part._src_idx
+                        if part._src_idx_type == SourceIndexType.FANCY:
+                            new_src_idx = np.zeros(new_size, dtype=DataType.DIMENSION_SRC_INDEX)
+                            for part in parts:
+                                new_src_idx[part.bounds_local[0]: part.bounds_local[1]] = part._src_idx
+                        else:
+                            part_bounds = [None] * len(parts)
+                            for idx2, part in enumerate(parts):
+                                part_bounds[idx2] = part._src_idx
+                            part_bounds = np.array(part_bounds)
+                            new_src_idx = (part_bounds.min(), part_bounds.max())
                     else:
                         new_src_idx = None
                     new_dim = dim.copy()
@@ -706,7 +822,7 @@ def variable_gather(variable, root=0):
         return new_variable
 
 
-def variable_scatter(variable, dest_dist, root=0):
+def variable_scatter(variable, dest_dist, root=0, strict=False):
     from ocgis import vm
 
     if variable is not None:
@@ -718,26 +834,55 @@ def variable_scatter(variable, dest_dist, root=0):
         if not dest_dist.has_updated_dimensions:
             raise ValueError('The destination distribution must have updated dimensions.')
 
-    # Synchronize distribution across processors.
-    dest_dist = vm.bcast(dest_dist)
-
     # Find the appropriate group for the dimensions.
     if vm.rank == root:
         group = variable.group
-        dimension_names = [dim.name for dim in variable.dimensions]
+        # dimension_names = [dim.name for dim in variable.dimensions]
+        dimension_names = variable.parent.dimensions.keys()
     else:
         group = None
         dimension_names = None
 
+    # Depending on the strictness level, not all dimensions may be present in the distribution. This is allowed for
+    # processes to more flexibly add undistributed dimensions. Distributed dimensions should be part of the destination
+    # distribution already.
+    not_in_dist = {}
+    if vm.rank == 0:
+        for dest_dimension_name in dimension_names:
+            try:
+                _ = dest_dist.get_dimension(dest_dimension_name, group=group)
+            except DimensionNotFound:
+                if strict:
+                    raise
+                else:
+                    not_in_dist[dest_dimension_name] = variable.parent.dimensions[dest_dimension_name]
+    not_in_dist = vm.bcast(not_in_dist)
+
     # Synchronize the processes with the MPI distribution and the group containing the dimensions.
-    dest_dist = vm.bcast(dest_dist)
-    group = vm.bcast(group)
-    dimension_names = vm.bcast(dimension_names)
+    dest_dist = vm.bcast(dest_dist, root=root)
+    group = vm.bcast(group, root=root)
+    # Need to convert the object to a list to be compatible with Python 3.
+    if dimension_names is not None:
+        dimension_names = list(dimension_names)
+    dimension_names = vm.bcast(dimension_names, root=root)
 
     # These are the dimensions for the local process.
-    dest_dimensions = dest_dist.get_dimensions(dimension_names, group=group)
+    dest_dimensions = [None] * len(dimension_names)
+    for ii, dest_dimension_name in enumerate(dimension_names):
+        try:
+            d = dest_dist.get_dimension(dest_dimension_name, group=group)
+        except DimensionNotFound:
+            if strict:
+                raise
+            else:
+                # Dimensions not in the distribution should have been received from the root process.
+                d = not_in_dist[dest_dimension_name]
+        dest_dimensions[ii] = d
 
-    # barrier_print('dest_dimensions bounds_global: ', [(d.name, d.bounds_global) for d in dest_dimensions])
+    # Populate the local destination dimensions dictionary.
+    dd_dict = OrderedDict()
+    for d in dest_dimensions:
+        dd_dict[d.name] = d
 
     # Slice the variables collecting the sequence to scatter to the MPI procs.
     if vm.rank == root:
@@ -770,39 +915,13 @@ def variable_scatter(variable, dest_dist, root=0):
 
     # Scatter the variable across processes.
     scattered_variable = vm.scatter(variables_to_scatter, root=root)
-    # Update the scattered variable dimensions with the destination dimensions on the process. Everything should align
-    # shape-wise. If they don't, an exception will be raised.
-    scattered_variable.set_dimensions(dest_dimensions, force=True)
+    # Update the scattered variable collection dimensions with the destination dimensions on the process. Everything
+    # should align shape-wise.
+    scattered_variable.parent._dimensions = dd_dict
 
     return scattered_variable
 
 
-def variable_collection_scatter(variable_collection, dest_dist):
-    from ocgis import vm
-
-    if vm.rank == 0:
-        target = variable_collection.first()
-    else:
-        target = None
-    sv = variable_scatter(target, dest_dist)
-    svc = sv.parent
-
-    if vm.rank == 0:
-        n_children = len(variable_collection.children)
-    else:
-        n_children = None
-    n_children = vm.bcast(n_children)
-    if vm.rank == 0:
-        children = list(variable_collection.children.values())
-    else:
-        children = [None] * n_children
-    for child in children:
-        scattered_child = variable_collection_scatter(child, dest_dist)[0]
-        svc.add_child(scattered_child, force=True)
-
-    return svc
-
-
 def vgather(elements):
     n = sum([e.shape[0] for e in elements])
     fill = np.zeros((n, elements[0].shape[1]), dtype=elements[0].dtype)
@@ -815,3 +934,12 @@ def vgather(elements):
         fill[start:stop, :] = e
         start = stop
     return fill
+
+
+def cancel_free_requests(request_sequence):
+    for r in request_sequence:
+        try:
+            r.Cancel()
+            r.Free()
+        except:
+            pass
